package actions;

import ihm.Gestionnaire;
import outils.Constantes;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.Timer;

import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.entity.ChartEntity;
import org.jfree.chart.entity.XYItemEntity;

import don.AxePartiel;
import don.DonneeID;
import don.Qualifiable;


/** ActionListener du ChartPanel, responsable des doubles clics sur les points
 * 
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class PanelMouseActionListener implements ChartMouseListener, ActionListener
{
	/** rayon de détection du clic */
	private static final int m_marge=outils.Constantes.FORME_INDIVIDUS.getBounds().width+4;

	/** le gestionnaire */
	private Gestionnaire m_gest;

	/** fenêtre JFreeChart */
	private ChartPanel m_panel;

	private Timer timer;

	private Qualifiable m_qualifiableSousSouris;

	private Object m_habillage;

	/** Constructeur
	 * @param gestionnaire le gestionnaire auquel on enverra les ordres
	 * @param panel le panel sur lequel on travaille
	 */
	public PanelMouseActionListener(Gestionnaire gestionnaire, ChartPanel panel)
	{
		m_gest=gestionnaire;
		m_panel=panel;
		timer=new Timer(Constantes.TEMPS_AVANT_DESCRIPTION_PAR_QUALIFIABLE,this);
	}

	/** action */
	public void chartMouseClicked(ChartMouseEvent event)
	{
		if(event.getTrigger().getClickCount()>1)
		{
			timer.stop();
			Vector<DonneeID> res=getDonneeIDs(event);
			if(res.size()>0){
				m_gest.setAnnotations(res,!m_gest.isAnnotations(res));
			}
		}else{
			Qualifiable qualif = getQualifiable(event);
			//mettreAJourHabillage(qualif);
			demarrerAttente(qualif);			
		}
	}

	private Qualifiable getQualifiable(ChartMouseEvent event) {
		Qualifiable qualif=null;
		Rectangle2D rect = getRect(event);
		if(m_gest.getCanvas().contientX(m_panel, rect, 0)
				//si la droite x=0 (axe Y) passe par le rectangle de clic
				|| m_gest.getCanvas().bordVertical(m_panel, rect)
				//si un des bords verticaux passe par le rectangle de clic
				){
			qualif=m_gest.getAxeY();
		}else if(m_gest.getCanvas().contientY(m_panel, rect, 0)
				//si la droite y=0 (axe X) passe par le rectangle de clic
				|| m_gest.getCanvas().bordHorizontal(m_panel, rect)
				//si un des bords horizontaux passe par le rectangle de clic
				){
			qualif=m_gest.getAxeX();		
		}else{
			Vector<DonneeID> liste = m_gest.getCanvas().findPoint(m_panel,rect);
			for (DonneeID donneeID : liste) {
				if (donneeID instanceof Qualifiable && !(donneeID instanceof AxePartiel)) {
					qualif = (Qualifiable) donneeID;						
				}
			}
		}
		return qualif;
	}

	private Rectangle2D getRectConfiance(ChartMouseEvent event) {
		Point2D point=m_panel.translateScreenToJava2D(event.getTrigger().getPoint());
		int x=(int)point.getX();
		int y=(int)point.getY();			
		Rectangle2D rect= new Rectangle2D.Double(x-m_marge,y-m_marge,m_marge*2,m_marge*2);
		return rect;
	}

	private Vector<DonneeID> getDonneeIDs(ChartMouseEvent event) {
		Vector<DonneeID> res=new Vector<DonneeID>();
		Rectangle2D rect = getRect(event);
		if(rect!=null)
		{
			res=m_gest.getCanvas().findPoint(m_panel,rect);			
		}
		return res;
	}

	private Rectangle2D getRect(ChartMouseEvent event) {
		ChartEntity chartEntity=event.getEntity();
		Rectangle2D rect=null;
		//cas ou le ChartPanel est capable de répondre				
		if(chartEntity instanceof XYItemEntity){
			XYItemEntity entity = (XYItemEntity) chartEntity;
			//On cherche à calculer la zone dans laquelle chercher des points
			String coord=entity.getShapeCoords();
			StringTokenizer st=new StringTokenizer(coord,",");

			int minx=Integer.MAX_VALUE;
			int maxx=Integer.MIN_VALUE;
			int miny=Integer.MAX_VALUE;
			int maxy=Integer.MIN_VALUE;

			boolean coordX=true;
			while(st.hasMoreTokens())
			{
				int v=Integer.valueOf(st.nextToken());
				if(coordX)
				{
					if(v>maxx)maxx=v;
					if(v<minx)minx=v;
				}
				else
				{
					if(v>maxy)maxy=v;
					if(v<miny)miny=v;
				}
				coordX=!coordX;
			}
			rect= new Rectangle2D.Double(minx-m_marge,miny-m_marge,maxx-minx+m_marge*2,maxy-miny+m_marge*2);
		}
		else
		{
			rect= getRectConfiance(event);
		}
		return rect;
	}

	public void chartMouseMoved(ChartMouseEvent event)
	{
		/*
		Qualifiable quali=getQualifiable(event);
		demarrerAttente(quali);
		*/
	}

	private void demarrerAttente(Qualifiable quali) {
		if(quali!=null){
			if(m_qualifiableSousSouris==null || !m_qualifiableSousSouris.equals(quali)){
				timer.restart();
				m_qualifiableSousSouris=quali;
			}
		}
		else{
			if(m_qualifiableSousSouris!=null){
				m_qualifiableSousSouris=null;
				mettreAJourHabillage(null);
				timer.stop();
			}
		}
	}

	public void actionPerformed(ActionEvent arg0) {
		timer.stop();
		mettreAJourHabillage(m_qualifiableSousSouris);		
	}

	private void mettreAJourHabillage(Qualifiable don){
		if(!(don==null && m_habillage==null) && don != m_habillage){
			//On ne fait quelque chose que si il y a changement d'état
			m_habillage=don;
			m_gest.setDescription(don);
			m_gest.sauverAnnotations();
			m_gest.updateGraphe();
		}
	}



}
