package criteres;

import ihm.IHM;
import don.DonneeID;
import don.Groupe;
import don.Individu;
import don.Modalite;
import don.Variable;

/** Classe les objets selon qu'ils sont actifs ou illustratifs
 * 
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class CritereActifIllustratif extends Qualificatif
{
	/** code d'identification des actifs */
	private static final int ACTIF=0;
	
	/** code d'identification des illustratifs */
	private static final int ILLUSTRATIF=1;

	/**@see criteres.Critere#evaluer(DonneeID)*/
	public Info evaluer(DonneeID obj)
	{
		if(obj.isActive())return new Singleton(ACTIF);
		else return new Singleton(ILLUSTRATIF);
	}

	/**@see criteres.Critere#semantique(Singleton)*/
	public String semantique(Singleton i)
	{
		if(i.get()==ACTIF)return IHM.getXmlLangue().getElement("CactifMF");
		else if(i.get()==ILLUSTRATIF) return IHM.getXmlLangue().getElement("CillusMF");
		else return CHAINE_ERREUR+" valeur inappropriée pour Actif/Illustratif";
	}

	/**@see criteres.Critere#semantique()*/
	public String semantique()
	{return IHM.getXmlLangue().getElement("CillusMF")+IHM.getXmlLangue().getElement("ou")+IHM.getXmlLangue().getElement("CactifMF");}

	/**@see criteres.Critere#getNbSeries()*/
	public int getNbSeries()
	{return 2;}

	/**@see criteres.Critere#qualifier(DonneeID)
	 * @param obj
	 * @return qualifie l'objet (par un texte)
	 */
	public String qualifier(DonneeID obj)
	{
		if(obj  instanceof Individu) {
			if(obj.isActive()) return IHM.getXmlLangue().getElement("CactifM");
			else return IHM.getXmlLangue().getElement("CillusM");
		}else if (obj instanceof Variable) {
			if(obj.isActive()) return IHM.getXmlLangue().getElement("CactifF");
			else return IHM.getXmlLangue().getElement("CillusF");
		}else if (obj instanceof Groupe) {
			if(obj.isActive()) return IHM.getXmlLangue().getElement("CactifM");
			else return IHM.getXmlLangue().getElement("CillusM");
		}else if(obj instanceof Modalite) {
			if(obj.isActive()) return IHM.getXmlLangue().getElement("CactifF");
			else return IHM.getXmlLangue().getElement("CillusF");
		}else return CHAINE_ERREUR+" objet de type inconnu pour Actif/Illustratif";
	}

	/**@see criteres.Critere#adjectif()*/
	public boolean adjectif()
	{return true;}
}
