package criteres;

import don.DonneeID;

/** Critère qui ajoute un critère
 * utilisé pour gérer la sélection
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class CritereAjout extends CritereCompose
{
	/** constructeur à partir de deux criteres,
	 * ajoute critere2 à critere1
	 * 
	 * @param critere1
	 * @param critere2
	 */
	public CritereAjout(Critere critere1, Critere critere2)
	{super(critere1, critere2);}

	/** ajoute un crit�re au critere1
	 * @param critere
	 */
	public void addCritere(Critere critere)
	{	
		if(m_critere1==null)m_critere1=critere;
		else m_critere1=new CritereCompose(m_critere1,critere);
	}

	/** On effectue le OU (addition) des sous-crit�res, le critere1 étant prioritaire
	 * @param obj 
	 * @return le OU des sous-crit�res
	 */
	public Info evaluer(DonneeID obj)
	{
		Info val1=m_critere1.evaluer(obj);
		Info val2=m_critere2.evaluer(obj);

		if(val1.isError() && val2.isError()) return new Singleton(Critere.CODE_ERROR);
		else if(val2.isError()) return new Paire(val1,new Singleton(Critere.CODE_ERROR));
		else return new Paire(new Singleton(Critere.CODE_ERROR),val2);
	}

	/**@see criteres.Critere#getNbSeries()*/
	public int getNbSeries()
	{
		int nb=0;
		if(m_critere1!=null)nb+=m_critere1.getNbSeries();
		if(m_critere2!=null)nb+=m_critere2.getNbSeries();
		return nb;
	}

	/** @see criteres.Critere#semantique()*/
	public String semantique()
	{return "("+m_critere1.semantique()+" + "+m_critere2.semantique()+")";}

	
	/**@see criteres.Critere#semantique(Paire)*/
	public String semantique(Paire i)
	{
		Info g = i.getG();
		Info d =i.getD();
		if(g.isError() && d.isError()) return CHAINE_ERREUR+" ajout (paire d'erreurs)";
		else if(g.isError())return m_critere2.semantique(d);
		else if(d.isError()) return m_critere1.semantique(g);
		else return CHAINE_ERREUR+" ajout (paire sans erreur)";
	}

	/**@see criteres.Critere#semantique(Singleton)*/
	public String semantique(Singleton i)
	{return CHAINE_ERREUR+" ajout (singleton)";}

	/**@see criteres.Critere#semantique(Info)*/
	public String semantique(Info i)
	{
		if (i instanceof Paire) {
			return semantique((Paire)i);	    
		}else if(i instanceof Singleton) {
			return semantique((Singleton)i);
		}else return CHAINE_ERREUR+" ajout (type impossible)";
	}

//	/**
//	 * main
//	 * @param args
//	 */
//	public static void main(String[] args)
//	{
//		IHM.m_dir=new File("").getAbsolutePath()+"\\";
//		Individu ind = new Individu("ind",false,null,null,null,null,null);
//		ind.setGeneration(3);
//		CritereTypeDeDonnee c3 = new CritereTypeDeDonnee();
//		System.out.println(c3.qualifier(ind));
//		Critere c1= new CritereActif();
//		System.out.println(c1.qualifier(ind));
//		CritereCompose crit=new CritereAjout(c3,c1);
//		System.out.println(crit.qualifier(ind));
//		System.out.println("-----------------");
//		Individu ind2 = new Individu("ind2",true,null,null,null,null,null);
//		ind.setGeneration(3);
//		System.out.println(c3.qualifier(ind2));
//		System.out.println(c1.qualifier(ind2));
//		System.out.println(crit.qualifier(ind2));
//
//		/*
//        individual
//        erreur
//        individual
//        -----------------
//        individual
//        active
//        active
//		 */
//	}
}
