package criteres;

import ihm.IHM;
import don.DonneeID;

/** Composition (multiplication) de 2 criteres
 * 
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
//TODO faire passer cette classe en abstraite et ne lui laisser que le comportement à deux critères, et en faire hériter une classe "CritereComposition/Union"
public class CritereCompose extends Qualificatif 
{
	/** 1er critere */
	protected Critere m_critere1;

	/** second critere */
	protected Critere m_critere2;

	/** Constructeur par defaut*/
	public CritereCompose()
	{
		m_critere1=null;
		m_critere2=null;
	}

	/** constructeur à partir de deux criteres
	 * 
	 * @param critere1
	 * @param critere2
	 */
	public CritereCompose(Critere critere1, Critere critere2)
	{
		m_critere1=critere1;
		m_critere2=critere2;
	}

	/** ajoute un critere
	 * @param critere
	 */
	public void addCritere(Critere critere)
	{	
		if(m_critere1==null)m_critere1=critere;
		else if(m_critere2==null) m_critere2=critere;
		else m_critere2=new CritereCompose(critere,m_critere2);
	}

	/** @return le 1er critère */
	public Critere getCritere1()
	{return m_critere1;}

	/** @return le 2e critère */
	public Critere getCritere2()
	{return m_critere2;}

	/** On effectue le OU (multiplication) des sous-critères
	 * @param obj 
	 * @return le OU des sous-criteres
	 */
	public Info evaluer(DonneeID obj)
	{
		Info val1=m_critere1.evaluer(obj);
		if(m_critere2!=null) {
			Info val2=m_critere2.evaluer(obj);

			if(val1.isError() && val2.isError()) return new Singleton(Critere.CODE_ERROR);
			else
			{
				return new Paire(val1,val2);
			}
		}else return val1;
	}

	/** @see criteres.Critere#semantique()*/
	public String semantique()
	{	if(m_critere1!=null && m_critere2!=null)
		return "("+m_critere1.semantique()+" "+IHM.getXmlLangue().getElement("ou")+" "+m_critere2.semantique()+")";
	else
		if(m_critere1!=null)return m_critere1.semantique();
		else return m_critere2.semantique();
	}

		/**@see criteres.Critere#getNbSeries()*/
		public int getNbSeries()
	{
		int nb=1;
		if(m_critere1!=null)nb*=m_critere1.getNbSeries();
		if(m_critere2!=null)nb*=m_critere2.getNbSeries();
		return nb;
	}

	/** @see criteres.Critere#semantique(Paire)*/
	public String semantique(Paire i)
	{
		String chaineA;
		String chaineB;
		if(IHM.getXmlLangue().adjectifsAvant())
		{
			if(m_critere1.adjectif()) {
				chaineA=m_critere1.semantique(i.getG());
				chaineB=m_critere2.semantique(i.getD());
			}else {
				chaineA=m_critere2.semantique(i.getD());
				chaineB=m_critere1.semantique(i.getG());		
			}

		}
		else
		{
			if(!(m_critere1.adjectif())) {
				chaineA=m_critere1.semantique(i.getG());
				chaineB=m_critere2.semantique(i.getD());
			}else {
				chaineA=m_critere2.semantique(i.getD());
				chaineB=m_critere1.semantique(i.getG());		
			}	    
		}
		if(chaineB.equals(""))	return chaineA;
		else if(chaineA.equals("")) return chaineB;
		else return chaineA+" "+chaineB;
	}

	/** @see criteres.Critere#semantique(Singleton)*/
	public String semantique(Singleton i)
	{
		if(m_critere2==null && m_critere1!=null) return m_critere1.semantique(i);
		else if(m_critere1==null && m_critere2!=null) return m_critere2.semantique(i);
		else return CHAINE_ERREUR+" singleton au lieu de paire pour composé";
	}

	/** @see criteres.Critere#semantique(Info)*/
	public String semantique(Info i)
	{
		if(m_critere2 != null && i instanceof Paire)
			return semantique((Paire)i);
		else
			return m_critere1.semantique(i);
	}

	/** @see criteres.Critere#adjectif()*/
	public boolean adjectif()
	{
		if(m_critere1 !=null && m_critere2 != null)return m_critere1.adjectif() && m_critere2.adjectif();
		else if(m_critere1 != null) return m_critere1.adjectif();
		else if(m_critere2 != null) return m_critere2.adjectif();
		return false;
	}

//	/** main
//	 * @param args
//	 */
//	public static void main(String[] args)
//	{
//		Individu ind = new Individu("ind",true,null,null,null,null,null);
//		ind.setGeneration(3);
//		CritereCompose crit=new CritereCompose();
//		CritereTypeDeDonnee c3 = new CritereTypeDeDonnee();
//		crit.addCritere(c3);
//		System.out.println(c3.evaluer(ind));
//		CritereActifIllustratif c1= new CritereActifIllustratif();
//		crit.addCritere(c1);
//		System.out.println(c1.evaluer(ind));
//		CritereGeneration c2= new CritereGeneration();
//		crit.addCritere(c2);
//		System.out.println(c2.evaluer(ind));
//		System.out.println(crit.evaluer(ind));
//		System.out.println(crit.qualifier(ind));
//
//		/*
//	  	  2
//          0
//          3
//          43
//          individu actif de generation 3
//		 */
//	}

	@Override
	public int compare(Paire i0, Paire i1) {
		int res1=m_critere1.compare(i0.getG(), i1.getG());
		if(res1!=0 || m_critere2==null)return res1;
		else{
			int res2=m_critere2.compare(i0.getD(), i1.getD());
			return res2;
		}
	}
}
