package criteres;

import ihm.IHM;
import don.DonneeID;

/**
 *  @author julien.durand@gmail.com
 *  Critère composé qui élimine les données qui ne vérifient pas les deux critères
 *
 */
public class CritereRestriction extends CritereCompose {

	/**
	 * 
	 */
	public CritereRestriction() {
	}

	/**
	 * Critère composé qui élimine les données qui ne vérifient pas les deux critères
	 * 
	 * @param critere1
	 * @param critere2
	 */
	public CritereRestriction(Critere critere1, Critere critere2) {
		super(critere1, critere2);
	}

	/** On effectue le ET (multiplication) des sous-critères
	 * @param obj 
	 * @return le ET des sous-criteres
	 */
	public Info evaluer(DonneeID obj)
	{
		Info val1=m_critere1.evaluer(obj);
		if(m_critere2!=null) {
			Info val2=m_critere2.evaluer(obj);

			if(val1.isError() || val2.isError()) return new Singleton(Critere.CODE_ERROR);
			else
			{
				return new Paire(val1,val2);
			}
		}else return val1;
	}

	/** @see criteres.Critere#semantique()*/
	public String semantique()
	{return "("+m_critere1.semantique()+" "+IHM.getXmlLangue().getElement("et")+" "+m_critere2.semantique()+")";}


	/** @see criteres.Critere#semantique(Paire)*/
	public String semantique(Paire i)
	{
		String chaineA;
		String chaineB;
		if(i.getD().isError()) return m_critere1.semantique(i.getG());
		else if(i.getG().isError()) return m_critere2.semantique(i.getD());
		else{
			if(IHM.getXmlLangue().adjectifsAvant())
			{
				if(m_critere1.adjectif()) {
					chaineA=m_critere1.semantique(i.getG());
					chaineB=m_critere2.semantique(i.getD());
				}else {
					chaineA=m_critere2.semantique(i.getD());
					chaineB=m_critere1.semantique(i.getG());		
				}

			}
			else
			{
				if(!(m_critere1.adjectif())) {
					chaineA=m_critere1.semantique(i.getG());
					chaineB=m_critere2.semantique(i.getD());
				}else {
					chaineA=m_critere2.semantique(i.getD());
					chaineB=m_critere1.semantique(i.getG());		
				}	    
			}
			if(!chaineB.equals(""))	return chaineA+" "+chaineB;
			else return chaineA;
		}
	}


}
