package criteres;

import ihm.IHM;
import don.ContinueAnalysee;
import don.DonneeID;
import don.Groupe;
import don.Individu;
import don.Modalite;

/** Classifie selon le type de donnee : individu, variable, groupe, ...
 * 
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class CritereTypeDeDonnee extends  Qualificatif {

	/** donnee de type : Continue */
	public static final int CONTINUE=0;
	
	/** donnee de type : Groupe */
	public static final int GROUPE=1;
	
	/** donnee de type : Individu */
	public static final int INDIVIDU=3;
	
	/** donnee de type : Modalite */
	public static final int MODALITE=2;

	/**@see criteres.Critere#semantique(Singleton)*/
	public String semantique(Singleton i)
	{
		switch (i.get()) {
		case CONTINUE:
			return IHM.getXmlLangue().getElement("Ccontinue");
		case GROUPE:
			return IHM.getXmlLangue().getElement("Cgroupe");
		case INDIVIDU:
			return IHM.getXmlLangue().getElement("Cindividu");
		case MODALITE:
			return IHM.getXmlLangue().getElement("Cmodalite");
		default:
			return CHAINE_ERREUR+" non singleton pour type de données";
		}
	}

	/**@see criteres.Critere#evaluer(DonneeID)*/
	public Info evaluer(DonneeID obj)
	{
		if(obj instanceof ContinueAnalysee) return new Singleton(CONTINUE);
		else if(obj instanceof Groupe) return new Singleton(GROUPE);
		else if(obj instanceof Individu) return new Singleton(INDIVIDU);
		else if(obj instanceof Modalite) return new Singleton(MODALITE);
		else return new Singleton(CODE_ERROR);
	}

	/**@see criteres.Critere#getNbSeries()*/
	public int getNbSeries()
	{return 4;}

	/**@see criteres.Critere#semantique()*/
	public String semantique()
	{return IHM.getXmlLangue().getElement("Cdata");}

	/**@see criteres.Critere#adjectif()*/
	public boolean adjectif()
	{return false;}
}
