package criteres;

import java.util.Iterator;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Vector;

import don.DonneeID;
import drawing.Series;


/** Classe qui effectue des tris à partir d'un critere
 * 
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class Trieur
{
	/** critère de sélection des donnees */
	private Critere m_critere;

	/** objet qui stocke les séries */
	private SortedSet<Series> m_mesSeries;

	/** Constructeur
	 * @param critere
	 */
	public Trieur(Critere critere)
	{
		m_critere=critere;
		m_mesSeries=new TreeSet<Series>(new Comparateur(m_critere));
	}

	/** @return le critère */
	public Critere getCritere(){return m_critere;}

	/**  effectue un tri de la liste selon le critere
	 * @param liste
	 */
	public void trier(Vector<DonneeID> liste)
	{
		Info i=new Singleton(Critere.CODE_ERROR);
		for (DonneeID don : liste) {
			i=m_critere.evaluer(don);
			if(!i.isError())addMesSeries(m_mesSeries,i,don,m_critere);
		}
	}

	/** ajoute la donnée à la série d'indice "index"
	 * @param mesSeries 
	 * @param i
	 * @param don
	 */
	private void addMesSeries(Set<Series> mesSeries, Info i,DonneeID don, Critere crit)
	{
		if(don!=null){
			Iterator<Series> iter=mesSeries.iterator();
			boolean trouve=false;
			while(iter.hasNext() && !trouve){
				Series courante=iter.next();
				if(courante.getDefinition().equals(i)){
					courante.addDonneeID(don);
					trouve=true;
				}
			}
			if(!trouve){
				Series nouvelle=new Series(i,crit.semantique(i));
				nouvelle.addDonneeID(don);
				mesSeries.add(nouvelle);
			}
		}
	}


	/** Effectue un tri supplémentaire selon le critère donné
	 * @param selection
	 * @return le résultat du tri
	 */
	@Deprecated
	public SortedSet<Series> getSeries(Critere selection)
	{
		TreeSet<Series> mesSeries=new TreeSet<Series>(new Comparateur(m_critere));
		Info n;
		//boucle qui parcour toutes les s�ries
		Iterator<Series> ite=m_mesSeries.iterator();
		while(ite.hasNext())
		{
			Series s=ite.next();
			for(int i=0;i<s.getNbDonneeID();i++)
			{
				DonneeID don=s.getDonneeID(i);
				CritereCompose crit=new CritereCompose(m_critere,selection);
				n=crit.evaluer(don);
				if(!n.isError())addMesSeries(mesSeries,n,don,crit);
			}
		}
		return mesSeries;
	}

	/**
	 * @return les series tri�es
	 */
	public SortedSet<Series> getSeries()
	{return m_mesSeries;}

//	/** fonction de test
//	 * @param args
//	 */
//	public static void main(String[] args)
//	{
//		Fichier.chargerDonnees("C:\\Documents and Settings\\sle\\Bureau\\MichaelOry\\dynGraph\\jeudedonnees.ser");
//		Critere c=new CritereIndividu();
//		Individu indActif=new Individu("ind",true,null,null,null,null,null);
//		Individu indIllus=new Individu("ind",false,null,null,null,null,null);
//		System.out.println("individu tri� par test individu : "+c.evaluer(indActif)+" qualif : "+c.qualifier(indActif));
//		c=new CritereActifIllustratif();
//		System.out.println("actif tri� par test actif/illus :"+c.evaluer(indActif)+" qualif : "+c.qualifier(indActif));
//		System.out.println("illus tri� par test actif/illus :"+c.evaluer(indIllus)+" qualif : "+c.qualifier(indIllus));
//		System.out.println("===========================");
//		c=new CritereCompose(new CritereCompose(new CritereIndividu(),new CritereActifIllustratif()), new CritereIndividu());
//		System.out.println("ind actif tri� par test ind + actif/illus :"+c.evaluer(indActif)+" qualif : "+c.qualifier(indActif));
//		System.out.println("ind illus tri� par test ind + actif/illus :"+c.evaluer(indIllus)+" qualif : "+c.qualifier(indIllus));
//	}
}
