package don;

import java.util.Vector;

import don.DonneeParse.TypeAnalyse;
import exceptions.ExceptionIntegrite;

/**
 * Représente une variable continue dans le cadre d'une analyse factorielle
 * 
 * @author michaelory@gmail.com
 * @author julien.durand.eu@gmail.com
 */
public class ContinueAnalysee extends QualifiableRepresentable implements Variable, Continue
{
	private static final long serialVersionUID = 1719739952929130158L;

	/**
	 * La variable est-elle orthonormee ?
	 */
	private boolean m_orthonorme;

	/**
	 * le numéro de la variable dans la liste
	 */
	private int num;


	/** Constructeur
	 * @param annotation
	 * @param active
	 * @param gr
	 * @param coord
	 * @param cos2
	 * @param contrib
	 * @param orthonorme
	 */
	public ContinueAnalysee(String annotation, boolean active, Groupe gr,
			double[] coord, double[] cos2, double[] contrib,
			boolean orthonorme)
	{
		super(annotation, active, gr, coord, cos2, contrib,0);
		num=-1;
		m_orthonorme = orthonorme;
	}

	/**
	 * @return the m_orthonorme
	 */
	public boolean isOrthonorme() {
		return m_orthonorme;
	}

	/**
	 * @param orthonorme the m_orthonorme to set
	 */
	public void setOrthonorme(boolean orthonorme) {
		m_orthonorme = orthonorme;
	}

	/**
	 * @return le nom de la variable
	 * @see don.Variable#getAnnotation()
	 */
	public String getNom() {
		return getAnnotation();
	}


	/**
	 * @param vect le jeu de données
	 * @return le minimum de la continue sur le jeu de données
	 */
	public double getMin(Vector<? extends DonneeID> vect) {
		double min=Double.MAX_VALUE;
		for(int i=0;i<vect.size();i++) {
			double res=getValeur(vect.get(i));
			if(res<min)min=res;
		}
		return min;
	}

	/**
	 * @param vect le jeu de données
	 * @return le maximum de la continue sur le jeu de données
	 */
	public double getMax(Vector<? extends DonneeID> vect)
	{
		double max=-Double.MAX_VALUE;
		for(int i=0;i<vect.size();i++) {
			double res=getValeur(vect.get(i));
			if(res>max)max=res;
		} 
		return max;    
	}



	public double getValeur(DonneeID donneeID) {
		if(donneeID instanceof Individu){
			return getValeur((Individu)donneeID);
		}else if(donneeID instanceof Modalite){
			return ((Modalite)donneeID).getValeur(this);
		}else
			return 0;
	}
	
	@Override
	public double getValeur(DonneeID donneeID, TypeAnalyse t) {
		// TODO Auto-generated method stub
		return getValeur(donneeID);
	}

	/**
	 * @param vect
	 * @return l'amplitude de la variable
	 */
	public double getAmplitude(Vector<? extends DonneeID> vect) {
		return getMax(vect)-getMin(vect);
	}
	
	public Double getValeurAbsolueMax(Vector<? extends DonneeID> donnees) {		
		return Math.max(getMax(donnees), -getMin(donnees));
	}


	/**
	 * @param ind
	 * @return la valeur de l'individu pour la continue
	 */
	public double getValeur(Individu ind) {
		return ind.getValeur(num);
	}

	/**
	 * @return the num
	 */
	public int getNum() {
		return num;
	}

	/**
	 * @param num the num to set
	 */
	public void setNum(int num) {
		this.num = num;
	}

	public void verifierIntegrite() throws ExceptionIntegrite {
		super.verifierIntegrite();
		if(num==-1)throw new ExceptionIntegrite(ExceptionIntegrite.errNumCont);
		else if(num==0)throw new ExceptionIntegrite(ExceptionIntegrite.errNumCont);
		//ca n'est pas forcément une erreur, mais il vaut mieux vérifier
	}

	
}
