package don;

import java.util.Vector;

import exceptions.ExceptionIntegrite;

/**
 * Variable continue moyenne (qui peut contenir des varaiables partielles)
 * 
 * @author michaelory@gmail.com
 * @author julien.durand.eu@gmail.com
 * 
 */
public class ContinueMoyenne extends ContinueAnalysee implements DonneeMoyenne
{
	private static final long serialVersionUID = 8682264074537419185L;
	
	/**
	 * Les variables partielles correspondantes
	 */
	private Vector<ContinueAnalysee> m_branches;

	/**
	 * @param annotation
	 * @param active
	 * @param gr
	 * @param coord
	 * @param cos2
	 * @param contrib
	 * @param orthonorme
	 */
	public ContinueMoyenne(String annotation, boolean active, Groupe gr,
			double[] coord, double[] cos2, double[] contrib, boolean orthonorme) {
		super(annotation, active, gr, coord, cos2, contrib, orthonorme);
		m_branches = new Vector<ContinueAnalysee>();
	}

	/**
	 * @param annotation
	 * @param active
	 * @param gr
	 * @param coord
	 * @param cos2
	 * @param contrib
	 * @param orthonorme
	 * @param branches
	 */
	public ContinueMoyenne(String annotation, boolean active, Groupe gr,
			double[] coord, double[] cos2, double[] contrib,
			boolean orthonorme, Vector<ContinueAnalysee> branches) {
		super(annotation, active, gr, coord, cos2, contrib, orthonorme);
		m_branches = branches;
	}


	public Vector<DonneeID> getBranches() {
		return new Vector<DonneeID>(m_branches);
	}

	/**
	 * @param i
	 * @return la i-éme variable fille
	 */
	public ContinueAnalysee get(int i) {
		return m_branches.get(i);
	}

	/**
	 * @param branches
	 *            the m_branches to set
	 */
	public void setBranches(Vector<ContinueAnalysee> branches) {
		m_branches = branches;
	}

	/**
	 * ajoute la branche fille
	 * 
	 * @param fille
	 */
	public void addBranche(ContinueAnalysee fille) {
		m_branches.add(fille);
	}


	public int getNbBranches() {
		return m_branches.size();
	}
	/**
	 * Rend la variable dont l'annotation est pass�e en param�tre
	 * il peut s'agir de this, ou d'un de ses fils
	 * rend null si il n'est pas trouv�
	 * @param annot
	 * @return la variable
	 */
	public ContinueAnalysee getVariable(String annot) {
		ContinueAnalysee trouve=null;
		if(getAnnotation().equals(annot))trouve=this;
		else for(int i=0 ; i<getNbBranches() && trouve==null;i++){
			ContinueAnalysee temp=m_branches.elementAt(i);
			if(temp instanceof ContinueMoyenne) 
				trouve=((ContinueMoyenne)temp).getVariable(annot);
			else if(temp.getAnnotation().equals(annot))trouve=temp;
		}
		return trouve;
	}
	
	
	/** (non-Javadoc)
	 * @see don.DonneeID#verifierIntegrite()
	 */
	public void verifierIntegrite() throws ExceptionIntegrite {
	    super.verifierIntegrite();
	    for (ContinueAnalysee var : m_branches) {
		var.verifierIntegrite();
	    }
	}


}
