package don;

import ihm.FenetreErreur;
import java.util.HashMap;
import java.util.Vector;

//@author
//Nom du développeur
//
//@deprecated
//Marque la méthode comme dépréciée. Certains IDEs créent un avertissement à la compilation si la méthode est appelée.
//
//@exception
//Documente une exception lancée par une méthode — voir aussi @throws.
//
//@param
//Définit un paramètre de méthode. Requis pour chaque paramètre.
//
//@return
//Documente la valeur de retour. Ce tag ne devrait pas être employé pour des constructeurs ou des méthodes définis avec un type de retour void.
//
//@see
//Documente une association à une autre méthode ou classe.
//
//@since
//Précise à quelle version de la SDK/JDK une méthode a été ajoutée à la classe.
//
//@throws
//Documente une exception lancée par une méthode. Un synonyme pour @exception disponible depuis Javadoc 1.2.
//
//@version
//Donne la version d'une classe ou d'une méthode.

/**
 * décrit un individu
 * peut être individus partiels
 * 
 * @author michaelory@gmail.com
 * @author julien.durand.eu@gmail.com
 */
public class Individu extends Partitionnable
{
    private static final long serialVersionUID = -5138037164711285332L;

    /** Individu père de cet Individu */
    private IndividuMoyen m_pere;

    /** Les coordonnées de l'individu dans l'espace d'origine */
    private double[] m_coordContinues;

    /** Les modalités qui concernent l'individu */
    private HashMap<Nominale,Modalite> m_modalites;

    /**
     * @param annotation l'annotation
     * @param active le statut
     * @param gr le groupe
     * @param coord les coordonnees sur les axes
     * @param cos2 les cos2
     * @param contrib les contributions
     * @param coordContinues les coordonnees dans l'espace de départ
     */
    public Individu(String annotation, boolean active, Groupe gr,
	    double[] coord, double[] cos2, double[] contrib,
	    double[] coordContinues) {
	super(annotation, active, gr, coord, cos2, contrib, 0);
	m_coordContinues = coordContinues;
	m_modalites = new HashMap<Nominale,Modalite>();
    }

    /**
     * @return the m_coordContinues
     */
    public double[] getCoordContinues() {
	return m_coordContinues;
    }

    /**
     * @param i
     * @return la coordonnée de l'individu pour la variable numéro i
     * valeur prise
     */
    public double getValeur(int i) {
	return m_coordContinues[i];
    }

    /**
     * @param z
     *            the m_z to set
     */
    public void setCoordContinues(double[] z) {
	m_coordContinues = z;
    }

    /**
     * @return the m_modalites
     */
    public Vector<Modalite> getModalites() {
	return new Vector<Modalite>(m_modalites.values());
    }

    /**
     * @param nominale
     * @return the m_modalites
     */
    public Modalite getModalite(Nominale nominale)
    {
	return m_modalites.get(nominale);
    }

    /**
     * ajoute une modalité, remplace éventuellement si une modalité de même nominale est déjà présente
     * avertit si nécessaire la nouvelle modalité
     * @param mod
     */
    public void addModalite(Modalite mod) {
	if(mod==null)new FenetreErreur("modalité nulle !");
	else {
	    if(m_modalites.containsKey(mod.getNominale())) {//si la nominale est déjà présente, on remplace
		m_modalites.remove(mod.getNominale());
	    }
	    m_modalites.put(mod.getNominale(), mod);
	    if(mod.getNbIndividus()==0 || !mod.getIndividus().contains(this)){
		//si la nouvelle modalite n'est pas encore au courant que la donnée lui appartient, l'avertir
		mod.addIndividu(this);
	    }
	}
    }

    /**
     * @return the m_pere
     */
    public IndividuMoyen getPere() {
	return m_pere;
    }

    /**
     * met à jour l'individu père
     * informe le père si nécessaire
     * calcule la génération
     * @param pere the m_pere to set
     */
    public void setPere(IndividuMoyen pere) {
	if(m_pere!=null){
	    //TODO avertir le père que l'individu ne lui appartient plus
	}
	m_pere = pere;
	if(pere.getBranches()==null || !pere.getBranches().contains(this)){
	    //si le nouveau père n'est pas au courant, l'avertir
	    pere.addBranche(this);
	}
	setGeneration(pere.getGeneration()+1);	
    }


    /**
     * @see don.Partitionnable#toString()
     */
    public String toString() {
	String chaine=super.toString();
	chaine+="\nvaleurs quanti : ";
	for (int i = 0; i < m_coordContinues.length; i++) {
		 chaine += m_coordContinues[i]+"|| ";		
	}
	chaine+="\nmodalités :";
	for (Modalite mod : m_modalites.values()) {
	    if(mod!=null)chaine+="\n"+mod.getNominale().getAnnotation()+" : "+mod.getAnnotation();
	    else chaine+="\nmodalité nulle";
	}

	return chaine;
    }

}
