package don;

import java.util.Vector;

import exceptions.ExceptionIntegrite;

/**
 * décrit un individu moyen
 * 
 * @author michaelory@gmail.com
 * @author julien.durand.eu@gmail.com
 */
public class IndividuMoyen extends Individu implements DonneeMoyenne{

	private static final long serialVersionUID = 6648446927310047453L;

	private Vector<Individu> m_branches;

	/**
	 * @param annotation
	 * @param active
	 * @param gr
	 * @param coord
	 * @param cos2
	 * @param contrib
	 * @param z
	 */
	public IndividuMoyen(String annotation, boolean active, Groupe gr,
			double[] coord, double[] cos2, double[] contrib,
			double[] z) {
		super(annotation, active, gr, coord, cos2, contrib, z);
		m_branches=new Vector<Individu>();
	}

	public Vector<DonneeID> getBranches() {
		return new Vector<DonneeID>(m_branches);
	}

	/** 
	 * @param i
	 * @return la i-ème variable fille
	 */
	public Individu get(int i) {
		return m_branches.get(i);
	}

	/**
	 * ajoute la branche fille
	 * 
	 * @param fille
	 */
	public void addBranche(Individu fille) {
//		System.out.println("toto");// ne passe pas là...
//		if (fille.getGr().isActive()) { //@trovalet 2012-03-23
//			System.out.println(" arg... "+fille);
			m_branches.add(fille);
			if(fille.getPere()==null || !fille.getPere().equals(this)){
				//si la fille n'est pas au courant, l'informer
				fille.setPere(this);
			}
//		}		
	}
	
	public int getNbBranches() {
		return m_branches.size();
	}

	/**
	 * @see don.Partitionnable#setGeneration(int)
	 */
	public void setGeneration(int generation) {
		super.setGeneration(generation);
		//avertir les enfants
		for (Individu mod_fille : m_branches) {
			mod_fille.setGeneration(generation+1);
		}
	}

	/**
	 * Rend l'individu dont l'annotation est passee en parametre
	 * il peut s'agir de this, ou d'un de ses fils
	 * rend null si il n'est pas trouve
	 * @param annot
	 * @return l'individu
	 */
	public Individu getIndividu(String annot) {
		Individu trouve=null;
		if(getAnnotation().equals(annot))trouve=this;
		else for(int i=0 ; i<getNbBranches() && trouve==null;i++){
			Individu temp=m_branches.elementAt(i);
			if(temp instanceof IndividuMoyen) 
				trouve=((IndividuMoyen)temp).getIndividu(annot);
			else if(temp.getAnnotation().equals(annot))trouve=temp;
		}
		return trouve;
	}


	/**
	 * methode affichage
	 * @return chaine charactere
	 */
	public String toString() {
		String chaine=super.toString();
		for (Individu fille : m_branches) {
			chaine+="\nenfant : "+fille.toString();
		}
		chaine+="\n(fin "+getAnnotation()+")\n";
		return chaine;
	}



	@Override
	public void verifierIntegrite() throws ExceptionIntegrite {
		super.verifierIntegrite();
		for (Individu ind : m_branches) {
			ind.verifierIntegrite();
		}
	}




//	/**
//	 * @param args
//	 */
//	public static void main(String[] args) {
//		Individu fils = new Individu("ind1",true,null,null,null,null,null);
//		IndividuMoyen pere =new IndividuMoyen("ind2",true,null,null,null,null,null);
//		Individu fils2 = new Individu("ind3",true,null,null,null,null,null);
//		fils.setPere(pere);
//		pere.addBranche(fils2);
//		System.out.println("Apres construction de l'arborescence :");
//		System.out.println(pere);
//
//		pere.setGeneration(3);
//		System.out.println("---------------------------\nApres avoir fixe le niveau du pere a 3 :");
//		System.out.println(pere);
//
//		System.out.println("---------------------------\nindividu d'annotation \"ind3\" :\n"+pere.getIndividu("ind3"));
//
//		/* 
//		 * R�sultat :
//		 * 
//        Apr�s construction de l'arborescence :
//        ind2 (pas de groupe) generation : 0
//        enfant : ind1 (pas de groupe) generation : 1
//        enfant : ind3 (pas de groupe) generation : 1
//        (fin ind2)
//
//        ---------------------------
//        Apr�s avoir fix� le niveau du p�re � 3 :
//        ind2 (pas de groupe) generation : 3
//        enfant : ind1 (pas de groupe) generation : 4
//        enfant : ind3 (pas de groupe) generation : 4
//        (fin ind2)
//
//        ---------------------------
//        individu d'annotation "ind3" :
//        ind3 (pas de groupe) generation : 4
//		 */
//	}




}
