package don;

import java.util.HashMap;
import java.util.Vector;

import exceptions.ExceptionIntegrite;

/**
 * décrit une modalité
 * 
 * @author michaelory@gmail.com
 * @author julien.durand.eu@gmail.com
 */
/** SLE : marche certainement comme les individus et les individus moyens */
/** SLE : comprend les individus moyens et partiels (à vérifier) */

public class Modalite extends QualifiableRepresentable
{
	private static final long serialVersionUID = -5668398340053544063L;

	/** sa nominale */
	/** SLE : la variable qualitative à laquelle la modalité appartient */
	private Nominale m_nominale;
	
	/** modalité père de cette modalité dans AFM */
	/** SLE : la modalité est à l'individu ce que la modalité moyenne est à l'individu moyen */
	private ModaliteMoyenne m_pere;
	
	/** Les individus concernés par la modalité */
	private Vector<Individu> m_individus;

	private HashMap<Continue,Double> m_val_cont;
	
	/* SLE : generation est en prévision de l'AFMH */
	/** constructeur : fonction pour initialiser une modalité 
	 * @param nominale 
	 * @param annotation 
	 * @param active 
	 * @param gr 
	 * @param coord 
	 * @param cos2 
	 * @param contrib 
	 * @param generation **/	
	public Modalite(Nominale nominale,String annotation, boolean active, Groupe gr,
			double[] coord, double[] cos2, double[] contrib, int generation) {
		super(annotation, active, gr, coord, cos2, contrib, generation);
		m_nominale=nominale;
		m_individus = new Vector<Individu>();
		m_val_cont=new HashMap<Continue, Double>();
	}


	/**
	 * @param nominale 
	 * @param annotation 
	 */
	
	/* SLE : le même avec des valeurs par défaut */
	/** Autre constructeur 
	 * @param nominale 
	 * @param annotation **/	
	public Modalite(Nominale nominale,String annotation) {
		super(annotation,false,null,null,null,null,0);
		m_nominale=nominale;
		m_individus = new Vector<Individu>();
	}
	
	/**
	 * @param m_nominale the m_nominale to set
	 */
	public void setNominale(Nominale m_nominale) {
	    this.m_nominale = m_nominale;
	}

	/**
	 * @return la nominale 
	 */
	public Nominale getNominale()
	{return m_nominale;}

	/**
	 * @return the m_individus
	 */
	public Vector<Individu> getIndividus() {
		return m_individus;
	}

	/* SLE : calcule la moyenne de la variable c pour les individus appartenant à une modalité, ex. pour les hommes ou pour les femmes */
	/* SLE : en général la variable c correspond au vecteur des coordonnées des individus sur un axe factoriel */
	/* SLE : dans le cas de la représentation d'un dataframe c correspond à une variable du jeu de données */
	/* SLE : fonction hyper importante car permet de représenter les modalités sur un grapĥique */
	/**
     * @param c
     * @return la coordonnée de la modalité pour la variable c
     */
    public double getValeur(Continue c) {
    	if(m_val_cont==null){
    		m_val_cont=new HashMap<Continue, Double>();
    	}
    	if(m_val_cont.get(c)==null){
    		int nb=0;
    		double val=0;
    		for (Individu ind : m_individus) {
				nb++;
				val+=c.getValeur(ind);
			}
    		m_val_cont.put(c, val/nb);
    	}
    	return m_val_cont.get(c);
    }
    

    /*  SLE : à voir */
    /**
	 * ajoute un individu aux individus concernés par la modalité 
	 * @param ind
	 */
    public void addIndividu(Individu ind) {
		m_individus.add(ind);
		if(!ind.getModalites().contains(this)){
			//si l'individu n'est pas au courant, l'informer
			ind.addModalite(this);
		}
	}


	/**
	 * @return le nombre d'individus concernés par la modalité
	 */
	public int getNbIndividus() {
		return m_individus.size();
	}

	/**
	 * @return the m_pere
	 */
	public ModaliteMoyenne getPere() {
		return m_pere;
	}


	/**
	 * met à jour l'individu père
	 * informe le père si nécessaire
	 * calcule la génération
	 * @param pere the m_pere to set
	 */
	public void setPere(ModaliteMoyenne pere) {
		if(m_pere!=null){
			//avertir le père que l'individu ne lui appartient plus
			//TODO
		}

		m_pere = pere;
		if(!pere.getBranches().contains(this)){
			//si le nouveau père n'est pas au courant, l'avertir
			pere.addBranche(this);
		}
		setGeneration(pere.getGeneration()+1);	
	}

	@Override
	public void verifierIntegrite() throws ExceptionIntegrite {
	    super.verifierIntegrite();
	    for(int i=0;i<m_individus.size();i++) {
		Individu ind=m_individus.get(i);
		if(ind.getModalite(m_nominale)==null) {
		    ind.addModalite(this);
		    System.out.println("l'individu n'était pas au courant de sa modalité");
		}else if(!ind.getModalite(m_nominale).equals(this)) {
		    if(!ind.getModalite(m_nominale).getNominale().equals(m_nominale)){//s'il semble que l'individu a tort
			ind.addModalite(this);
		    }else//si c'est vraiment trop ambigu
		    throw new ExceptionIntegrite("incohérence modalité - individu : \nmodalité : "+
				this.getAnnotation()+"\nl'individu : "+ind.getAnnotation()+"est lié à "+ind.getModalite(getNominale()).getAnnotation()+"("+ind.getModalite(getNominale()).getNominale().getAnnotation()+") pour la nominale "+m_nominale.getAnnotation());
		}
		ind.verifierIntegrite();
		
	    }
	}


}
