package don;

import java.util.Vector;

import exceptions.ExceptionIntegrite;

/**
 * décrit une modalité moyenne
 * 
 * @author michaelory@gmail.com
 * @author julien.durand.eu@gmail.com
 */
public class ModaliteMoyenne extends Modalite implements DonneeMoyenne{

	private static final long serialVersionUID = 4332798465977843188L;

	/** les modalites filles  */
	private Vector<Modalite> m_branches;

	/**
	 * @param nominale 
	 * @param annotation
	 * @param active
	 * @param gr
	 * @param coord
	 * @param cos2
	 * @param contrib
	 * @param generation
	 */
	public ModaliteMoyenne(Nominale nominale,String annotation, boolean active, Groupe gr,
			double[] coord, double[] cos2, double[] contrib, int generation) {
		super(nominale,annotation, active, gr, coord, cos2, contrib, generation);
		m_branches = new Vector<Modalite>();
	}

	/**
	 * @param nominale
	 * @param annotation
	 */
	public ModaliteMoyenne(Nominale nominale,String annotation) {
		super(nominale,annotation);
		m_branches = new Vector<Modalite>();
	}

	
	public Vector<DonneeID> getBranches() {
		return new Vector<DonneeID>(m_branches);
	}

	/**
	 * @param i
	 * @return la i-ème variable fille
	 */
	public Modalite get(int i) {
		return m_branches.get(i);
	}

	/**
	 * ajoute la branche fille
	 * 
	 * @param fille
	 */
	public void addBranche(Modalite fille) {
		m_branches.add(fille);
		if(fille.getPere()==null || !fille.getPere().equals(this)){
			//si la fille n'est pas au courant, l'informer
			fille.setPere(this);
		}
	}

	
	public int getNbBranches() {
		return m_branches.size();
	}
	

	/**
	 * @see don.Partitionnable#setGeneration(int)
	 */
	@Override
	public void setGeneration(int generation) {
		super.setGeneration(generation);
		//avertir les enfants
		for (Modalite mod_fille : m_branches) {
			mod_fille.setGeneration(generation+1);
		}
	}
	


	
	/**
	 * Rend la modalité dont l'annotation est passée en paramètre
	 * il peut s'agir de this, ou d'un de ses fils
	 * rend null si elle n'est pas trouvée
	 * @param annot
	 * @return la modalité
	 */
	public Modalite getModalite(String annot) {
		Modalite trouve=null;
		if(getAnnotation().equals(annot))trouve=this;
		else for(int i=0 ; i<getNbBranches() && trouve==null;i++){
			Modalite temp=m_branches.elementAt(i);
			if(temp instanceof ModaliteMoyenne) 
				trouve=((ModaliteMoyenne)temp).getModalite(annot);
			else if(temp.getAnnotation().equals(annot))trouve=temp;
		}
		return trouve;
	}

	@Override
	public String toString() {
		String chaine=super.toString();
		chaine+="\nenfants : ";
		for (Modalite fille : m_branches) {
			chaine+=fille.toString()+"\n";
		}
		chaine+="fin modalite moyenne\n";
		return chaine;
	}

	
	@Override
	public void verifierIntegrite() throws ExceptionIntegrite {
	    super.verifierIntegrite();
	    for (Modalite mod : m_branches) {
		mod.verifierIntegrite();
	    }
	}

}
