package don;

/**
 * Interface pour les données qui peuvent être décrites par 
 * des modalités, des continues et des nominales
 * 
 *  @author julien.durand@gmail.com
 */
public interface Qualifiable {
	
	/**
	 * @param mod
	 * @return la vtest pour la description par la modalite
	 */
	public double getDescMod(Modalite mod);

	/**
	 * fixe selon quelle vtest la donnée est décrite par mod
	 * @param mod
	 * @param vtest
	 */
	public void setDescMod(Modalite mod, Double vtest);

	/** 
	 * @param cont
	 * @return la proba critique signée pour la description par la continue
	 */
	public double getDescCont(ContinueAnalysee cont);

	/**
	 * fixe selon quelle proba critique signée la donnée est décrite par cont
	 * @param cont
	 * @param vtest
	 */
	public void setDescCont(ContinueAnalysee cont, Double vtest);
	
	/**
	 * @param nom
	 * @return la proba critique signée pour la description par la nominale
	 */
	public double getDescNom(Nominale nom);

	/**fixe selon quelle proba critique signée la donnée est décrite par nom
	 * 
	 * @param nom
	 * @param vtest
	 */
	public void setDescNom(Nominale nom, Double vtest);
	
	/**
	 * @param don
	 * @return la vtest ou proba critique signée pour la description par la donnee (mod ou cont)
	 */
	public double getDesc(DonneeID don);

	/**
	 * @param don
	 * @return la proba critique signée pour la description par la nominale
	 */
	public double getDesc(Nominale don);

	
	/**
	 * @return le nom de la variable
	 */
	public String getAnnotation();
	

}
