package drawing;

import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Vector;

import org.jfree.chart.ChartPanel;
import org.jfree.chart.ChartRenderingInfo;
import org.jfree.chart.annotations.XYIDAnnotation;
import org.jfree.chart.annotations.XYLineAnnotation;
import org.jfree.chart.plot.XYPlot;

import don.DonneeID;

/**
 * Canevas sur lequel on va dessiner les annotations et les points partiels
 * 
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class Canvas
{
	/** 
	 * les annotations
	 */
	private Vector<XYIDAnnotation> m_annot;

	/** 
	 * les "pattes" ou "branches" des individus moyens 
	 */
	private Vector<XYLineAnnotation> m_branche;

	/** 
	 * les points sélectionnés 
	 */
	private Vector<DonneeID> m_point;

	/**
	 * constructeur par défaut
	 *
	 */
	public Canvas()
	{
		m_annot = new Vector<XYIDAnnotation>();
		m_branche = new Vector<XYLineAnnotation>();
		m_point=new Vector<DonneeID>();
	}

	/**
	 * Vide la liste des branches
	 */
	public void clearBranche() {
		m_branche.clear();
	}

	/**
	 * ajoute une branche
	 * @param branche
	 */
	public void addBranche(XYLineAnnotation branche) {
		m_branche.add(branche);
	}

	/**
	 * @param i
	 * @return la branche d'indice i
	 */
	public XYLineAnnotation getBranche(int i) {
		return m_branche.get(i);
	}

	/**
	 * @return le nombre de branches
	 */
	public int tailleBranche() {
		return m_branche.size();
	}

	/**
	 * @return la liste d'annotations
	 */
	public Vector<XYIDAnnotation> getVectAnnot() {
		return m_annot;
	}

	/**
	 * fixe la liste d'annotations
	 * @param annot : la nouvelle liste d'annotations
	 */
	public void setVectAnnot(Vector<XYIDAnnotation> annot) {
		m_annot = annot;
	}

	/**
	 * @param i
	 * @return l'annotation d'indice i
	 */
	public XYIDAnnotation getAnnot(int i) {
		return m_annot.get(i);
	}

	/**
	 * Retouve l'indice d'une annotation
	 * 
	 * @return l'index de l'annotation	  
	 * @param annot :l'annotation é rechercher
	 */
	public int indexOfAnnot(XYIDAnnotation annot) {
		return m_annot.indexOf(annot);
	}

	/**
	 * remplace une annotation
	 * 
	 * @param nouvelle : la nouvelle annotation
	 * @param destinationIndex : l'index de l'ancienne annotation
	 */
	public void remplaceAnnot(XYIDAnnotation nouvelle, int destinationIndex) {
		m_annot.setElementAt(nouvelle, destinationIndex);
	}

	/**
	 * supprime une annotation
	 * 
	 * @param index l'indice de l'annotation é supprimer 
	 */
	public void rmAnnot(int index) {
		m_annot.remove(index);
	}

	/**
	 * supprime une annotation
	 * 
	 * @param annot l'annotation é supprimer 
	 */
	public void rmAnnot(XYIDAnnotation annot) {
		m_annot.remove(annot);
	}

	/** 
	 * vide la liste d'annotations 
	 */
	public void clearAnnot() {
		m_annot.clear();
	}

	/**
	 * @return le nombre d'annotations
	 */
	public int tailleAnnot() {
		return m_annot.size();
	}

	/**
	 * ajoute une annotation
	 * 
	 * @param annot l'annotation XYIDAnnotation é ajouter
	 */
	public void addAnnot(XYIDAnnotation annot) {
		m_annot.add(annot);
	}

	/**
	 * ajoute un point
	 * @param point
	 */
	public void addPoint(DonneeID point){m_point.add(point);}

	/**
	 * remplace la collection de points
	 * @param point
	 */
	public void setPoint(Vector<DonneeID> point){m_point=point;}

	/**
	 * @param i
	 * @return le point d'index i
	 */
	public DonneeID getPoint(int i){return m_point.get(i);}

	/**
	 * vide la liste de points
	 */
	public void clearPoint(){m_point.clear();}

	/**
	 * @return la collection de points
	 */
	public Vector<DonneeID> getPoint(){return m_point;}

	/**
	 * @return le nombre de points
	 */
	public int taillePoint(){return m_point.size();}

	/**
	 * cherche une annotation en fonction d'un point
	 * 
	 * @param f le panel dans lequel on cherche
	 * @param pointChart le point oé chercher
	 * @return l'annotation qui se trouve aux coordonnées du point, avec marge d'erreur. null si on ne trouve rien
	 */
	public XYIDAnnotation findAnnot(ChartPanel f, Point2D pointChart) {
		int i = 0;
		double x = pointChart.getX();
		double y = pointChart.getY();
		while (i < tailleAnnot()) {
			XYIDAnnotation annot = getAnnot(i);
			double xa=chartToJava2D(f, annot.getX(), true);
			double ya=chartToJava2D(f, annot.getY(), false);
			Rectangle2D s = (Rectangle2D) annot.getBounds((Graphics2D) f.getGraphics()); 
			double minlblx=xa+s.getX()-0.5*s.getWidth();
			double maxlblx=xa+s.getX()+0.5*s.getWidth();
			double minlbly=ya+s.getY();
			double maxlbly=ya+s.getY()+s.getHeight();

			if (x >= minlblx && x <= maxlblx && y >= minlbly && y <= maxlbly)
				return annot;
			i++;
		}
		return null;
	}


	/**
	 * cherche des points en fonction d'un rectangle de sélection
	 * 
	 * @param f le panel dans lequel on cherche
	 * @param rec le rectangle oé chercher
	 * @return les annotations qui se trouvent dans le rectangle. une liste vide si on ne trouve rien
	 */
	public Vector<DonneeID> findPoint(ChartPanel f,Rectangle2D rec) {
		// conversion des coordonnées Java en coordonnées jfreechart
		double minlblx = java2Dtochart(f, rec.getX(), true);
		double maxlblx = java2Dtochart(f, rec.getX() + rec.getWidth(), true);
		//les axes Java et JfreeChart ne sont pas orientés pareil :
		//le max sur y en JFreeChart est le min sur y en Java 
		double maxlbly = java2Dtochart(f, rec.getY(), false);
		double minlbly = java2Dtochart(f, rec.getY() + rec.getHeight(), false);
		Vector<DonneeID> save = new Vector<DonneeID>();
		for (int i = 0; i < taillePoint(); i++) {
			double x=getPoint(i).getCoord(f.getGestionnaire().getAxeX());
			double y=getPoint(i).getCoord(f.getGestionnaire().getAxeY());

			if (x >= minlblx && x <= maxlblx && y >= minlbly && y <= maxlbly)
				save.add(getPoint(i));
		}
		return save;
	}

	/**
	 * 
	 * @param f
	 * @param rectJava
	 * @param Xchart
	 * @return vrai si le rectangle de l'espace java <b>rectJava<b> contient la droite x=Xchart
	 */
	public boolean contientX(ChartPanel f, Rectangle2D rectJava, double Xchart){
		// conversion des coordonnées Java en coordonnées jfreechart
		double minlblx = java2Dtochart(f, rectJava.getX(), true);
		double maxlblx = java2Dtochart(f, rectJava.getX() + rectJava.getWidth(), true);
		return Xchart >= minlblx && Xchart <= maxlblx;
	}

	/**
	 * @param f
	 * @param rectJava
	 * @return vrai si le rectangle de l'espace java <b>rectJava<b> contient un des bords verticaux de la zone de graphe
	 */
	public boolean bordVertical(ChartPanel f, Rectangle2D rectJava){
		ChartRenderingInfo info = f.getChartRenderingInfo();
		Rectangle2D dataArea = info.getPlotInfo().getDataArea();
		//bord gauche
		double bord_gauche = dataArea.getX();
		double bord_droit = bord_gauche+dataArea.getWidth();
		double rect_gauche = rectJava.getX();
		double rect_droite = rect_gauche+rectJava.getWidth();
		if(bord_gauche >= rect_gauche && bord_gauche <= rect_droite)
			return true;
		else
			if(bord_droit >= rect_gauche && bord_droit <=rect_droite)
				return true;
			else return false;
	}

	/**
	 * @param f 
	 * @param rectJava
	 * @return vrai si le rectangle de l'espace java <b>rectJava<b> contient un des bords horizontaux de la zone de graphe
	 */
	public boolean bordHorizontal(ChartPanel f, Rectangle2D rectJava){
		ChartRenderingInfo info = f.getChartRenderingInfo();
		Rectangle2D dataArea = info.getPlotInfo().getDataArea();
		//bord gauche
		double bord_haut = dataArea.getY();
		double bord_bas = bord_haut+dataArea.getHeight();
		double rect_haut = rectJava.getY();
		double rect_bas = rect_haut+rectJava.getHeight();
		if(bord_haut >= rect_haut && bord_haut <= rect_bas)
			return true;
		else
			if(bord_bas >= rect_haut && bord_bas <=rect_bas)
				return true;
			else return false;
	}
	/**
	 * 
	 * @param f
	 * @param rectJava
	 * @param Ychart
	 * @return vrai si le rectangle de l'espace java <b>rectJava<b> contient la droite y=Ychart
	 */
	public boolean contientY(ChartPanel f, Rectangle2D rectJava, double Ychart){
		// conversion des coordonnées Java en coordonnées jfreechart
		//les axes Java et JfreeChart ne sont pas orientés pareil :
		//le max sur y en JFreeChart est le min sur y en Java 
		double maxlbly = java2Dtochart(f, rectJava.getY(), false);
		double minlbly = java2Dtochart(f, rectJava.getY() + rectJava.getHeight(), false);
		return Ychart >= minlbly && Ychart <= maxlbly;
	}


	/**
	 * translation entre repére java et repére jfreechart
	 * @param f le panel	  
	 * @param val valeur transformée
	 * @param isX boolean pour savoir si on travaille sur l'axe x ou y
	 * @return les coordonnées dans le chart
	 */
	public static double java2Dtochart(ChartPanel f,double val, boolean isX) {
		XYPlot plot = (XYPlot) f.getChart().getPlot();
		ChartRenderingInfo info = f.getChartRenderingInfo();
		Rectangle2D dataArea = info.getPlotInfo().getDataArea();
		if (isX) {
			return plot.getDomainAxis().java2DToValue(val, dataArea,
					plot.getDomainAxisEdge());
		} else {
			return plot.getRangeAxis().java2DToValue(val, dataArea,
					plot.getRangeAxisEdge());
		}
	}

	/**
	 * translation entre repére java et repére jfreechart
	 * @param f le panel	  
	 * @param val valeur transformée
	 * @param isX boolean pour savoir si on travaille sur l'axe x ou y
	 * @return les coordonnées Java
	 */
	public static double chartToJava2D(ChartPanel f,double val, boolean isX) {
		XYPlot plot = (XYPlot) f.getChart().getPlot();
		ChartRenderingInfo info = f.getChartRenderingInfo();
		Rectangle2D dataArea = info.getPlotInfo().getDataArea();
		if (isX) {
			return plot.getDomainAxis().valueToJava2D(val, dataArea,
					plot.getDomainAxisEdge());			
		} else {
			return plot.getRangeAxis().valueToJava2D(val, dataArea,
					plot.getRangeAxisEdge());
		}
	}

}