package drawing;

import ihm.Gestionnaire;
import ihm.Habilleur;

import java.io.Serializable;
import java.util.HashMap;

import don.ContinueAnalysee;
import don.DonneeID;
import don.DonneeParse;

/** 
 * Classe qui sert à sauver la configuration de l'affichage de Gestionnaire
 * elle sera sérialisée lors de la sauvegarde
 * 
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class ConfAffichage implements Serializable
{
	private static final long serialVersionUID = 5687014387903355006L;
	
	/**
	 * @see Gestionnaire#m_type
	 */
	private int m_type;
	
	
	/**
	 * @see Gestionnaire#annotation
	 */
	private HashMap<DonneeID, Boolean> m_annotation;
	
	/**
	 * @see Gestionnaire#partiel
	 */
	private HashMap<DonneeID, Boolean> m_partiel;
	
	/**
	 * @see Gestionnaire#fantome
	 */
	private HashMap<DonneeID, Boolean> m_fantome;
	
	/**
	 * @see Gestionnaire#m_axeX
	 */
	private ContinueAnalysee[] m_axeX;
	
	/**
	 * @see Gestionnaire#m_axeY
	 */
	private ContinueAnalysee[] m_axeY;


	
	/**
	 * @see Gestionnaire#m_affGrid
	 */
	private boolean m_affGrille;
	
	/**
	 * @see Gestionnaire#m_titres
	 */
	private String[] m_titresGraphe;
	
	/**
	 * @see Gestionnaire#m_titresX
	 */
	private String[] m_titresX;
	
	/**
	 * @see Gestionnaire#m_titresY
	 */
	private String[] m_titresY;

	private Habilleur m_habilleur_facto;

	private Habilleur m_habilleur_data;
	
	/**
	 * Constructeur, demande toutes les infos qui doivent étre sauvegardées
	 * @param type
	 * @param titresGraphe 
	 * @param titresX 
	 * @param titresY 
	 * @param fantome 
	 * @param partiel 
	 * @param annotation 
	 * @param axeX
	 * @param axeY
	 * @param affGrille
	 * @param habilleurFacto 
	 * @param habilleurData 
	 */
	public ConfAffichage(int type, String[] titresGraphe, String[] titresX, String[] titresY,
			HashMap<DonneeID, Boolean> annotation, HashMap<DonneeID, Boolean> partiel,
			HashMap<DonneeID, Boolean> fantome, ContinueAnalysee[] axeX,ContinueAnalysee[] axeY,boolean affGrille, Habilleur habilleurFacto, Habilleur habilleurData)
	{
		m_type=type;
		m_titresGraphe=titresGraphe;
		m_titresX=titresX;
		m_titresY=titresY;
		m_annotation=annotation;
		m_partiel=partiel;
		m_fantome=fantome;
		m_axeX=axeX;m_axeY=axeY;
		m_habilleur_facto=habilleurFacto;
		m_habilleur_data=habilleurData;
		m_affGrille=affGrille;
	}
	
	/**
	 * @return le type du graphique
	 */
	public int getType(){return m_type;}
	

	/**
	 * @return la table d'annotation
	 */
	public HashMap<DonneeID, Boolean> getAnnotation(){return m_annotation;}
	
	/**
	 * @return la table de partiel
	 */
	public HashMap<DonneeID, Boolean> getPartiel(){return m_partiel;}
	
	/**
	 * @return la table de fantome
	 */
	public HashMap<DonneeID, Boolean> getFantome(){return m_fantome;}
	
	/**
	 * @return l'axe X
	 */
	public ContinueAnalysee[] getAxeX(){return m_axeX;}
	
	/**
	 * @return l'axe Y
	 */
	public ContinueAnalysee[] getAxeY(){return m_axeY;}

	
	/**
	 * @return si la grille est affiché
	 */
	public boolean isAffGrille(){return m_affGrille;}

	/**
	 * @return les titres des graphes
	 */
	public String[] getTitresGraphe() {
		if(m_titresGraphe!=null)return m_titresGraphe;
	    else {
		String[] t={null,null,null,null};
		return t;
	    }
	}

	/**
	 * @return les titres des axes X
	 */
	public String[] getTitresX() {
		if(m_titresX!=null)return m_titresX;
	    else {
		String[] t={null,null,null,null};
		return t;
	    }
	}

	/**
	 * @return les titres des axes Y
	 */
	public String[] getTitreY() {
		if(m_titresY!=null)return m_titresY;
	    else {
		String[] t={null,null,null,null};
		return t;
	    }
	}

	/**
	 * @param m_don
	 * @return habilleur facto
	 */
	public Habilleur getHabilleurFacto(DonneeParse m_don) {
		return m_habilleur_facto;
	}

	/**
	 * @param m_don
	 * @return habilleur data
	 */
	public Habilleur getHabilleurData(DonneeParse m_don) {
		return m_habilleur_data;
	}
}
