package drawing;

import ihm.Gestionnaire;

import java.awt.Color;
import java.awt.Paint;
import java.awt.Shape;
import java.io.Serializable;
import java.util.Vector;

import criteres.Info;

import don.DonneeID;

/**
 * Une série correspond à un ensemble de données 
 * qui vont être affichées de la même façon dans un graphe
 * 
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class Series implements Serializable
{
	private static final long serialVersionUID = -2741100465918269601L;

	/**
	 * les données concernées
	 */
	private Vector<DonneeID> m_val;

	/**
	 * la couleur de la serie
	 */
	private Color m_couleur;
	
	/**
	 * la forme de la serie
	 */
	private Shape m_forme;

	/**
	 * la définition de la série
	 */
	private Info m_definition;

	private String m_semantique;

	private Boolean isAffLegende;

	/**
	 * Constructeur
	 * initialise la collection
	 * @param i 
	 * @param semantique 
	 */
	public Series(Info i, String semantique)
	{m_val=new Vector<DonneeID>();
	m_definition=i;
	m_semantique=semantique;
	isAffLegende=true;}

	/**
	 * Constructeur
	 * initialise la collection
	 * pas de définition
	 */
	public Series() {
		m_val=new Vector<DonneeID>();
		m_definition=null;
	}

	/**
	 * retourne la donneeID à l'index i
	 * @param i
	 * @return la donneeID
	 */
	public DonneeID getDonneeID(int i)
	{return m_val.get(i);}

	/**
	 * @return la collection de donneeID
	 */
	public Vector<DonneeID> getMesDonneeID()
	{return m_val;}

	/**
	 * ajout d'une donneeID
	 * @param obj
	 * */
	public void addDonneeID(DonneeID obj)
	{m_val.add(obj);}

	/**
	 * @return le nombre de donneeID
	 */
	public int getNbDonneeID()
	{return m_val.size();}

	/**
	 * @return vrai si la série est représentée par une forme pleine (série active)
	 */
	public boolean isFormePleine()
	{
		/*if( m_val.get(0) instanceof Groupe)
	    return false;
	else*/
		return m_val.get(0).isActive();
	}

	/**
	 * @return la forme des données : 
	 * cercle pour les individus, rectangle pour les modalités et triangle pour les groupes
	 * Ne dit pas s'il la forme est pleine ou non : @see {@link Series#isFormePleine()}
	 */
	public Shape getForme()
	{
		return m_forme;
	}

	@Override
	public String toString() {
		String chaine=getSemantique()+getDefinition()+"\n";
		for(int i=0;i<m_val.size();i++)
			chaine+=m_val.get(i).toString()+"\n";
		return chaine+"\n";
	}

	/**
	 * @return la couleur de la serie
	 */
	public Paint getColor() {
		return m_couleur;
	}

	/**
	 * fixe la couleur de la série
	 * @param couleur
	 */
	public void setColor(Color couleur){m_couleur=couleur;}

	/**
	 * @param m_gestionnaire : le gestionnaire qui connait le statut des données
	 * @return vrai si la serie est entièrement fantome
	 */
	public boolean isFantome(Gestionnaire m_gestionnaire) {
		return m_gestionnaire.isFantome(m_val);
	}

	/**
	 * @return the m_definition
	 */
	public Info getDefinition() {
		return m_definition;
	}

	/**
	 * @return the m_semantique
	 */
	public String getSemantique() {
		return m_semantique;
	}

	/**
	 * @return boolean
	 */
	public Boolean isAffLegende() {
		return isAffLegende;
	}
	
	/**
	 * @param aff
	 */
	public void setAffLegende(boolean aff){
		isAffLegende=aff;
	}

	/**
	 * @param forme
	 */
	public void setForme(Shape forme) {
		m_forme=forme;
	}
}
