package dyn;

import ihm.Gestionnaire;
import ihm.Habilleur;

import java.util.Iterator;
import java.util.Vector;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.renderer.xy.XYBubbleRendererDynGraph;
import org.jfree.data.xy.DefaultXYZDataset;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYZDataset;

import don.Continue;
import don.DonneeID;
import don.Individu;
import don.IndividuMoyen;
import don.Modalite;
import drawing.Series;

/** Classe qui correspond à un nuage de points habillage par variable continue sous forme de bulles (Bubble)
 * 
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class Bubble extends NuagePointHabille
{	
	/** constructor 
	 * @param gest : le Gestionnaire qui commande le graphe 
	 * @param hab 
	 * @param titreGraphe : le titre du graphe 
	 * @param titreX : le titre de l'axe X 
	 * @param titreY : le titre de l'axe Y
	 */
	public Bubble(Gestionnaire gest,Habilleur hab, String titreGraphe,String titreX,String titreY) {
		super(gest,hab,titreGraphe,titreX,titreY);
	}

	@Override
	protected JFreeChart createChart(XYDataset dataset)
	{
		if(dataset instanceof XYZDataset) {
			JFreeChart chart = ChartFactory.createBubbleChart(getTitreGraphe(),
					getTitreX(), getTitreY(), (XYZDataset)dataset, getOrientation(), m_gestionnaire.isAffLegende(),
					false, false, new XYBubbleRendererDynGraph());
			return chart;
		}else return null;
	}

	@Override
	protected XYDataset construireSeries()
	{
		setCanvas();
		m_data = new DefaultXYZDataset();
		Iterator<Series> ite=getDonnee().iterator();
		Vector<Series> partiels=new Vector<Series>();
		double[][] don;
		int taille=0;
		while(ite.hasNext())
		{
			Series s=ite.next();
			String cde=s.getSemantique();
			//construction du tableau de coordonnees
			don=new double[3][s.getNbDonneeID()];
			int i=0;
			int nb=0;
			for(;i<s.getNbDonneeID();i++)
			{
				DonneeID donneeID = s.getDonneeID(i);
				if(!m_gestionnaire.isFantome(donneeID))
				{//on ne prend que les donnees qui ne sont pas fantomes
					don[0][nb]=donneeID.getCoord(m_gestionnaire.getAxeX());
					don[1][nb]=donneeID.getCoord(m_gestionnaire.getAxeY());
					m_canvas.addPoint(donneeID);
					if(donneeID instanceof IndividuMoyen && m_gestionnaire.isAffPartiels(donneeID))
					{
						Series part=new Series();
						IndividuMoyen ind=(IndividuMoyen)donneeID;
						for(int j=0;j<ind.getNbBranches();j++)
						{
							part.addDonneeID(ind.getBranches().get(j));
							taille++;
						}
						partiels.add(part);
					}
					if(donneeID instanceof Individu)
					{
						Continue habillageInd = m_habilleur.getHabillageContinu(Habilleur.INDIVIDUS);
						if(habillageInd!=null && m_habilleur.getHabillageStyleContinu(Habilleur.INDIVIDUS)==Habilleur.STYLE_BULLES){
							//double z=m_gestionnaire.getDon().getValeur((Individu)s.getDonneeID(i),habillageInd);
							don[2][nb]=m_habilleur.getZ(donneeID,habillageInd);
						}else{
							don[2][nb]=0.05;
						}
					}					
					else if(donneeID instanceof Modalite){
						Continue habillageMod = m_habilleur.getHabillageContinu(Habilleur.MODALITES);
						if(habillageMod!=null && m_habilleur.getHabillageStyleContinu(Habilleur.MODALITES)==Habilleur.STYLE_BULLES){
							/*double z=m_gestionnaire.getDon().getValeur((Modalite)s.getDonneeID(i),
									((ContinueArtificiellePourMod)habillageMod));*/
							don[2][nb]=m_habilleur.getZ(donneeID,habillageMod);
						}else{
							don[2][nb]=0.05;
						}
					}
					nb++;
				}
			}
			//reconstruction d'un tableau a la bonne taille
			double[][] donF=new double[3][nb];
			for(int j=0;j<nb;j++)
			{
				donF[0][j]=don[0][j];
				donF[1][j]=don[1][j];
				donF[2][j]=don[2][j];
			}
			//Construction de la serie, si elle n'est pas vide
			if(i!=0)((DefaultXYZDataset)m_data).addSeries(cde, donF);
		}
		don=new double[3][taille];
		int cpt=0;
		for(int i=0;i<partiels.size();i++)
		{
			m_partiel=true;
			for(int j=0;j<partiels.get(i).getNbDonneeID();j++)
			{
				don[0][cpt]=partiels.get(i).getDonneeID(j).getCoord(m_gestionnaire.getAxeX());
				don[1][cpt]=partiels.get(i).getDonneeID(j).getCoord(m_gestionnaire.getAxeY());
				don[2][cpt]=0;
				m_canvas.addPoint(partiels.get(i).getDonneeID(j));
				cpt++;
			}
		}
		if(partiels.size()!=0)((DefaultXYZDataset)m_data).addSeries("partiel", don);
		return m_data;
	}
}
