package dyn;

import ihm.Gestionnaire;
import ihm.Habilleur;

import java.util.Iterator;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.data.xy.DefaultXYDataset;
import org.jfree.data.xy.XYDataset;

import drawing.Series;

/** Classe qui correspond aux graphes de type nuage de points 
 * 
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class NuagePoint extends Graphe
{
	/** constructeur 
	 * @param gest : le Gestionnaire qui commande le graphe 
	 * @param hab 
	 * @param titreGraphe : le titre du graphe 
	 * @param titreX : le titre de l'axe X 
	 * @param titreY : le titre de l'axe Y
	 */
	public NuagePoint(Gestionnaire gest, Habilleur hab, String titreGraphe,String titreX,String titreY) {
		super(gest, hab, titreGraphe,titreX,titreY);
	}

	@Override
	protected JFreeChart createChart(XYDataset dataset)
	{
		if(m_gestionnaire.getType()==Gestionnaire.GROUPES)
			return ChartFactory.createGroupPlot(getTitreGraphe(), getTitreX(),
					getTitreY(), dataset, getOrientation(), m_gestionnaire.isAffLegende(), false, false);
		else
			return ChartFactory.createScatterPlot(getTitreGraphe(), getTitreX(),
					getTitreY(), dataset, getOrientation(), m_gestionnaire.isAffLegende(), false, false);
	}

	@Override
	protected XYDataset construireSeries()
	{
		setCanvas();
		m_data = new DefaultXYDataset();
		Iterator<Series> ite=getDonnee().iterator();
		double[][] don;
		while(ite.hasNext())
		{
			Series s=ite.next();
			String cde=s.getSemantique();
			//construction du tableau de coordonnées
			don=new double[2][s.getNbDonneeID()];
			int i=0;
			int nb=0;
			for(;i<s.getNbDonneeID();i++)
			{
				if(!m_gestionnaire.isFantome(s.getDonneeID(i)))
				{//on ne prend que les données qui ne sont pas fantomes
					don[0][nb]=s.getDonneeID(i).getCoord(m_gestionnaire.getAxeX());
					don[1][nb]=s.getDonneeID(i).getCoord(m_gestionnaire.getAxeY());
					m_canvas.addPoint(s.getDonneeID(i));
					nb++;
				}
			}
			//reconstruction d'un tableau é la bonne taille
			double[][] donF=new double[2][nb];
			for(int j=0;j<nb;j++)
			{
				donF[0][j]=don[0][j];
				donF[1][j]=don[1][j];
			}
			//Construction de la série, si elle n'est pas vide
			if(nb!=0)((DefaultXYDataset)m_data).addSeries(cde, donF);
		}
		return m_data;
	}

}
