package dyn;

import java.util.Vector;
import org.jfree.chart.annotations.XYIDAnnotation;
import org.jfree.chart.annotations.XYLineAnnotation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.AbstractXYItemRenderer;

import don.Continue;
import don.DonneeID;
import don.DonneeMoyenne;
import ihm.Gestionnaire;
import ihm.Habilleur;

/** Classe qui va correspondre aux graphes de type nuages de points, 
 * habillés avec des variables continues ou avec des individus partiels 
 * 
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public abstract class NuagePointHabille extends NuagePoint {

	/** permet de savoir si on doit afficher les points partiels */
	protected boolean m_partiel=false;

	/** constructeur 
	 * @param gest : le Gestionnaire qui commande le graphe 
	 * @param hab 
	 * @param titreGraphe : le titre du graphe 
	 * @param titreX : le titre de l'axe X 
	 * @param titreY : le titre de l'axe Y
	 */
	public NuagePointHabille(Gestionnaire gest, Habilleur hab, String titreGraphe, String titreX, String titreY) {
		super(gest, hab, titreGraphe, titreX, titreY);
	}

	@Override
	public void affAnnotation(Vector<DonneeID> liste)
	{
		setCanvas();
		XYPlot plot = (XYPlot) m_gestionnaire.getPanel().getChart().getPlot();
		AbstractXYItemRenderer renderer = (AbstractXYItemRenderer) plot.getRenderer();
		if(m_gestionnaire.getCanvas()!=null)
			m_gestionnaire.getCanvas().clearAnnot();
		renderer.removeAnnotations();
		if(m_data.getSeriesCount()!=0)
			trace(renderer,liste);
	}

	/** Trace les branches des points partiels et affiche les annotations
	 * @param renderer : le Renderer auquel on va donner les infos
	 * @param obj : la liste des données à afficher
	 */
	private void trace(AbstractXYItemRenderer renderer,Vector<DonneeID> obj)
	{
		for (DonneeID id : obj)
		{
			double x,y;
			Continue axeX=m_gestionnaire.getAxeX();
			Continue axeY=m_gestionnaire.getAxeY();
			x=id.getCoord(axeX);
			y=id.getCoord(axeY);
			XYIDAnnotation annot;
			annot = new XYIDAnnotation(id.getAnnotation(),id.getXannot(axeX,axeY),id.getYannot(axeX,axeY), x, y,getCouleur(id));
			m_habilleur.setFont(annot, id);
			if(id instanceof DonneeMoyenne && m_gestionnaire.isAffPartiels(id))
				tracePartiel(renderer,(DonneeMoyenne) id); //@trovalet 2012-03-23
				
			annot.setTextAnchor(id.getAnchor());
			if(m_gestionnaire.isAnnotations(id) && !m_gestionnaire.isFantome(id))
			{
				renderer.addAnnotation(annot);
				m_canvas.addAnnot(annot);
			}
		}
	}


	@Override
	public void renderer()
	{
		super.renderer();
		XYPlot plot = (XYPlot) m_gestionnaire.getPanel().getChart().getPlot();
		AbstractXYItemRenderer renderer=
				(AbstractXYItemRenderer)plot.getRenderer();
		if(m_data.getSeriesCount()>0 && m_partiel)
			renderer.setSeriesVisible(m_data.getSeriesCount()-1,false);
	}

	/**
	 * Trace un individu moyen et ses individus partiels, et stocke les annotations
	 * trace une ligne entre individu moyen et ses annotations partiels (+ affiche annotations partielles)
	 * @param renderer
	 * @param donneeMoyenne
	 */
	private void tracePartiel(AbstractXYItemRenderer renderer, DonneeMoyenne donneeMoyenne)
	{
		if (donneeMoyenne instanceof DonneeID) {
			DonneeID donnee = (DonneeID) donneeMoyenne;
			Continue axeX=m_gestionnaire.getAxeX();
			Continue axeY=m_gestionnaire.getAxeY();
			if(!m_gestionnaire.isFantome(donnee))
//				System.out.println("nombre de branches : "+donneeMoyenne.getNbBranches());
			{
				for(int i=0;i<donneeMoyenne.getNbBranches();i++)
				{
					DonneeID partiel=donneeMoyenne.getBranches().get(i);
//					System.out.println("partiel en cours : "+partiel);
//					System.out.println("X partiel : "+partiel.getCoord(axeX));
//					System.out.println("Y partiel : "+partiel.getCoord(axeY));					

					String texte=partiel.getAnnotation()+" ("+partiel.getGr().getAnnotation()+")";
//					System.out.println("groupe du partiel : "+partiel.getGr());
//					System.out.println("texte affiche pour le partiel : "+texte);
					XYIDAnnotation annot = new XYIDAnnotation(
							texte,
							partiel.getXannot(axeX, axeY),
							partiel.getYannot(axeX, axeY),
							partiel.getCoord(axeX),
							partiel.getCoord(axeY),
							getCouleur((DonneeID)donneeMoyenne)
							);
					m_habilleur.setFont(annot, partiel);
					annot.setTextAnchor(partiel.getAnchor());					
					if(m_gestionnaire.isAnnotations(donnee)) {						
						renderer.addAnnotation(annot);						
						m_canvas.addAnnot(annot);
					}	
					XYLineAnnotation line = new XYLineAnnotation(
							donnee.getCoord(m_gestionnaire.getAxeX()),
							donnee.getCoord(m_gestionnaire.getAxeY()),
							partiel.getCoord(m_gestionnaire.getAxeX()),
							partiel.getCoord(m_gestionnaire.getAxeY())
							);
					m_canvas.addBranche(line);
					renderer.addAnnotation(line);	
					
				}
			}
		}
	}
}
