package ihm;

import javax.swing.JPanel;

import java.awt.Component;
import java.awt.Frame;
import java.awt.BorderLayout;
import javax.swing.JDialog;
import javax.swing.JTabbedPane;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import javax.swing.JButton;
import java.awt.Insets;
import javax.swing.BoxLayout;

import don.DonneeParse;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 *  @author julien.durand@gmail.com
 *
 */
public class FenetreHabillage extends JDialog {


	private Gestionnaire m_gestionnaire;

	private static final long serialVersionUID = 1L;

	private JPanel jContentPane = null;
	private JTabbedPane jTabbedPane = null;
	private JPanel m_panelOkAnnuler = null;
	private JPanel m_panelSSOkAnnuler = null;
	private JButton m_btOk = null;
	private JButton m_btAnnuler = null;

	/**
	 * @param owner
	 * @param gestionnaire 
	 */
	public FenetreHabillage(Frame owner, Gestionnaire gestionnaire) {
		super(owner);
		m_gestionnaire = gestionnaire;
		initialize();
	}

	
	/**
	 * This method initializes this
	 */
	private void initialize() {
		this.setContentPane(getJContentPane());
		this.setTitle(IHM.getXmlLangue().getElement("Mhabillage"));
		this.pack();	
		this.setResizable(false);
		this.setLocationRelativeTo(null);
		this.setVisible(true);
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BoxLayout(getJContentPane(), BoxLayout.Y_AXIS));
			jContentPane.add(getJTabbedPane(), null);
			jContentPane.add(getPanelOkAnnuler(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jTabbedPane	
	 * 	
	 * @return javax.swing.JTabbedPane	
	 */
	private JTabbedPane getJTabbedPane() {
		if (jTabbedPane == null) {
			jTabbedPane = new JTabbedPane();
			if(m_gestionnaire.getType()==Gestionnaire.INDIVIDUS){
				if(m_gestionnaire.getDon().getType()!=DonneeParse.TypeAnalyse.CA)jTabbedPane.add(IHM.getXmlLangue().getElement("MtypeHabInd"), new PanelHabillageIndividus(m_gestionnaire));
				if(m_gestionnaire.isModalitesVisibles())jTabbedPane.add(IHM.getXmlLangue().getElement("MtypeHabMod"), new PanelHabillageModalites(m_gestionnaire));
			}else if(m_gestionnaire.getType()==Gestionnaire.VARIABLES){
				jTabbedPane.add(IHM.getXmlLangue().getElement("MtypeHabVar"), new PanelHabillageVariables(m_gestionnaire,Habilleur.CONTINUES));
			}else if(m_gestionnaire.getType()==Gestionnaire.AXES_PARTIELS){
				jTabbedPane.add(IHM.getXmlLangue().getElement("MtypeHabAxesPart"), new PanelHabillageVariables(m_gestionnaire,Habilleur.AXES_PARTIELS));				
			}else if(m_gestionnaire.getType()==Gestionnaire.DATAFRAME){
				jTabbedPane.add(IHM.getXmlLangue().getElement("MtypeHabInd"), new PanelHabillageIndividus(m_gestionnaire));
				if(m_gestionnaire.isModalitesVisibles())jTabbedPane.add(IHM.getXmlLangue().getElement("MtypeHabMod"), new PanelHabillageModalites(m_gestionnaire));
			}
		}
		return jTabbedPane;
	}

	/**
	 * This method initializes m_panelOkAnnuler	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPanelOkAnnuler() {
		if (m_panelOkAnnuler == null) {
			m_panelOkAnnuler = new JPanel();
			m_panelOkAnnuler.setLayout(new BorderLayout());
			m_panelOkAnnuler.add(getPanelSSOkAnnuler(), java.awt.BorderLayout.EAST);
		}
		return m_panelOkAnnuler;
	}

	/**
	 * This method initializes m_panelSSOkAnnuler	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPanelSSOkAnnuler() {
		if (m_panelSSOkAnnuler == null) {
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.insets = new Insets(8, 8, 8, 8);
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.insets = new Insets(8, 8, 8, 8);
			m_panelSSOkAnnuler = new JPanel();
			m_panelSSOkAnnuler.setLayout(new GridBagLayout());
			m_panelSSOkAnnuler.add(getBtOk(), gridBagConstraints6);
			m_panelSSOkAnnuler.add(getBtAnnuler(), gridBagConstraints7);
		}
		return m_panelSSOkAnnuler;
	}

	/**
	 * This method initializes m_btOk	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getBtOk() {
		if (m_btOk == null) {
			m_btOk = new JButton();
			m_btOk.setText(IHM.getXmlLangue().getElement("Lok"));
			m_btOk.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					Component[] tab = jTabbedPane.getComponents();
					for (Component component : tab) {
						if (component instanceof PanelHabillage) {
							PanelHabillage panel = (PanelHabillage) component;
// version 1.05_light
//							if(m_gestionnaire.getDon().getType()!=DonneeParse.TypeAnalyse.PCA){
//								m_gestionnaire.setHabillage(panel.getType(),panel.getHabillageDiscret());
//							}
//							m_gestionnaire.setHabillage(panel.getType(),panel.getHabillageContinu(),panel.getStyleHabillageContinu());
// fin version 1.05_light
// version 1.05_reverse
							if(m_gestionnaire.getDon().getType() == don.DonneeParse.TypeAnalyse.PCA && m_gestionnaire.getType() == Gestionnaire.VARIABLES)
	                        {
	                            m_gestionnaire.setHabillage(panel.getType(), panel.getHabillageContinu(), panel.getStyleHabillageContinu());
	                        } else
	                        {
// version 1.0	                        	
	                            m_gestionnaire.setHabillage(panel.getType(), panel.getHabillageDiscret());
	                            m_gestionnaire.setHabillage(panel.getType(), panel.getHabillageContinu(), panel.getStyleHabillageContinu());
// fin version 1.0
	                        }
// fin version 1.05_reverse
						}
					}
					m_gestionnaire.sauverAnnotations();
					m_gestionnaire.updateGraphe();
					getJDialog().dispose();	
				}
			});
		}
		return m_btOk;
	}

	/**
	 * This method initializes m_btAnnuler	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getBtAnnuler() {
		if (m_btAnnuler == null) {
			m_btAnnuler = new JButton();
			m_btAnnuler.setText(IHM.getXmlLangue().getElement("Lannuler"));
			m_btAnnuler.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					getJDialog().dispose();	};
			});
		}
		return m_btAnnuler;
	}

	protected JDialog getJDialog(){
		return this;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
