package ihm;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListModel;

import don.AxePartiel;
import don.ContinueAnalysee;
import don.ContinueMoyenne;
import don.DonneeID;
import don.DonneeParse;
import don.Groupe;
import don.IndividuMoyen;
import don.Modalite;
import don.Nominale;

/** Fenêtre de selection à partir d'une liste
 * 
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class FenetreSelection extends JDialog
{
	private static final long serialVersionUID = 1L;

	/** String localisée pour "individu" (initialisée à la création de la fenêtre) */
	private final String indiv=IHM.getXmlLangue().getElement("Lindiv");

	/** String localisée pour "modalite active" (initialisée à la création de la fenêtre) */
	private final String modActif=IHM.getXmlLangue().getElement("LmodActive");
	
	/** String localisée pour "modalite illustrative" (initialisée à la création de la fenêtre) */
	private final String modIllustratif=IHM.getXmlLangue().getElement("LmodIllustrative");
	
	/** String localisée pour "individus moyens" (initialisée à la création de la fenêtre) */
	private final String indivMoyens=IHM.getXmlLangue().getElement("LindivMoyens");
	
	/** String localisée pour "individus partiels" (initialisée à la création de la fenêtre) */
	private final String indivPartiels=IHM.getXmlLangue().getElement("LindivPartiels");
	
	/** String localisée pour "nom groupes" (initialisée à la création de la fenêtre) */
	private final String nomGroupes=IHM.getXmlLangue().getElement("LnomGroupes");

	/** String localisée pour "variable" (initialisée à la création de la fenêtre) */
	private final String var=IHM.getXmlLangue().getElement("Lvar");

	/** String localisée pour "groupe" (initialisée à la création de la fenêtre) */
	private final String groupe=IHM.getXmlLangue().getElement("Lgroupe");

	/** String localisée pour "axe partiel" (initialisée à la création de la fenêtre) */
	private final String axesPart=IHM.getXmlLangue().getElement("CaxePart"); 

	/** éléments graphique  */

	/** conteneur */
	private JPanel m_contentPane = null;

	/** panneau liste */
	private JPanel m_panelListe = null;

	/** panneau ok-annuler */
	private JPanel m_panelOKAnnuler = null;

	/** label types */
	private JLabel m_lblTypes = null;
	
	private JCheckBox m_checkTriModalites = null;

	/** bouton ok */
	private JButton m_btOk = null;

	/** bouton annuler */
	private JButton m_btAnnuler = null;

	/** liste des selections possibles */
	private JList m_lstPossibles = null;  

	/** liste des selectionnes */
	private JList m_lstChoisis = null;
	
	
	/** les differents tris possibles */
	private boolean isTriModaliteChoisi=false;	

	
	/** bouton prendre 1 */
	private JButton m_btPrendre1 = null;

	/** bouton prendre tous */
	private JButton m_btPrendreTous = null;

	/** bouton enlever tous */
	private JButton m_btEnleverTous = null;

	/** bouton enlever 1 */
	private JButton m_btEnlever1 = null;

	/** combobox types */
	private JComboBox m_cmbTypes = null;

	/** attributs */

	/** le Gestionnaire */
	private Gestionnaire m_gestionnaire;

	/** le jeu de données */
	private DonneeParse m_donneeParse;

	

	/** Constructeur
	 * @param owner
	 * @param parse 
	 * @param gest 
	 */
	public FenetreSelection(IHM owner, DonneeParse parse, Gestionnaire gest)
	{
		super(owner);
		m_gestionnaire=gest;
		m_donneeParse=parse;
		initialize();
		setResizable(false);
		setLocationRelativeTo(null);
		setVisible(true);
	}

	/** This method initializes this
	 */
	private void initialize()
	{
		this.setSize(441, 344);
		this.setTitle(IHM.getXmlLangue().getElement("Tselect"));
		this.setContentPane(getJContentPane());
		fixerListe(m_lstChoisis, m_gestionnaire.getSelectionnes());
		mettreAJourPossibles((String)m_cmbTypes.getSelectedItem());
	}

	/** This method initializes m_contentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane()
	{
		if (m_contentPane == null)
		{
			m_contentPane = new JPanel();
			m_contentPane.setLayout(new BorderLayout());
			m_contentPane.add(getJPanelListes(), BorderLayout.CENTER);
			m_contentPane.add(getJPanelOkAnnuler(), BorderLayout.SOUTH);
		}
		return m_contentPane;
	}
	
	

	/** This method initializes m_checkTriModalites
	 * 
	 * @return la checkbox du tri par modalites
	 */
	public JCheckBox getM_checkTriModalites() {
		if (m_checkTriModalites ==null) {
			m_checkTriModalites = new JCheckBox(IHM.getXmlLangue().getElement("LtriModalites"));
			m_checkTriModalites.setSelected(false);
			m_checkTriModalites.setEnabled(false);
			m_checkTriModalites.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if(m_checkTriModalites.isSelected()) {
						setTriModaliteChoisi(true);
						mettreAJourPossibles();
					}
                    else setTriModaliteChoisi(false);
					mettreAJourPossibles();
				}				
			});	
		}		
		return m_checkTriModalites;
	}



	/** This method initializes m_panelListe	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelListes()
	{
		if (m_panelListe == null)
		{
			// choix du type
			GridBagConstraints gridBagConstraintsCombo = new GridBagConstraints();
			gridBagConstraintsCombo.gridx = 1;
			gridBagConstraintsCombo.gridy = 0;
			gridBagConstraintsCombo.insets = new Insets(4, 8, 4, 8);
			gridBagConstraintsCombo.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraintsCombo.weightx = 1.0;
			gridBagConstraintsCombo.gridwidth = 1;
			gridBagConstraintsCombo.gridheight = 1;
			

			GridBagConstraints gridBagConstraintsLabel = new GridBagConstraints();
			gridBagConstraintsLabel.gridx=0;
			gridBagConstraintsLabel.gridy=0;
			gridBagConstraintsLabel.insets = new Insets(8, 8, 8, 0);
			gridBagConstraintsLabel.weightx = 0.0;
			
			// fin choix du type			
			
			GridBagConstraints gridBagConstraintsCheckTri = new GridBagConstraints();
			gridBagConstraintsCheckTri.gridx=4;
			gridBagConstraintsCheckTri.gridy=0;

			GridBagConstraints gridBagConstraintsPanelVide1=new GridBagConstraints();
			gridBagConstraintsPanelVide1.gridx=2;
			gridBagConstraintsPanelVide1.gridy=1;
			gridBagConstraintsPanelVide1.weighty=2.0;    

			// bouton selection individuelle
			GridBagConstraints gridBagConstraintsPrendre1 = new GridBagConstraints();
			gridBagConstraintsPrendre1.gridx=2;
			gridBagConstraintsPrendre1.gridy=2;
			gridBagConstraintsPrendre1.weighty = 1.0;
			gridBagConstraintsPrendre1.insets= new Insets(0, 8, 0, 8);
			// fin bouton selection individuelle

			// bouton selection totale
			GridBagConstraints gridBagConstraintsPrendreTous = new GridBagConstraints();
			gridBagConstraintsPrendreTous.gridx = 2;
			gridBagConstraintsPrendreTous.gridy = 3; 
			gridBagConstraintsPrendreTous.weighty = 1.0;  
			gridBagConstraintsPrendreTous.insets= new Insets(0, 8, 0, 8);
			// fin bouton selection individuelle

			GridBagConstraints gridBagConstraintsPanelVide2=new GridBagConstraints();
			gridBagConstraintsPanelVide2.gridx=2;
			gridBagConstraintsPanelVide2.gridy=4;
			gridBagConstraintsPanelVide2.weighty=2.5;

			GridBagConstraints gridBagConstraintsEnleverTous = new GridBagConstraints();
			gridBagConstraintsEnleverTous.gridx = 2; 
			gridBagConstraintsEnleverTous.gridy = 5; 
			gridBagConstraintsEnleverTous.weighty = 1.0;  
			gridBagConstraintsEnleverTous.insets= new Insets(0, 8, 0, 8);

			GridBagConstraints gridBagConstraintsEnelever1 = new GridBagConstraints();
			gridBagConstraintsEnelever1.gridx = 2;  
			gridBagConstraintsEnelever1.gridy = 6;  
			gridBagConstraintsEnelever1.weighty = 1.0;  
			gridBagConstraintsEnelever1.insets= new Insets(0, 8, 0, 8);

			GridBagConstraints gridBagConstraintsPanelVide3=new GridBagConstraints();
			gridBagConstraintsPanelVide3.gridx=2;
			gridBagConstraintsPanelVide3.gridy=7;
			gridBagConstraintsPanelVide3.weighty=2.0;   

			GridBagConstraints gridBagConstraintsChoisis = new GridBagConstraints();
			gridBagConstraintsChoisis.fill = GridBagConstraints.BOTH;  
			gridBagConstraintsChoisis.weightx = 3.3; 
			gridBagConstraintsChoisis.weighty = 1.0; 
			gridBagConstraintsChoisis.gridx = 3; 
			gridBagConstraintsChoisis.gridy = 1; 
			gridBagConstraintsChoisis.insets = new Insets(8, 8, 8, 8);  
			gridBagConstraintsChoisis.gridheight = 7;  
			gridBagConstraintsChoisis.gridwidth = 2;

			GridBagConstraints gridBagConstraintsPossibles = new GridBagConstraints();
			gridBagConstraintsPossibles.fill = GridBagConstraints.BOTH; 
			gridBagConstraintsPossibles.weighty = 1.0; 
			gridBagConstraintsPossibles.gridheight = 7; 
			gridBagConstraintsPossibles.gridwidth=2;
			gridBagConstraintsPossibles.weightx = 2.0;  
			gridBagConstraintsPossibles.insets=new Insets(8, 8, 8, 8);
			gridBagConstraintsPossibles.gridx=0;
			gridBagConstraintsPossibles.gridy=1;

			m_panelListe = new JPanel();
			m_panelListe.setLayout(new GridBagLayout());

			m_lblTypes = new JLabel();
			m_lblTypes.setText(IHM.getXmlLangue().getElement("Ltypes")+" :");  
			JScrollPane jsChoisis = new JScrollPane(getJListChoisis());

			m_panelListe.add(m_lblTypes, gridBagConstraintsLabel);
			m_panelListe.add(getM_checkTriModalites(), gridBagConstraintsCheckTri);
			m_panelListe.add(getJComboBoxTypes(), gridBagConstraintsCombo);
			m_panelListe.add(jsChoisis, gridBagConstraintsChoisis);  
			m_panelListe.add(getJButtonPrendre1(), gridBagConstraintsPrendre1);
			
			JScrollPane jsPossibles =new JScrollPane(getJListPossibles());
			JPanel vide1=new JPanel();
			m_panelListe.add(jsPossibles, gridBagConstraintsPossibles);  
			m_panelListe.add(vide1,gridBagConstraintsPanelVide1);
			m_panelListe.add(getJButtonPrendreTous(), gridBagConstraintsPrendreTous); 

			JPanel vide2 = new JPanel();
			m_panelListe.add(vide2,gridBagConstraintsPanelVide2);
			m_panelListe.add(getJButtonEnleverTous(), gridBagConstraintsEnleverTous);  
			m_panelListe.add(getJButtonEnlever1(), gridBagConstraintsEnelever1);  

			JPanel vide3=new JPanel();
			m_panelListe.add(vide3,gridBagConstraintsPanelVide3);
		}
		return m_panelListe;
	}
	
	
	/** This method initializes m_panelOKAnnuler	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelOkAnnuler()
	{
		if (m_panelOKAnnuler == null)
		{
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.insets = new Insets(8, 0, 8, 0);  
			gridBagConstraints1.weightx = 1.0;  
			gridBagConstraints1.gridx = 4;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.insets = new Insets(8, 0, 8, 0); 
			gridBagConstraints.weightx = 1.0;  
			gridBagConstraints.gridx = 5;
			m_panelOKAnnuler = new JPanel();
			m_panelOKAnnuler.setLayout(new BorderLayout());  
			JPanel ssPanelOkAnnuler = new JPanel();
			ssPanelOkAnnuler.add(getJButtonOK(), gridBagConstraints1);  
			ssPanelOkAnnuler.add(getJButtonAnnuler(), gridBagConstraints); 
			m_panelOKAnnuler.add(ssPanelOkAnnuler,BorderLayout.EAST);
		}
		return m_panelOKAnnuler;
	}

	/** This method initializes m_btOk	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonOK()
	{
		if (m_btOk == null)
		{
			m_btOk = new JButton();
			m_btOk.setText(IHM.getXmlLangue().getElement("Lok"));  
			m_btOk.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					ListModel list=m_lstChoisis.getModel();
					Collection<DonneeID> collec=new Vector<DonneeID>();
					for(int i=0;i<list.getSize();i++)
						collec.add((DonneeID)list.getElementAt(i));
					m_gestionnaire.rmSelectionnes();
					m_gestionnaire.addSelectionnes(collec);
					dispose();
				}
			});
		}
		return m_btOk;
	}

	/** This method initializes m_btAnnuler	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonAnnuler()
	{
		if (m_btAnnuler == null)
		{
			m_btAnnuler = new JButton();
			m_btAnnuler.setText(IHM.getXmlLangue().getElement("Lannuler"));  
			m_btAnnuler.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{dispose();}
			});
		}
		return m_btAnnuler;
	}

	/** This method initializes m_lstPossibles	
	 * 	
	 * @return javax.swing.JList	
	 */
	private JList getJListPossibles()
	{
		if (m_lstPossibles == null) {
			m_lstPossibles = new JList();
			m_lstPossibles.addMouseListener(new MouseAdapter() {
				public void mouseClicked(MouseEvent evt) {
					JList list = (JList)evt.getSource();
					if (evt.getClickCount() == 2) {          // Double-click
						// Get item index
						int index = list.locationToIndex(evt.getPoint());
						Object[] elements=new Object[1];
						elements[0]=m_lstPossibles.getModel().getElementAt(index);
						ajouterChoisis(elements);
					} 
				}
			});
		}
		m_lstPossibles.setCellRenderer(new RenduDonneeID(m_gestionnaire));
		return m_lstPossibles;
	}

	/** This method initializes m_lstChoisis	
	 * 	
	 * @return javax.swing.JList	
	 */
	private JList getJListChoisis()
	{
		if (m_lstChoisis == null) {
			m_lstChoisis = new JList();
			m_lstChoisis.addMouseListener(new MouseAdapter() {
				public void mouseClicked(MouseEvent evt) {
					JList list = (JList)evt.getSource();
					if (evt.getClickCount() == 2) {          // Double-click
						// Get item index
						int index = list.locationToIndex(evt.getPoint());
						Object[] elements=new Object[1];
						elements[0]=m_lstChoisis.getModel().getElementAt(index);
						enleverChoisis(elements);
					} 
				}
			});
		}
		m_lstChoisis.setCellRenderer(new RenduDonneeID(m_gestionnaire));
		return m_lstChoisis;
	}

	/** This method initializes m_btPrendre1	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonPrendre1()
	{
		if (m_btPrendre1 == null)
		{
			m_btPrendre1 = new JButton();
			m_btPrendre1.setText(">");  
			m_btPrendre1.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					ajouterChoisis(m_lstPossibles.getSelectedValues());
				}
			});
		}
		return m_btPrendre1;
	}

	/** This method initializes m_btPrendreTous	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonPrendreTous()
	{
		if (m_btPrendreTous == null)
		{
			m_btPrendreTous = new JButton();
			m_btPrendreTous.setText(">>");  
			m_btPrendreTous.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					ListModel listeChoisis= m_lstChoisis.getModel();
					ListModel listePossibles = m_lstPossibles.getModel();
					SortedSet<Object> vect=new TreeSet<Object>(getComparateur());
					for (int i=0 ; i<listeChoisis.getSize(); i++)
						vect.add(listeChoisis.getElementAt(i));
					for (int i=0 ; i<listePossibles.getSize(); i++)
						vect.add(listePossibles.getElementAt(i));
					m_lstPossibles.setListData(new Vector<DonneeID>());
					m_lstChoisis.setListData(vect.toArray());
				}
			});
		}
		return m_btPrendreTous;
	}

	/** This method initializes m_btEnleverTous	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonEnleverTous()
	{
		if (m_btEnleverTous == null)
		{
			m_btEnleverTous = new JButton();
			m_btEnleverTous.setText("<<"); 
			m_btEnleverTous.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent arg0)
				{
					m_lstChoisis.setListData(new Vector<DonneeID>());
					mettreAJourPossibles((String)m_cmbTypes.getSelectedItem());
				}
			});
		}
		return m_btEnleverTous;
	}

	/** This method initializes m_btEnlever1	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonEnlever1()
	{
		if (m_btEnlever1 == null)
		{
			m_btEnlever1 = new JButton();
			m_btEnlever1.setText("<");  
			m_btEnlever1.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					enleverChoisis(m_lstChoisis.getSelectedValues());
				}
			});
		}
		return m_btEnlever1;
	}


	/** This method initializes m_cmbTypes	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getJComboBoxTypes()
	{
		if (m_cmbTypes == null)
		{
			Vector<String> tab=new Vector<String>();
//			System.out.println("type : "+m_gestionnaire.getType());
			switch (m_gestionnaire.getType())
			{
			case Gestionnaire.INDIVIDUS:
			{
				if(m_donneeParse.isIndActif() || m_donneeParse.isIndIllus()) tab.add(indiv);
//				if(m_donneeParse.isIndActif() tab.add(indivActif);
//				if(m_donneeParse.isIndIllus()) tab.add(indivIllustratif);
				//
				if((m_donneeParse.isModActif())) tab.add(modActif);
				if((m_donneeParse.isModIllus())) tab.add(modIllustratif);
//				if((m_donneeParse.isIndMoy())) tab.add(indivMoyens);
//				if((m_donneeParse.isIndPart())) tab.add(indivPartiels);
//				if((m_donneeParse.isGroup())) tab.add(nomGroupes);
//				if((m_donneeParse.isGroupeActif())) tab.add(nomGroupes);
				break;
			}
			case Gestionnaire.VARIABLES:
			{
				if(m_donneeParse.isVarActifNonOrtho() || m_donneeParse.isVarActifOrtho() || 
						m_donneeParse.isVarIllusNonOrtho() || m_donneeParse.isVarIllusOrtho() || m_donneeParse.isVarPart()) tab.add(var);
				break;
			}
			case Gestionnaire.GROUPES:
			{
				if(m_donneeParse.isGroupeActif() || m_donneeParse.isGroupeIllus())tab.add(groupe);
				break;
			}
			case Gestionnaire.AXES_PARTIELS:
			{
				if(m_donneeParse.isAxesPart())tab.add(axesPart);
				break;
			}
			case Gestionnaire.DATAFRAME:
			{
				if(m_donneeParse.isIndActif() || m_donneeParse.isIndIllus()) tab.add(indiv);
				if((m_donneeParse.isModActif())) tab.add(modActif);
				if((m_donneeParse.isModIllus())) tab.add(modIllustratif);
				break;
			}			
			default:
				break;
			}

			m_cmbTypes = new JComboBox(tab.toArray());
			m_cmbTypes.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent arg0)
				{
					String select=(String)m_cmbTypes.getSelectedItem();
					mettreAJourPossibles(select);					
					boolean isModalitesChoisi = (modActif.equals(select) || modIllustratif.equals(select));					
					if (isModalitesChoisi) {
						m_checkTriModalites.setEnabled(true);
					}
					else {m_checkTriModalites.setEnabled(false);}					
				}
			});
		}
		return m_cmbTypes;
	}


	/** Met a jour la Jlist des possibles selon le type passe en parametre
	 * @param type
	 */
	private void mettreAJourPossibles(String type)
	{fixerListe(m_lstPossibles,proposerPossibles(type));}
	
	private void mettreAJourPossibles() {
	String type = (String) m_cmbTypes.getSelectedItem();	
	fixerListe(m_lstPossibles,proposerPossibles(type));
		
	}

	/** Affiche les DonneeID qui peuvent etre selectionnees
	 * @param type
	 * @return la liste des DonneeID concernees par le type passe en parametre
	 */
	private Vector<? extends DonneeID> proposerPossibles(String type)
	{
		if(indiv.equals(type)) return proposerIndividus();
		else if(modActif.equals(type))return proposerModalitesActives();
		else if(modIllustratif.equals(type))return proposerModalitesIllustratives();
		else if(indivMoyens.equals(type)) return proposerIndividusMoyens(); // strictement identique à indiv ...
//		else if(indivPartiels.equals(type)) return proposerIndividusPartiels();
		else if(nomGroupes.equals(type)) return proposerNomGroupes(); // doit etre specifique aux points partiels
		else if(var.equals(type))return proposerVariables();
		else if(groupe.equals(type))return proposerGroupes();
		else if(axesPart.equals(type))return proposerAxesPart();
		return new Vector<DonneeID>();
	}
	
	/** @return la liste des individus */
	protected Vector<IndividuMoyen> proposerIndividus()
	{
		Vector<IndividuMoyen> vect =m_donneeParse.getIndividusMoyens();
		Vector<IndividuMoyen> res = new Vector<IndividuMoyen>();
		for (IndividuMoyen ind : vect)
		{
			if(!isChoisi(ind)) {
				res.add(ind);
			}
		}
		return res;

	}

	private Vector<IndividuMoyen> proposerIndividusMoyens() {
		Vector<IndividuMoyen> vect = m_donneeParse.getIndividusMoyens();
		Vector<IndividuMoyen> res = new Vector<IndividuMoyen>();
		for (IndividuMoyen individuMoyen : vect) {
			if(!isChoisi(individuMoyen)) {
				res.add(individuMoyen);	
			}		
		}
		return res;
	}
	
//	private Vector<IndividuPartiel> proposerIndividusPartiels() {		
////		SortedSet<IndividuPartiel> vect= new TreeSet<IndividuPartiel>(getComparateur());
//		Vector<IndividuPartiel> vect =m_donneeParse.getIndividusPartiels();
//		Vector<IndividuPartiel> res=new Vector<IndividuPartiel>();
////		for (IndividuPartiel individuPartiel : m_donneeParse.getIndividusPartiels()) // null pour le moment :(
//		for (IndividuPartiel individuPartiel : vect)
//			if(!isChoisi(individuPartiel)) {
//				res.add(individuPartiel);
//		}		
//		res.addAll(vect);
//		return res;
//	}
	

	/** @return la liste des groupes */
	protected Vector<Groupe> proposerNomGroupes()
	{
		Vector<Groupe> vect =m_donneeParse.getGroupes();
		Vector<Groupe> res = new Vector<Groupe>();
		for (Groupe groupe : vect)
		{
			if(!isChoisi(groupe) && groupe.getNum()!=0){
				if(groupe.isActive()){
					res.add(groupe);
				}
			}
		}
		return res;
	}

	/** @return la liste des groupes */
	protected Vector<Groupe> proposerGroupes()
	{
		Vector<Groupe> vect =m_donneeParse.getGroupes();
		Vector<Groupe> res = new Vector<Groupe>();
		for (Groupe groupe : vect)
		{
			if(!isChoisi(groupe) && groupe.getNum()!=0){
				if(groupe.isActive()){
					res.add(groupe);
				}
			}
		}
		return res;
	}
	
	/** @return la liste des variables */
	protected Vector<ContinueMoyenne> proposerVariables()
	{
		Vector<ContinueMoyenne> vect =m_donneeParse.getVariablesMoyennes();
		Vector<ContinueMoyenne> res = new Vector<ContinueMoyenne>();
		for (ContinueMoyenne var : vect)
		{
			if(!isChoisi(var)) {
				res.add(var);
			}
		}
		return res;

	}

	/** @return la liste des axes partiels */
	protected Vector<ContinueAnalysee> proposerAxesPart()
	{
		Vector<AxePartiel> vect =m_donneeParse.getAxesPartiels();
		Vector<ContinueAnalysee> res = new Vector<ContinueAnalysee>();
		for (ContinueAnalysee var : vect)
		{
			if(!isChoisi(var)) {
				res.add(var);
			}
		}
		return res;

	}

	/** @return la liste des modalites */
	protected Vector<DonneeID> proposerModalitesActives()
	{
		SortedSet<DonneeID> vect= new TreeSet<DonneeID>(getComparateur());
		for (Nominale nom : m_donneeParse.getNominales())
			for (Modalite mod : nom.getModalites()) {
				if(!isChoisi(mod) && mod.isActive()) {
					vect.add(mod);
				}
			}		
		Vector<DonneeID> vect2=new Vector<DonneeID>();
		vect2.addAll(vect);		
		return vect2;

	}
	
	/** @return la liste des modalites */
	protected Vector<DonneeID> proposerModalitesIllustratives()
	{
		SortedSet<DonneeID> vect= new TreeSet<DonneeID>(getComparateur());
		for (Nominale nom : m_donneeParse.getNominales())
			for (Modalite mod : nom.getModalites()) {
				if(!isChoisi(mod) && !mod.isActive()) {
					vect.add(mod);
				}
			}		
		Vector<DonneeID> vect2=new Vector<DonneeID>();
		vect2.addAll(vect);		
		return vect2;

	}


	private boolean isChoisi(DonneeID don) {
		ListModel listeChoisis= m_lstChoisis.getModel();
		Vector<Object> vectChoisis = new Vector<Object>();
		for(int i=0; i<listeChoisis.getSize();i++)
			vectChoisis.add(listeChoisis.getElementAt(i));
		return vectChoisis.contains(don);

	}

	
	

	/** @return un comparator approprie pour trier les donnees */
	protected Comparator<Object> getComparateur() {
		return new Comparator<Object>() {
			public int compare(Object arg0, Object arg1) {
				String chaine0=null;
				String chaine1=null;
				RenduDonneeID r=new RenduDonneeID(m_gestionnaire);			
				
				boolean isModalite= ( (arg0 instanceof Modalite) && (arg1 instanceof Modalite) );					
				if (isTriModaliteChoisi && isModalite) {// alors on fait le tri (ascendant) sur la modalité et ensuite sur la variable
					Modalite mod0=(Modalite)arg0;
					Modalite mod1=(Modalite)arg1;
					chaine0=mod0.getAnnotation()+" : "+mod0.getNominale().getAnnotation();					
					chaine1=mod1.getAnnotation()+" : "+mod1.getNominale().getAnnotation();
				}							
				else {
					chaine0=((RenduDonneeID)r.getListCellRendererComponent(null, arg0, 0, true, false)).getText();
					chaine1=((RenduDonneeID)r.getListCellRendererComponent(null, arg1, 0, true, false)).getText();
				}	
				return chaine0.compareTo(chaine1);
			}
		};
	}
	


	/** affecte a la liste list les elements de vect
	 * @param list
	 * @param vect
	 */
	@SuppressWarnings("unchecked")
	private void fixerListe(JList list, Vector vect)
	{
		SortedSet set=new TreeSet(getComparateur());
		set.addAll(vect);
		list.setListData(set.toArray());
	}

	private void enleverChoisis(Object[] element) {
		ListModel listeChoisis= m_lstChoisis.getModel();
		ListModel listePossibles= m_lstPossibles.getModel();

		Vector<Object> vectChoisis = new Vector<Object>();
		for(int i=0; i<listeChoisis.getSize();i++)
			vectChoisis.add(listeChoisis.getElementAt(i));

		Vector<Object> vectPossibles = new Vector<Object>();
		for(int i=0; i<listePossibles.getSize();i++)
			vectPossibles.add(listePossibles.getElementAt(i));

		Vector<Object> enleves = new Vector<Object>();
		enleves.addAll(Arrays.asList(element));

		vectChoisis.removeAll(enleves);
		fixerListe(m_lstChoisis, vectChoisis);

		//on ne va remettre dans les possibilites que les donnees qui correspondent au type en cours
		Vector<? extends DonneeID> vectPossiblesInit=proposerPossibles((String)m_cmbTypes.getSelectedItem());
		enleves.retainAll(vectPossiblesInit);
		vectPossibles.addAll(enleves);
		fixerListe(m_lstPossibles,vectPossibles);
	}

	private void ajouterChoisis(Object[] elements) {
		ListModel listeChoisis= m_lstChoisis.getModel();
		ListModel listePossibles= m_lstPossibles.getModel();

		Vector<Object> vectChoisis = new Vector<Object>();
		for(int i=0; i<listeChoisis.getSize();i++)
			vectChoisis.add(listeChoisis.getElementAt(i));

		Vector<Object> vectPossibles = new Vector<Object>();
		for(int i=0; i<listePossibles.getSize();i++)
			vectPossibles.add(listePossibles.getElementAt(i));

		Vector<Object> nouveaux = new Vector<Object>();
		nouveaux.addAll(Arrays.asList(elements));

		vectChoisis.addAll(nouveaux);
		fixerListe(m_lstChoisis, vectChoisis);
		vectPossibles.removeAll(nouveaux);
		fixerListe(m_lstPossibles,vectPossibles);
	}

	/**
	 * @return si le tri est sur les modalites puis variables
	 */
	public boolean isTriModaliteChoisi() {
		return isTriModaliteChoisi;
	}

	/**
	 * @param isTriModaliteChoisi
	 */
	public void setTriModaliteChoisi(boolean isTriModaliteChoisi) {
		this.isTriModaliteChoisi = isTriModaliteChoisi;
	}

//	/**
//	 * main
//	 * @param args
//	 */
//	public static void main(String[] args) {
////		IHM.m_dir=new File("").getAbsolutePath()+"\\";
//		IHM.m_dir=new File("").getAbsolutePath()+"/";
////		System.out.println(IHM.m_dir);
//		IHM ihm=new IHM(new NullDonneeParse());
//		ihm.setGoByR(false);
////		ihm.getGest().ouvrir("K:/dynGraph/test/130707_MCA_a.ser");
//		ihm.getGest().ouvrir("/home/trovalet/workspace/dynGraph_unstable/test/2012-02-20/2012-02-20_new.tea.mca.ser");
//		FenetreSelection fen=new FenetreSelection(ihm,ihm.getDonnee(),ihm.getGest());
//		fen.setVisible(true);
//	}
}
