package ihm;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Vector;
import javax.swing.JFrame;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.annotations.XYIDAnnotation;
import outils.Fichier;
import criteres.Critere;
import criteres.CritereActif;
import criteres.CritereActifIllustratif;
import criteres.CritereAjout;
import criteres.CritereAppartient;
import criteres.CritereCompose;
import criteres.CritereGroupe;
import criteres.CritereHabGroupeContinue;
import criteres.CritereIndividu;
import criteres.CritereModNominale;
import criteres.CritereNominale;
import criteres.CritereRestriction;
import criteres.CritereVariableContinue;
import criteres.Trieur;
import don.Continue;
import don.ContinueAnalysee;
import don.ContinueMoyenne;
import don.Dimension;
import don.DonneeID;
import don.DonneeParse;
import don.Individu;
import don.IndividuMoyen;
import don.Modalite;
import don.ModaliteMoyenne;
import don.Nominale;
import don.NominaleArtificielleGroupe;
import don.NullDonneeParse;
import don.Partitionnable;
import don.Qualifiable;
import don.ValeurTest;
import don.Variable;
import don.DonneeParse.TypeAnalyse;
import drawing.Canvas;
import drawing.ConfAffichage;
import drawing.Registre;
import drawing.Series;
import dyn.Araignee;
import dyn.Bubble;
import dyn.Graphe;
import dyn.NuagePoint;
import dyn.Vecteur;
import exceptions.ExceptionIntegrite;

/**
 * @author michaelory@gmail.com
 * @author julien.durand.eu@gmail.com
 *
 */
public class Gestionnaire
{
	/**
	 * Le jeu de donnees sur lequel on travaille
	 */
	private DonneeParse m_don;


	/**
	 * Type de graphique ici=individu
	 */
	public static final int INDIVIDUS=0;

	/**
	 * Type de graphique ici=variable
	 */
	public static final int VARIABLES=1;

	/**
	 * Type de graphique ici=groupe
	 */
	public static final int GROUPES=2;

	/**
	 * Type de graphique ici= axes partiels
	 */
	public static final int AXES_PARTIELS = 3;


	/**
	 * Type de graphique ici= représentation d'un dataframe qcq
	 */
	public static final int DATAFRAME = 4;

	/**
	 * Nombre maximum de types de graphique
	 */
	public static final int NB_TYPES_MAX=5;



	/** 
	 * Le type de graphique courant 
	 */
	private int m_type;

	/**
	 * Les titres de graphe par type
	 */
	private String[] m_titres;

	/**
	 * Les titres de l'axe X par type
	 */
	private String[] m_titresX;

	/**
	 * Les titres de l'axe Y par type
	 */
	private String[] m_titresY;

	/** 
	 * L'interface graphique à laquelle le gestionnaire est lié
	 */
	private IHM m_interf;


	/**
	 * Objets selectionnes
	 */
	private Vector<DonneeID> m_selectionnes;

	/**
	 * L'axe sur X
	 */
	private ContinueAnalysee[] m_axeX;

	/**
	 * L'axe sur Y
	 */
	private ContinueAnalysee[] m_axeY;

	/**
	 * Faut-il afficher la grille ?
	 */
	private boolean m_affGrid;

	/**
	 * Le graphe sur lequel on travaille
	 */
	private Graphe m_graphe;

	/**
	 * les séries affichées
	 */
	private SortedSet<Series> series;

	/** 
	 * tableau qui dit pour chaque donnee si on affiche son annotation 
	 */
	private HashMap<DonneeID, Boolean> annotation;

	/** 
	 * tableau qui dit pour chaque donnee si elle est fantome
	 */
	private HashMap<DonneeID, Boolean> fantome;

	/** 
	 * tableau qui dit pour chaque donnee si on affiche ses partiels 
	 */
	private HashMap<DonneeID, Boolean> partiel;

	/** 
	 * les donnees actuellement visibles 
	 */
	private Vector<DonneeID> courante;

	/** 
	 * les donnees visibles ou que l'on pourrait rendre visible 
	 */
	private Vector<DonneeID> affichable;


	private boolean m_affLegende;


	private double[] rapport;


	private Habilleur m_habilleur_facto;
	private Habilleur m_habilleur_data;


	private boolean m_ortho;




	/**
	 * constructeur, initialise le gestionnaire avec des valeurs par defaut
	 * @param interf l'IHM à laquelle le gestionnaire est attaché
	 */
	public Gestionnaire(IHM interf)
	{
		m_don = interf.getDonneeParse();
		courante=new Vector<DonneeID>();
		affichable=new Vector<DonneeID>();
		m_interf = interf;
		m_selectionnes = new Vector<DonneeID>();
		setValDefaut();
	}


	/** 
	 * teste si tous les éléments sont de la valeur passé en paramétre 
	 */
	private boolean all(HashMap<DonneeID, Boolean> table,boolean val)
	{
		Iterator<Boolean> ite=table.values().iterator();
		while(ite.hasNext())
		{
			boolean t=ite.next();
			if(!(t==val))return false;
		}
		return true;
	}


	/**
	 * @param obj : la donnée concernée
	 * @return vrai si on affiche les partiels de la donnée
	 */
	public boolean isAffPartiels(DonneeID obj)
	{
		if(obj != null) {
			Iterator<DonneeID> ite=partiel.keySet().iterator();
			while(ite.hasNext())
			{
				DonneeID don=ite.next();
				if(don.equals(obj))return partiel.get(don);
			}
		}
		return false;
	}

	/**
	 * @return vrai si on affiche des points partiels, faux si on n'en affiche aucun
	 */
	public boolean isAffPartiels()
	{
		if(all(partiel,false))return false;
		else return true;

	}

	/**
	 * @param obj : les données concernées
	 * @return vrai si on affiche les points partiels de toutes les données
	 */
	private boolean isAffPartiels(Vector<DonneeID> obj)
	{
		Iterator<DonneeID> iter = obj.iterator();
		while(iter.hasNext()){
			if(!partiel.get(iter.next()))return false;
		}
		return true;
	}

	/**
	 * dit si les données qui vont être affectées (les données sélectionnées, toutes les données si aucune n'est sélectionnée) ont leurs partiels visibles
	 * @return un boolean (vrai/faux)
	 */
	public boolean isAffPartielsDonnesEnCours() {
		if(getSelectionnes().size()==0)
			return isAffPartiels(getAffichable());
		else
			return isAffPartiels(getSelectionnes());
	}

	/**
	 * indique s'il faut afficher ou non les points partiels 
	 * des données passées en paramétre
	 * @param obj  : les données concernées
	 * @param val (oui/non)
	 */
	public void setAffPartiels(Vector<DonneeID> obj,boolean val)
	{
		sauverAnnotations();
		for (DonneeID donnee : obj)partiel.put(donnee, val);
		updateGraphe();
	}

	/**
	 * indique s'il faut afficher ou non les points partiels 
	 * des données en cours
	 * @param val (oui/non)
	 */
	public void setAffPartielsDonnesEnCours(boolean val) {
		if(getSelectionnes().size()==0)
			setAffPartiels(getAffichable(),val);
		else
			setAffPartiels(getSelectionnes(),val);
	}

	/**
	 * fait afficher les partiels des données passées en paramétre
	 * @param obj : les données concernées
	 */
	@Deprecated
	public void setAffPartiels(Vector<DonneeID> obj)
	{setAffPartiels(obj,true);}

	/**
	 * @return  vrai si la grille doit étre affichée, faux sinon 
	 */
	public boolean isAffGrid(){return m_affGrid;}

	/**
	 * Fixe l'affichage de la grille
	 * @param b : vrai si la grille doit étre affichée, faux sinon
	 */
	public void setAffGrid(boolean b)
	{
		m_affGrid=b;
		m_graphe.affGrid();
	}

	/**
	 * @return vrai si des annotations sont affichées, faux sinon
	 */
	public boolean isAnnotations()
	{

		if(all(annotation,false))return false;
		else return true;

	}

	/**
	 * @param obj : les données concernées
	 * @return vrai si des annotations sont affichées, faux sinon
	 */
	public boolean isAnnotations(Vector<DonneeID> obj)
	{
		if(obj != null) {
			Iterator<DonneeID> iter = obj.iterator();
			while(iter.hasNext()){
				if(!annotation.get(iter.next()))return false;
			}
		}
		return true;
	}

	/**
	 * dit si les données qui vont être affectées (les données sélectionnées, toutes les données si aucune n'est sélectionnée) ont leurs annotations visibles
	 * @return un boolean (vrai/faux)
	 */
	public boolean isAnnotationsDonnesEnCours() {
		if(getSelectionnes().size()==0)
			return isAnnotations(getAffichable());
		else
			return isAnnotations(getSelectionnes());
	}

	/**
	 * @param obj : la donnée concernée
	 * @return vrai si on affiche les annotations de la donnée passée en paramétre, faux sinon
	 */
	public boolean isAnnotations(DonneeID obj)
	{
		return annotation.get(obj) && (getType()==Gestionnaire.DATAFRAME || (getType()!=Gestionnaire.DATAFRAME && !getHabilleur().isSousSeuilAffinage(obj)));
	}

	/**
	 * Fixe l'affichage de l'annotation d'une donnée
	 * @param obj : la donnée concernée
	 * @param val : vrai s'il faut afficher son annotation, faux sinon
	 */
	public void setAnnotations(DonneeID obj,boolean val) {
		Vector<DonneeID> vect= new Vector<DonneeID>();
		vect.add(obj);
		setAnnotations(vect, val);
	}

	/**
	 * Fixe l'affichage de l'annotation des données
	 * @param val : vrai s'il faut afficher les annotations, faux sinon
	 */
	public void setAnnotationsDonnesEnCours(boolean val) {
		if(getSelectionnes().size()==0)
			setAnnotations(getAffichable(),val);
		else
			setAnnotations(getSelectionnes(),val);
	}

	/**
	 * Fixe l'affichage des annotations pour des données
	 * @param obj : les données concernées
	 * @param val : vrai s'il faut afficher les annotations, faux sinon
	 */
	public void setAnnotations(Vector<DonneeID> obj,boolean val)
	{
		sauverAnnotations();
		for (DonneeID donnee : obj)annotation.put(donnee, val);
		m_graphe.affAnnotation();
	}

	/**
	 * dit si les données sont toutes en fantome
	 * @return un boolean (vrai/faux)
	 */
	public boolean isFantome()
	{
		return courante.size()==0;
	}

	/**
	 * dit si les données contiennent un fantome ou plus
	 * @return un boolean (vrai/faux)
	 */
	public boolean isContientFantome()
	{
		if(all(fantome,false))return false;
		else return true;

	}

	/**
	 * dit si les données contiennent un fantome ou plus
	 * @param obj 
	 * @return un boolean (vrai/faux)
	 */
	public boolean isContientFantome(Vector<DonneeID> obj)
	{
		if(fantome!=null && obj != null) {
			if(obj.size()==0)return isFantome();
			Iterator<DonneeID> iter = obj.iterator();
			while(iter.hasNext()){
				if(fantome.get(iter.next()))return true;
			}		
			return false;
		}else return false;
	}

	/**
	 * dit si les données qui vont être affectées (les données sélectionnées, toutes les données si aucune n'est sélectionnée) contiennent des fantomes
	 * @return un boolean (vrai/faux)
	 */
	public boolean isContientFantomeDonnesEnCours() {
		if(getSelectionnes().size()==0)
			return isContientFantome(getAffichable());
		else
			return isContientFantome(getSelectionnes());
	}

	/**
	 * @param obj  : les données concernées. 
	 * Si obj est vide, la réponse vaut pour les données courantes
	 * @return vrai si les données sont toutes fantomes
	 */
	public boolean isFantome(Vector<DonneeID> obj)
	{
		if(fantome!=null) {
			if(obj.size()==0)return isFantome();
			Iterator<DonneeID> iter = obj.iterator();
			while(iter.hasNext()){
				if(!fantome.get(iter.next()))return false;
			}		
			return true;
		}else return false;
	}

	/**
	 * @param obj : la donnée concernée
	 * @return vrai si la donnée est en fantome, faux sinon
	 */
	public boolean isFantome(DonneeID obj)
	{
		if(fantome.get(obj)==null)return false;
		else return fantome.get(obj);
	}

	/**
	 * Fixe le masquage des données
	 * @param obj : les données concernées
	 * @param val : vrai si les donnée doivent étre en fantome, faux sinon
	 */
	private void setFantome(Vector<DonneeID> obj,boolean val)
	{
		sauverAnnotations();
		if(!isIndividusVisibles())getHabilleur().supprimerHabillage(Habilleur.INDIVIDUS);
		for (DonneeID donnee : obj) {
			fantome.put(donnee, val);
		}
		if(val==true)	{//On enlève les fantomes de la sélection
			m_selectionnes.removeAll(obj);
		}
		updateGraphe();
	}

	/**
	 * @return si des individus sont actuellement visibles
	 */
	public boolean isIndividusVisibles() {
		for (DonneeID don : courante) {
			if(don instanceof Individu) return true;
		}
		return false;
	}


	/**
	 * @return boolean
	 */
	public boolean isModalitesVisibles() {
		for (DonneeID don : courante) {
			if(don instanceof Modalite) return true;
		}
		return false;
	}



	private boolean isIllustratifsVisibles() {
		for (DonneeID don : courante) {
			if(!don.isActive()) return true;
		}
		return false;
	}



	/**
	 * Fixe le masquage des données
	 * @param val : vrai si les donnée doivent étre en fantome, faux sinon	 * 
	 */
	public void setFantomeDonnesEnCours(boolean val) {
		if(getSelectionnes().size()==0)
			setFantome(getAffichable(),val);
		else
			setFantome(getSelectionnes(),val);
	}


	/**
	 * @return les données sur lesquelles le gestionnaire travaille
	 */
	public DonneeParse getDon()
	{return m_don;}



	/**
	 * Il faut utiliser les accesseurs par type de données
	 * modifie l'habillage
	 * @param habillage : le nouvel habillage
	 */
	@Deprecated
	public void setHabillage(Variable habillage)
	{
		sauverAnnotations();
		//m_habilleur.supprimerHabillage(Habilleur.INDIVIDUS);
		if(habillage instanceof ContinueAnalysee){
			getHabilleur().setHabillage(Habilleur.INDIVIDUS, (ContinueAnalysee)habillage, Habilleur.STYLE_NUAGE);
		}else{
			getHabilleur().setHabillage(Habilleur.INDIVIDUS, habillage);
		}
		updateGraphe();
	}

	/**
	 * @param type_donnees
	 * @param habillage
	 */
	public void setHabillage(int type_donnees, Variable habillage) {
		getHabilleur().setHabillage(type_donnees, habillage);
	}



	/**
	 * @param type_donnees
	 * @param habillage
	 * @param styleHabillage
	 */
	public void setHabillage(int type_donnees, Continue habillage, int styleHabillage) {
		getHabilleur().setHabillage(type_donnees, habillage, styleHabillage);
	}


	/**
	 * @return le type de graphique
	 */
	public int getType()
	{return m_type;}

	/**
	 * modifie le type de graphique
	 * @param type (utiliser les valeurs proposées en statique, il n'y a pas de vérification)
	 */
	public void setType(int type)
	{
		sauverAnnotations();
		m_type = type;
		m_selectionnes.clear();
		mettreAJourIHM();
	}

	/**
	 * redémarre l'IHM pour la mettre à jour
	 */
	public void mettreAJourIHM(){
		m_interf.redemarrer();
	}

	/**
	 * @return la collection des objets sélectionnés
	 */
	public Vector<DonneeID> getSelectionnes()
	{return m_selectionnes;}

	/**
	 * inverse la sélection (ignore les fantomes)
	 */
	public void inverserSelectionnes()
	{
		sauverAnnotations();
		Vector<DonneeID> ancienneSelection=new Vector<DonneeID>(m_selectionnes);
		m_selectionnes.clear();
		m_selectionnes.addAll(getCourant());
		m_selectionnes.removeAll(ancienneSelection);
		m_interf.updateMenuBarre();
		updateGraphe();
	}

	/**
	 * Fixe la collection des données sélectionnés
	 * @param selectionnes
	 */
	public void setSelectionnes(Vector<DonneeID> selectionnes)
	{
		sauverAnnotations();
		m_selectionnes = selectionnes;
		m_interf.updateMenuBarre();
		updateGraphe();		
	}

	/**
	 * @param selectionnes : la collection à ajouter aux données sélectionnés
	 */
	public void addSelectionnes(Collection<DonneeID> selectionnes)
	{
		sauverAnnotations();
		for (DonneeID donnee : selectionnes)
		{
			if(donnee instanceof Partitionnable)
			{
				if(((Partitionnable)donnee).getGeneration()==0)
					if(!(m_selectionnes.contains(donnee)))m_selectionnes.add(donnee);
			}
			else if(!(m_selectionnes.contains(donnee)))m_selectionnes.add(donnee);
		}
		m_interf.updateMenuBarre();
		updateGraphe();		
	}

	/**
	 * @param selectionnes : la donnée à ajouter aux données sélectionnées
	 */
	public void addSelectionnes(DonneeID selectionnes)
	{
		sauverAnnotations();
		m_selectionnes.add(selectionnes);
		m_interf.updateMenuBarre();
		updateGraphe();		
	}

	/**
	 * vide la collection d'objets sélectionnés
	 */
	public void rmSelectionnes()
	{
		sauverAnnotations();
		m_selectionnes.clear();
		m_interf.updateMenuBarre();
		updateGraphe();		
	}

	/**
	 * @return l'axe X (numéro)
	 */
	public ContinueAnalysee getAxeX()
	{return m_axeX[getType()];}

	/**
	 * Fixe le numéro de l'axe X
	 * @param axeX
	 */
	public void setAxeX(ContinueAnalysee axeX)
	{
		if(getType()==DATAFRAME){
			m_axeX[DATAFRAME] = axeX;
		}else{
			m_axeX[INDIVIDUS] = axeX;
			m_axeX[GROUPES] = axeX;
			m_axeX[AXES_PARTIELS] = axeX;
			m_axeX[VARIABLES] = axeX;
			m_habilleur_facto.setAxeX(axeX);
			m_habilleur_data.setAxeX(axeX);
		}		
	}

	/**
	 * @return l'axe Y (numéro)
	 */
	public ContinueAnalysee getAxeY()
	{return m_axeY[getType()];}

	/**
	 * Fixe le numéro de l'axe Y
	 * Si le type de graphe actuel n'est pas un dataframe, la variable doit être un axe
	 * @param axeY
	 */
	public void setAxeY(ContinueAnalysee axeY)
	{
		if(getType()==DATAFRAME){
			m_axeY[DATAFRAME] = axeY;
		}else{
			m_axeY[INDIVIDUS] = axeY;
			m_axeY[GROUPES] = axeY;
			m_axeY[AXES_PARTIELS] = axeY;
			m_axeY[VARIABLES] = axeY;
			m_habilleur_facto.setAxeY(axeY);
			m_habilleur_data.setAxeY(axeY);
		}
	}


	/**
	 * @return vrai si la représentation actuelle correspond à un plan factoriel
	 */
	public boolean isPlanFactoriel(){
		return getAxeX() instanceof Dimension && getAxeY() instanceof Dimension;
	}

	/**
	 * @return le panel jfreechart courant
	 */
	public ChartPanel getPanel()
	{return m_interf.getChartPanel();}

	/**
	 * modifie le panel jfreechart
	 * @param panel
	 */
	public void setPanel(ChartPanel panel)
	{
		m_interf.setChartPanel(panel);
	}

	/**
	 * modifie le canevas
	 * @param canvas
	 */
	public void setCanvas(Canvas canvas)
	{
		if(m_interf.getChartPanel()!=null)
		{
			m_interf.getChartPanel().setGestionnaire(this);
			m_interf.getChartPanel().setCanvas(canvas);
		}
	}


	/**
	 * retourne le canevas
	 * @return le canevas 
	 */
	public Canvas getCanvas()
	{
		if(m_interf.getChartPanel()!=null)
			return m_interf.getChartPanel().getCanvas();
		else return null;
	}

	/**
	 * met é jour le jeu de données courant
	 */
	public void setCourant()
	{
		courante=new Vector<DonneeID>();
		Iterator<Series> ite=series.iterator();
		while(ite.hasNext())
		{
			Series s=ite.next();
			for(int i=0;i<s.getNbDonneeID();i++)
				if(!isFantome(s.getDonneeID(i)))courante.add(s.getDonneeID(i));
		}
	}

	/**
	 * met a jour le jeu de données affichables
	 */
	public void setAffichable()
	{
		affichable=new Vector<DonneeID>();
		Iterator<Series> ite=series.iterator();
		while(ite.hasNext())
		{
			Series s=ite.next();
			for(int i=0;i<s.getNbDonneeID();i++)
				affichable.add(s.getDonneeID(i));
		}
	}

	/**
	 * @return le jeu de donnees courant
	 */
	public Vector<DonneeID> getCourant()
	{return courante;}

	/**
	 * @return le jeu de donnees affichable
	 */
	public Vector<DonneeID> getAffichable()
	{return affichable;}

	/**
	 * met a zéro le jeu de données
	 */
	public void setDonnee()
	{
		m_don=new NullDonneeParse();
		setValDefaut();
		m_interf.setDonneeParse(m_don);
	}

	/**
	 * modifie les valeur par défaut en fonction du fichier ouvert
	 * @param aff : les paramétres à appliquer
	 */
	public void setValDefaut(ConfAffichage aff)
	{
		if(aff!=null) {
			m_type=aff.getType();
			series=new TreeSet<Series>();
			setCourant();
			setAffichable();
			m_axeX=aff.getAxeX();
			m_axeY=aff.getAxeY();
			annotation=aff.getAnnotation();
			partiel=aff.getPartiel();
			fantome=aff.getFantome();
			m_affGrid=aff.isAffGrille();
			m_habilleur_facto = aff.getHabilleurFacto(m_don);
			m_habilleur_data = aff.getHabilleurData(m_don);
			m_titres=aff.getTitresGraphe();
			m_titresX=aff.getTitresX();
			m_titresY=aff.getTitreY();
			m_ortho=true;
		}else{
			setValDefaut();
		}
	}


	/**
	 * affecte les valeur par défaut au gestionnaire
	 */
	public void setValDefaut()
	{
		if(m_don.getType()==DonneeParse.TypeAnalyse.DATAFRAME){
			m_type=DATAFRAME;
		}else if(m_don.getType()==DonneeParse.TypeAnalyse.AUTRE && m_don.isRepDataframePossible()){
			m_type=DATAFRAME;
		}else{
			m_type=INDIVIDUS;
		}

		m_axeX=new ContinueAnalysee[NB_TYPES_MAX];
		m_axeY=new ContinueAnalysee[NB_TYPES_MAX];
		for(int i=0; i<NB_TYPES_MAX-1;i++){
			m_axeX[i]=m_don.getAxe(0);
			m_axeY[i]=m_don.getAxe(1);
		}
		if(m_don.isRepDataframePossible()){
			m_axeX[DATAFRAME]=m_don.getVariablesMoyennes().get(0);
			m_axeY[DATAFRAME]=m_don.getVariablesMoyennes().get(1);
		}
		if(annotation==null)
			annotation=new HashMap<DonneeID, Boolean>();
		if(partiel==null)
			partiel=new HashMap<DonneeID, Boolean>();
		if(fantome==null)
			fantome=new HashMap<DonneeID, Boolean>();
		for (DonneeID donnee : m_don.toDonneeID())
		{
			annotation.put(donnee, true);
			partiel.put(donnee, false);
			fantome.put(donnee, false);
		}
		m_affGrid=false;

		m_titres = new String[NB_TYPES_MAX];
		m_titresX = new String[NB_TYPES_MAX];
		m_titresY = new String[NB_TYPES_MAX];
		rapport=new double[NB_TYPES_MAX];
		for(int j=0;j<NB_TYPES_MAX;j++){
			m_titresX[j]=null;
			m_titresY[j]=null;
			m_titres[j]=null;
			rapport[j]=1;
		}
		m_habilleur_facto=new Habilleur(m_don, getAxeX(), getAxeY());
		m_habilleur_data=new Habilleur(m_don, getAxeX(), getAxeY());
		m_affLegende=false;
		m_ortho=true;
	}

	/**
	 * @return le titre par défaut pour l'axe Y 
	 * avec le pourcentage d'inertie entre parenthèses
	 */
	private String getTitreYDefautAvecInertie() {
		return getTitreYDefaut()+" ("+getInertieY()+")";
	}

	/**
	 * @return le titre par défaut pour l'axe X 
	 * avec le pourcentage d'inertie entre parenthèses
	 */
	private String getTitreXDefautAvecInertie() {
		return getTitreXDefaut()+" ("+getInertieX()+")";
	}

	/**
	 * Ouvre et charge un fichier
	 * @param chemin du fichier chargé
	 */
	public void ouvrir(String chemin)
	{
		try
		{
			Registre reg=Fichier.chargerDonnees(chemin);
			if(reg!=null){
				DonneeParse chargees=reg.getDonnee();
				chargees.verifierIntegrite();
				m_don=chargees;
				setValDefaut(reg.getConfAffichage());
				m_interf.setDonneeParse(chargees);
			}
		}
		catch (Exception e)
		{
			new FenetreErreur(e);
		}
	}

	/**
	 * Enregistre dans un fichier
	 * @param chemin du fichier à sauvegarder
	 */
	public void enregistrer(String chemin)
	{
		try
		{
			sauverAnnotations();
			m_don.verifierIntegrite();
			Registre reg=new Registre(m_don,
					new ConfAffichage(getType(),m_titres,m_titresX,m_titresY,annotation,partiel,fantome,m_axeX,m_axeY,isAffGrid(),m_habilleur_facto, m_habilleur_data));
			Fichier.serialiser(reg,chemin);
		}
		catch (ExceptionIntegrite e)
		{
			new FenetreErreur(e);
		}
	}

	/** 
	 * enregistre la position des annotations 
	 */
	public void sauverAnnotations()
	{
		if(getPanel()!=null && getPanel().getCanvas()!=null)
		{
			//On commence par rassembler l'ensemble des données 
			// dont les annotations peuvent nous intéresser
			Vector<DonneeID> id=m_don.toDonneeID();
			Vector<DonneeID> idUtil=new Vector<DonneeID>(id);
			for (DonneeID donnee : idUtil)
			{
				if(donnee instanceof IndividuMoyen)
				{
					id.addAll(((IndividuMoyen)donnee).getBranches());
				}
				else if(donnee instanceof ContinueMoyenne)
				{
					id.addAll(((ContinueMoyenne)donnee).getBranches());
				}
				else if(donnee instanceof ModaliteMoyenne)
				{
					id.addAll(((ModaliteMoyenne)donnee).getBranches());
				}
			}
			//puis on retrouve l'annotation de chaque donnée
			Vector<XYIDAnnotation> mesAnnot=getPanel().getCanvas().getVectAnnot();
			for(int i=0;i<mesAnnot.size();i++)
			{
				XYIDAnnotation annot=mesAnnot.get(i);
				boolean fait=false;
				for(int j=0;j<id.size() && !fait;j++)
				{
					DonneeID donnee=id.get(j);
					String txt=donnee.getAnnotation();
					if(donnee instanceof Partitionnable && ((Partitionnable)donnee).getGeneration()>0)
						txt=donnee.getAnnotation()+" ("+donnee.getGr().getAnnotation()+")";
					if(annot.getAnnot().equals(txt))
					{
						donnee.setXYannot(getAxeX(), getAxeY(), annot.getLblx(), annot.getLbly(),annot.getTextAnchor());
						fait=true;
					}
				}

			}
		}
	}

	/**
	 * mise é jour du graphique
	 */
	public void updateGraphe()
	{		
		prepareDonnees();		
		afficheGraphe();
	}


	private boolean isHabContinue() {
		boolean hab_continue=false;
		if(getType()==INDIVIDUS){   /* gettype : type de graphique*/
			hab_continue = getHabilleur().getHabillageContinu(Habilleur.INDIVIDUS)!=null 
			|| getHabilleur().getHabillageContinu(Habilleur.MODALITES)!=null ;			
		}else if(getType()==VARIABLES || getType()==AXES_PARTIELS) {
			hab_continue = getHabilleur().getHabillageContinu(Habilleur.CONTINUES)!=null;
		}else if(getType()==GROUPES){
			hab_continue = false;
		}else if(getType()==DATAFRAME){
			hab_continue = (getHabilleur().getHabillageContinu(Habilleur.INDIVIDUS)!=null 
					&& getHabilleur().getHabillageStyleContinu(Habilleur.INDIVIDUS) == Habilleur.STYLE_BULLES);			
		}
		return hab_continue;
	}


	/**
	 * 
	 */
	public void prepareDonnees() {
		//on trie le jeu de données :
		CritereCompose affichage=new CritereCompose();
		if(getType()==INDIVIDUS || getType()==DATAFRAME)
		{		    
			if(getHabilleur().getHabillageDiscret(Habilleur.INDIVIDUS)!=null) {
				//en cas d'habillage par nominale, on sépare les individus selon leurs modalités
				affichage.addCritere(new CritereNominale((Nominale)getHabilleur().getHabillageDiscret(Habilleur.INDIVIDUS)));
			}
			else {
				//s'il n'y a pas d'habillage par nominale, tous les individus sont ensemble
				affichage.addCritere(new CritereIndividu());
			}

			Iterator<Nominale> iter=m_don.getNominales().iterator();
			Vector<Nominale> actives=new Vector<Nominale>();
			Vector<Nominale> illustratives=new Vector<Nominale>();
			Nominale nom;
			while(iter.hasNext()){
				nom=iter.next();
				if(nom.isModActif())
					actives.add(nom);
				else
					illustratives.add(nom);
			}
			for (Nominale nominale : illustratives) {
				affichage.addCritere(new CritereModNominale(nominale));
			}
			for (Nominale nominale : actives) {
				affichage.addCritere(new CritereModNominale(nominale));
			}

		}
		else if(getType()==VARIABLES ){
			Critere habillage;
			Variable var = getHabilleur().getHabillageDiscret(Habilleur.CONTINUES);
			if(var!=null && var instanceof NominaleArtificielleGroupe) {
				habillage=new CritereHabGroupeContinue();
			}else {
				if(getType()==VARIABLES){
					habillage=new CritereVariableContinue();
				}else{
					habillage=new CritereVariableContinue(IHM.getXmlLangue().getElement("CaxePart"));
				}
			}
			affichage.addCritere(new CritereRestriction(habillage,new CritereAppartient(m_don.getVariablesMoyennes())));
		}
		else if(getType()==AXES_PARTIELS){
			affichage.addCritere(new CritereRestriction(new CritereHabGroupeContinue(),new CritereAppartient(m_don.getAxesPartiels())));
		}
		else if(getType()==GROUPES) affichage.addCritere(new CritereGroupe());


		//critère qui va mettre dans une série séparée 
		//toutes les données sélectionnées
		CritereCompose affichageSelection = new CritereAjout(affichage,new CritereAppartient(m_selectionnes) {
			public String semantique() {
				return String.valueOf(IHM.getXmlLangue().hashCode());
			}
		});


		//On effectue un premier tri pour être en mesure de dire si
		//les données affichées contiendront des illus
		Trieur t;
		t=new Trieur(affichageSelection);
		t.trier(m_don.toDonneeID());
		series=t.getSeries();
		setCourant();

		Critere critereFinal;
		//on sélectionne les données qui nous intéressent vraiment
		//en construisant des séries qui contiennent des actifs et/ou illustratifs
		if(getType()!=AXES_PARTIELS){
			if(!isIllustratifsVisibles()) {
				boolean affIllustratifsNecessaire=false;
				if(getType() == INDIVIDUS)
					for (Nominale nom : m_don.getNominales()) {
						if(!nom.isModActif())affIllustratifsNecessaire=true;
					}
				if(affIllustratifsNecessaire) {
					critereFinal=new CritereRestriction(affichageSelection,new CritereActifIllustratif());
				}else {
					critereFinal=new CritereRestriction(affichageSelection,new CritereActif());
				}
			}
			else 
				critereFinal=new CritereRestriction(affichageSelection,new CritereActifIllustratif());
		}else{
			critereFinal=affichageSelection;
		}

		if(isHabContinue()){
			critereFinal=new CritereRestriction(getHabilleur().getCriterePositifNegatif(),critereFinal);
		}

		if(getType()!=DATAFRAME){
			critereFinal=new CritereRestriction(critereFinal, getHabilleur().getCritereSeuilAffinage());
		}


		//on effectue le tri
		t=new Trieur(critereFinal);
		t.trier(m_don.toDonneeID());
		series=t.getSeries();

		setCourant();
		setAffichable();
	}


	private void afficheGraphe() {
		String titre=getTitreGraphe();
		String titreX=getTitreX();
		String titreY=getTitreY();
		if(getType()==INDIVIDUS)
		{
			boolean hab_bulles = (getHabilleur().getHabillageContinu(Habilleur.INDIVIDUS)!=null 
					&& getHabilleur().getHabillageStyleContinu(Habilleur.INDIVIDUS) == Habilleur.STYLE_BULLES)
					||
					(getHabilleur().getHabillageContinu(Habilleur.MODALITES)!=null 
							&& getHabilleur().getHabillageStyleContinu(Habilleur.MODALITES) == Habilleur.STYLE_BULLES);
			if(hab_bulles)
			{
				m_graphe=new Bubble(this,getHabilleur(),titre,titreX,titreY);
			}
			else if(isAffPartiels())
			{
				m_graphe=new Araignee(this,getHabilleur(),titre,titreX,titreY);
			}
			else
			{
				m_graphe=new NuagePoint(this,getHabilleur(),titre,titreX,titreY);
			}
		}
		else if(getType()==VARIABLES || getType()==AXES_PARTIELS) {
			m_graphe=new Vecteur(this,getHabilleur(),titre,titreX,titreY);
		}
		else if(getType()==GROUPES){
			m_graphe=new NuagePoint(this,getHabilleur(),titre,titreX,titreY);
		}else if(getType()==DATAFRAME){
			if(isHabContinue())
			{
				m_graphe=new Bubble(this,getHabilleur(),titre,titreX,titreY);
			}else{
				m_graphe=new NuagePoint(this,getHabilleur(),titre,titreX,titreY);
			}
		}		
		setCanvas(new Canvas());		
		m_graphe.update(series);
	}

	/**
	 * @return le titre par défaut pour l'axe Y
	 */
	public String getTitreYDefaut() {
		if(getAxeY()==null){
			return "*error";
		}else
			return getAxeY().getNom();
	}

	/**
	 * @return le titre par défaut pour l'axe X
	 */    
	public String getTitreXDefaut() {
		if(getAxeX()==null){
			return "*error";
		}else		
			return getAxeX().getNom();
	}

	/**
	 * @return le pourcentage d'inertie de l'axe Y sous la forme d'une chaine de caractéres
	 */
	public String getInertieY() {
		Continue axe = getAxeY();
		if(axe instanceof Dimension){
			return DonneeParse.pourcentage(m_don.getInerties()[((Dimension)axe).getNum()]);
		}
		else return null;
	}

	/**
	 * @return le pourcentage d'inertie de l'axe X sous la forme d'une chaine de caractéres
	 */
	public String getInertieX() {
		Continue axe = getAxeX();
		if(axe instanceof Dimension){
			return DonneeParse.pourcentage(m_don.getInerties()[((Dimension)axe).getNum()]);
		}
		else return null;
	}

	/**
	 * Fixe le titre de l'axe X
	 * @param titre
	 */
	public void setTitreAxeX(String titre){
		m_interf.getChartPanel().getChart().getXYPlot().getDomainAxis().setLabel(titre);
		m_titresX[m_type]=titre;
	}

	/**
	 * Fixe le titre de l'axe Y
	 * @param titre
	 */
	public void setTitreAxeY(String titre){
		m_interf.getChartPanel().getChart().getXYPlot().getRangeAxis().setLabel(titre);
		m_titresY[m_type]=titre;
	}

	/**
	 * @return le titre de l'axe X
	 */
	private String getTitreX(){
		String titreX = m_titresX[m_type];
		if(titreX==null){
			if(getType()!=DATAFRAME){
				titreX=getTitreXDefautAvecInertie();
			}else{
				titreX=getTitreXDefaut();
			}
		}
		return titreX;
	}

	/**
	 * @return le titre de l'axe Y
	 */
	private String getTitreY(){
		String titreY = m_titresY[m_type];
		if(titreY==null){
			if(getType()!=DATAFRAME){
				titreY=getTitreYDefautAvecInertie();
			}else{
				titreY=getTitreYDefaut();
			}
		}
		return titreY;
	}

	/**
	 * Fixe le titre du graphe
	 * @param titre
	 */
	public void setTitreGraphe(String titre){
		m_interf.getChartPanel().getChart().setTitle(titre);
		m_titres[m_type]=titre;
	}

	/**
	 * @return le titre du graphe
	 */
	private String getTitreGraphe() {
		String titre=m_titres[m_type];
		if(m_don.getType()==TypeAnalyse.CA){
			titre=IHM.getXmlLangue().getElement("LGrapheAFC");
		}else
			if(titre==null){
				switch (m_type) {
				case INDIVIDUS:
					titre=IHM.getXmlLangue().getElement("MtypeGRInd");
					break;
				case VARIABLES:
					titre=IHM.getXmlLangue().getElement("MtypeGRVar");
					break;
				case GROUPES:
					titre=IHM.getXmlLangue().getElement("MtypeGRGr");
					break;
				case AXES_PARTIELS:
					titre=IHM.getXmlLangue().getElement("MtypeGRAxesPart");
					break;
				case DATAFRAME:
					titre=IHM.getXmlLangue().getElement("MtypeGRAxesDataframe");
					break;
				default:
					titre=IHM.getXmlLangue().getElement("erreur");
				break;
				}
			}
		return titre;
	}


	/**
	 * @return vrai si la légende va être affichée, faux sinon
	 */
	public boolean isAffLegende() {
		return m_affLegende;
	}


	/**
	 * @param b vrai si la légende doit être affichée, faux sinon
	 */
	public void setAffLegende(boolean b) {
		m_affLegende=b;
		updateGraphe();
	}


	/**
	 * @param value
	 */
	public void setRapport(double value) {
		rapport[m_type]=value;

	}


	/**
	 * @return rapport[m_type]
	 */
	public double getRapport() {
		return rapport[m_type];
	}


	/**
	 * @return habilleur
	 */
	public Habilleur getHabilleur() {
		if(getType()==DATAFRAME){
			return m_habilleur_data;
		}else{
			return m_habilleur_facto;
		}
	}


	/**
	 * @return IHM
	 */
	public JFrame getIHM() {
		return m_interf;
	}


	/**
	 * @param axeX
	 */
	public void setAxeX(int axeX) {
		setAxeX(m_don.getAxe(axeX));
	}


	/**
	 * @param axeY
	 */
	public void setAxeY(int axeY) {
		setAxeY(m_don.getAxe(axeY));		
	}


	/**
	 * @return boolean
	 */
	public boolean isOrtho() {
		return m_ortho;
	}


	/**
	 * @param b
	 */
	public void setOrtho(boolean b) {
		m_ortho=b;		
	}


	/**
	 * @param d
	 */
	public void setAffinage(double d) {
		getHabilleur().setSeuilAffinage(d);
		sauverAnnotations();
		updateGraphe();
	}

	/**
	 * @return seuil affinage
	 */
	public double getSeuilAffinage(){
		return getHabilleur().getSeuilAffinage();
	}


	/**
	 * @return boolean
	 */
	public boolean isAffinagePossible() {
		return (m_type!=DATAFRAME) && m_don!=null && !(m_don instanceof NullDonneeParse);
	}


	/**
	 * @param qualifiable
	 */
	public void setDescription(Qualifiable qualifiable) {
		if(qualifiable!=null){
			getHabilleur().setDescription(new ValeurTest(qualifiable));
			if(!(qualifiable instanceof Modalite)){
				FenetreDescriptionContinue.montrer(m_interf,m_don,qualifiable);
			}
		}else{
			getHabilleur().setDescription(null);
		}

	}





}

