package ihm;

import java.awt.Font;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Vector;

import org.jfree.chart.annotations.XYIDAnnotation;

import outils.Constantes;
import outils.Variables;
import criteres.Critere;
import criteres.CriterePositifNegatif;
import criteres.CritereSeuilAffinage;
import don.AxePartiel;
import don.Continue;
import don.ContinueAnalysee;
import don.ContinueArtificielleSurAxes;
import don.ContinueMoyenne;
import don.Cos2;
import don.Dimension;
import don.DonneeID;
import don.DonneeParse;
import don.Individu;
import don.Modalite;
import don.Nominale;
import don.NominaleArtificielleGroupe;
import don.ValeurTest;
import don.Variable;
import dyn.Bubble;

/**
 * 
 * Classe qui regroupe les informations sur l'habillage du graphe
 * 
 *  @author julien.durand@gmail.com
 *
 */
public class Habilleur implements Serializable{


	/**
	 * 
	 */
	private static final long serialVersionUID = -6930776226529343758L;


	private DonneeParse m_don;
	
	private double m_seuil_affinage;


	/**
	 * Quel habillage discret utiliser pour les différents types de données ? 
	 * null si aucun
	 */
	private Vector<Variable> m_habillageDiscret;

	/**
	 * Quel habillage continu utiliser pour les différents types de données ? 
	 * null si aucun
	 */
	private Vector<Continue> m_habillageContinu;


	/**
	 * Pour fixer un style "bulles" pour l'habillage par continue
	 */
	public static final int STYLE_BULLES=0;
	/**
	 * Pour fixer un style "nuage" pour l'habillage par continue
	 */	
	public static final int STYLE_NUAGE=1;

	/**
	 * Quel style d'habillage continu utiliser pour les différents types de données ? 
	 * null si aucun
	 */
	private Vector<Integer> m_habillageStyleContinu;




	private Vector<HashMap<Continue, Double>> coeffZ;

	private Vector<HashMap<Continue, Double>> decZ;


	private Vector<Continue> m_habillageContinu_avant_desc;


	private Vector<Integer> m_habillageStyleContinu_avant_desc;


	private Vector<Variable> m_habillageDiscret_avant_desc;


	private Continue m_var_affinage;


	private ContinueAnalysee m_axeX;


	private ContinueAnalysee m_axeY;


	/**
	 * Pour accéder aux habillages des continues
	 */	
	public static final int AXES_PARTIELS=3;

	/**
	 * Pour accéder aux habillages des continues
	 */	
	public static final int CONTINUES=2;

	/**
	 * Pour accéder aux habillages des modalités
	 */	
	public static final int MODALITES=1;

	/**
	 * Pour accéder aux habillages des individus
	 */
	public static final int INDIVIDUS=0;

	private static final int NB_TYPES_DONNEES = 4;

	/**
	 * @param don : le jeu de données de référence
	 * @param axeX 
	 * @param axeY 
	 */
	public Habilleur(DonneeParse don, ContinueAnalysee axeX, ContinueAnalysee axeY){
		m_don=don;
		initTabCoeff();
		initTabHabillages();
		setAxeX(axeX);
		setAxeY(axeY);
		setAffinageDefaut();
	}

	private void initTabCoeff() {
		coeffZ=new Vector<HashMap<Continue,Double>>();
		coeffZ.setSize(NB_TYPES_DONNEES);
		decZ=new Vector<HashMap<Continue,Double>>();
		decZ.setSize(NB_TYPES_DONNEES);
		for (int i = 0; i < NB_TYPES_DONNEES; i++) {
			decZ.set(i, new HashMap<Continue, Double>());
			coeffZ.set(i, new HashMap<Continue, Double>());
		}
	}

	private void initTabHabillages() {
		m_habillageContinu = new Vector<Continue>();
		m_habillageContinu.setSize(NB_TYPES_DONNEES);
		m_habillageStyleContinu = new Vector<Integer>();
		m_habillageStyleContinu.setSize(NB_TYPES_DONNEES);
		for (int i = 0; i < NB_TYPES_DONNEES; i++) {
			m_habillageStyleContinu.setElementAt(STYLE_NUAGE, i);
		}
		m_habillageDiscret = new Vector<Variable>();
		m_habillageDiscret.setSize(NB_TYPES_DONNEES);
		m_habillageDiscret.setElementAt(new NominaleArtificielleGroupe(m_don.getGroupes().size()), AXES_PARTIELS);
		
		m_habillageContinu_avant_desc=(Vector<Continue>) m_habillageContinu.clone();
		m_habillageStyleContinu_avant_desc=(Vector<Integer>) m_habillageStyleContinu.clone();
		m_habillageDiscret_avant_desc=(Vector<Variable>) m_habillageDiscret.clone();
	}

	private double getCoeffZ(Continue habillage, int type_donnees) {
		Double coeff=coeffZ.get(type_donnees).get(habillage);
		if(coeff==null){
			setCoefficientsZ(habillage, type_donnees);	
			coeff=coeffZ.get(type_donnees).get(habillage);
		}
		return coeff;
	}

	private double getDecZ(Continue habillage, int type_donnees) {
		Double dec=decZ.get(type_donnees).get(habillage);
		if(dec==null){
			setCoefficientsZ(habillage, type_donnees);
			dec=decZ.get(type_donnees).get(habillage);
		}
		return dec;
	}

	private void setCoefficientsZ(Continue habillage, int type_donnees) {
		Double min=habillage.getMin(getDonnees(type_donnees));
		Double max=habillage.getAmplitude(getDonnees(type_donnees))+min;
		Double coeff;
		Double dec;
		if(max>0){
			if(min<0){
				dec = 0.0;
				coeff=Math.max(max, -min);
			}else{
				dec= min;
				coeff=max-min;
			}
		}else{
			dec = min;
			coeff=max-min;
		}		
		decZ.get(type_donnees).put(habillage, dec);
		coeffZ.get(type_donnees).put(habillage, coeff);
	}

	/**
	 * @param type_donnees
	 * @return si habillage centre
	 */
	public boolean isHabillageCentre(int type_donnees){
		Continue habillageContinu = getHabillageContinu(type_donnees);
		if(habillageContinu instanceof ValeurTest){
			return true;
		}else{
			Double min=habillageContinu.getMin(getDonnees(type_donnees));
			Double max=habillageContinu.getAmplitude(getDonnees(type_donnees))+min;
			return min < 0 && max > 0;
			//return getDecZ(habillageContinu,type_donnees)==0.0;
		}
	}

	/**
	 * @param don
	 * @return si habillage centre
	 */
	public boolean isHabillageCentre(DonneeID don){
		return isHabillageCentre(getNumTypeHab(don));
	}



	private Vector<? extends DonneeID> getDonnees(int type_donnees) {
		if(type_donnees == INDIVIDUS)return m_don.getIndividusMoyens();
		else if(type_donnees == CONTINUES) return m_don.getVariablesMoyennes();
		else if(type_donnees == MODALITES) return m_don.getModalites();
		else if(type_donnees == AXES_PARTIELS) return m_don.getAxesPartiels();
		else return null;
	}

	/**
	 * @param type_donnee
	 * @return la continue qui habille type_donnee. null s'il n'y en a pas
	 */
	public Continue getHabillageContinu(int type_donnee) {
		return m_habillageContinu.get(type_donnee);
	}


	/**
	 * @param type_donnee
	 * @return la variable discrete qui habille type_donnee. null s'il n'y en a pas
	 */
	public Variable getHabillageDiscret(int type_donnee) {
		return m_habillageDiscret.get(type_donnee);
	}


	/** 
	 * @param type_donnee
	 * @return le style d'habillage par continue pour le type type_donnee
	 */
	public Integer getHabillageStyleContinu(int type_donnee) {
		return m_habillageStyleContinu.get(type_donnee);
	}

	/**
	 * @param don
	 * @return la continue qui habille les données du même type que don. null s'il n'y en a pas
	 */
	public Continue getHabillageContinu(DonneeID don) {
		int numTypeHab = getNumTypeHab(don);
		if(numTypeHab==-1) return null;
		else
			return m_habillageContinu.get(numTypeHab);
	}


	/**
	 * @param don
	 * @return  la variable discrete qui habille les données du même type que don. null s'il n'y en a pas
	 */
	public Variable getHabillageDiscret(DonneeID don) {
		int numTypeHab = getNumTypeHab(don);
		if(numTypeHab==-1) return null;
		else
			return m_habillageDiscret.get(numTypeHab);
	}


	/**
	 * @param don
	 * @return le style d'habillage par continue pour les données du même type que don
	 */
	public Integer getHabillageStyleContinu(DonneeID don) {
		int numTypeHab = getNumTypeHab(don);
		if(numTypeHab==-1) return null;
		else
			return m_habillageStyleContinu.get(numTypeHab);
	}

	private int getNumTypeHab(DonneeID don){
		if(don instanceof Individu)return INDIVIDUS;
		else if(don instanceof Modalite) return MODALITES;
		else if(don instanceof AxePartiel) return AXES_PARTIELS;
		else if (don instanceof ContinueAnalysee) return CONTINUES;
		else return -1;
	}


	/**
	 * @param type_donnees
	 * Supprime l'habillage pour type_donnees
	 */
	public void supprimerHabillage(int type_donnees) {
		m_habillageContinu.setElementAt(null, type_donnees);
		m_habillageStyleContinu.setElementAt(STYLE_NUAGE, type_donnees);
		m_habillageDiscret.setElementAt(null, type_donnees);
	}


	/**
	 * @param cont
	 */
	public void setDescription(Continue cont) {
		if(cont!=null){
			m_habillageContinu_avant_desc=(Vector<Continue>) m_habillageContinu.clone();
			m_habillageStyleContinu_avant_desc=(Vector<Integer>) m_habillageStyleContinu.clone();
			m_habillageDiscret_avant_desc=(Vector<Variable>) m_habillageDiscret.clone();
			initTabHabillages();
			setHabillage(MODALITES, cont, STYLE_NUAGE);
		}else{
			m_habillageContinu=m_habillageContinu_avant_desc;
			m_habillageStyleContinu=m_habillageStyleContinu_avant_desc;
			m_habillageDiscret=m_habillageDiscret_avant_desc;
		}
	}

	/**
	 * @param type_donnees
	 * @param cont
	 * @param style_habillage
	 * Pour type_donnees, fixe l'habillage par la continue cont avec le style style_habillage
	 */
	public void setHabillage(int type_donnees, Continue cont, int style_habillage) {
		m_habillageContinu.setElementAt(cont, type_donnees);
		m_habillageStyleContinu.setElementAt(style_habillage, type_donnees);		
	}

	/**
	 * @param type_donnees
	 * @param habillage
	 * Pour type_donnees, fixe l'habillage par la variable discrete habillage
	 */
	public void setHabillage(int type_donnees, Variable habillage) {
		m_habillageDiscret.setElementAt(habillage, type_donnees);
	}

	/**
	 * @param axeX
	 * change l'axe X
	 * Important à appeler en cas de changement de plan pour que les variables qui en dépendent restent à jour
	 */
	public void setAxeX(ContinueAnalysee axeX) {
		m_axeX=axeX;
		if (axeX instanceof Dimension) {
			Dimension axeXDim = (Dimension) axeX;
		if(m_var_affinage instanceof ContinueArtificielleSurAxes){
			((ContinueArtificielleSurAxes)m_var_affinage).updateAxeX(axeXDim);
		}
		for (Continue var : m_habillageContinu) {
			if(var != null) 
				if (var instanceof ContinueArtificielleSurAxes) {
					//si l'habillage dépend des axes, il faut le mettre à jour
					((ContinueArtificielleSurAxes)var).updateAxeX(axeXDim);
				}
		}			
		}
	}

	/**
	 * @return the m_axeX
	 */
	public ContinueAnalysee getAxeX() {
		return m_axeX;
	}

	/**
	 * @return the m_axeY
	 */
	public ContinueAnalysee getAxeY() {
		return m_axeY;
	}

	/**
	 * @param axeY
	 * change l'axe Y
	 * Important à appeler en cas de changement de plan pour que les variables qui en dépendent restent à jour
	 */
	public void setAxeY(ContinueAnalysee axeY) {
		m_axeY=axeY;
		if (axeY instanceof Dimension) {
			Dimension axeYDim = (Dimension) axeY;
			
			if(m_var_affinage instanceof ContinueArtificielleSurAxes){
				((ContinueArtificielleSurAxes)m_var_affinage).updateAxeY(axeYDim);
			}
			for (Continue var : m_habillageContinu) {
				if(var != null) 
					if (var instanceof ContinueArtificielleSurAxes) {
						//si l'habillage dépend des axes, il faut le mettre à jour
						((ContinueArtificielleSurAxes)var).updateAxeY(axeYDim);
	
					}
			}
		}
	}

	/**
	 * @param habillage
	 * @param id
	 * @return
	 */
	public double getDecZ(Continue habillage, DonneeID id) {
		return getDecZ(habillage, getNumTypeHab(id));
	}

	/**
	 * @param habillage
	 * @param id
	 * @return
	 */
	public double getCoeffZ(Continue habillage, DonneeID id) {
		return getCoeffZ(habillage, getNumTypeHab(id));
	}


	/** 
	 * retourne une valeur comprise entre 0.05 et 1 
	 * calculée en prenant compte de la valeur minimale de la continue
	 * ainsi que du rapport entre la valeur minimale et la valeur maximale
	 * @param id : la donnée à évaluer
	 * @param habillage : l'habillage à utiliser
	 * @return une valeur réduite et utilisable dans les graphes {@link Bubble}
	 */
	public double getZ(DonneeID id, Continue habillage) {
		double valeurBrute = m_don.getValeur(id, habillage);
		return getZajuste(getNumTypeHab(id), habillage, valeurBrute);
	}

	private double getZajuste(int type, Continue habillage, double valeurBrute) {
		double val=valeurBrute;
		//double test=(val+getDecZ(habillage,type))/(getCoeffZ(habillage,type)+getDecZ(habillage,type))+0.05;
		double test=((val-getDecZ(habillage,type))/(getCoeffZ(habillage,type)))+0.05;
		double valeur;
		if(test>1)
			valeur=1;
		else if(test<0.05)
			valeur= 0.05;
		//les deux tests ci-dessus devraient, normalement, toujours être faux 
		else valeur= test;
		return valeur;
	}

	/**
	 * @param id
	 * @param habillage
	 * @return calcule la font de l'annotation de la donnée id en utilisant l'habillage habillage
	 */
	public Font getFont(DonneeID id, Continue habillage) {
		int style = Variables.POLICE_ANNOTATIONS.getStyle();
		int taille;
		double coeff=getZ(id,habillage);
		if(habillage instanceof ValeurTest){
			double moyen=getZajuste(getNumTypeHab(id),habillage,2.0);
			taille = (int)(((Variables.POLICE_ANNOTATIONS.getSize()-Variables.TAILLE_MIN_POLICE_ANNOTATIONS)/moyen)*coeff+Variables.TAILLE_MIN_POLICE_ANNOTATIONS);
		}else{
			taille = (int)(coeff * (Variables.TAILLE_MAX_POLICE_ANNOTATIONS-Variables.TAILLE_MIN_POLICE_ANNOTATIONS)+Variables.TAILLE_MIN_POLICE_ANNOTATIONS);
			//if(coeff>0.7)style=Font.BOLD;
		}
		if(taille>Variables.TAILLE_MAX_POLICE_ANNOTATIONS){taille=Variables.TAILLE_MAX_POLICE_ANNOTATIONS;}
// version 1.05
		if(taille<((Variables.TAILLE_MIN_POLICE_ANNOTATIONS)-2)){taille=((Variables.TAILLE_MIN_POLICE_ANNOTATIONS)-2);}
// fin version 1.05
		return new Font(Variables.POLICE_ANNOTATIONS.getName(), style,taille);
	}

	/**
	 * @param annot
	 * @param id
	 */
	public void setFont(XYIDAnnotation annot, DonneeID id) {
		Continue habillage = getHabillageContinu(id);
		if(habillage!=null && getHabillageStyleContinu(id)==Habilleur.STYLE_NUAGE){						
			annot.setFont(getFont(id,habillage));
		}

	}

	/**
	 * @return
	 */
	public Critere getCriterePositifNegatif() {
		return new CriterePositifNegatif(this);
	}
	
	/**
	 * @return
	 */
	public Critere getCritereSeuilAffinage(){
		return new CritereSeuilAffinage(this);
	}


	/**
	 * @return
	 */
	public boolean isHabillageNominalePossible() {
		return m_don.getNominales().size() > 0;
	}

// version 1.05
	/**
	 * @return
	 */
	public DonneeParse.TypeAnalyse getTypeAnalyse(){
		return m_don.getType();
	}
// fin version 1.05
	
	/**
	 * @return
	 */
	public boolean isHabillageContinuePossible(){
		return m_don.getVariablesMoyennes().size()>0;
	}

	/**
	 * @param type
	 * @return
	 */
	public Vector<ContinueMoyenne> getHabillageParContinuesPossible(int type) {
		if((type==Habilleur.MODALITES || type==Habilleur.INDIVIDUS)
				&& m_don.isVar()){
			return m_don.getVariablesMoyennes();
		}else return new Vector<ContinueMoyenne>();
	}

	/**
	 * @return
	 */
	public Variable getHabillageGroupe() {
		return new NominaleArtificielleGroupe(m_don.getGroupes().size());
	}

	/**
	 * @param type
	 * @return
	 */
	public boolean isHabillageCos2Possible(int type) {
		return type!=Habilleur.AXES_PARTIELS;
	}

	/**
	 * @param type
	 * @return
	 */
	public boolean isHabillageContribPossible(int type) {
		return type==Habilleur.AXES_PARTIELS ||m_don.isVarActifNonOrtho()||m_don.isVarActifOrtho();
	}

	/**
	 * @return
	 */
	public boolean isHabillageIndividusParNominalePossible() {
		return m_don.isModActif() || m_don.isModIllus();
	}

	/**
	 * @param type
	 * @return
	 */
	public Vector<Nominale> getHabillageParNominalePossible(int type) {
		if(type==Habilleur.INDIVIDUS && m_don.isNominales()){
			return m_don.getNominales();
		}else return new Vector<Nominale>();
	}

	/**
	 * @param typeAff
	 * @return
	 */
	public boolean isHabillagePossible(int typeAff) {				
		return typeAff!=Gestionnaire.GROUPES &&
		(m_don.getType()==DonneeParse.TypeAnalyse.MFA
				|| m_don.getType()==DonneeParse.TypeAnalyse.DATAFRAME
				|| typeAff==Gestionnaire.INDIVIDUS
// version 1.05
				|| ((typeAff==Gestionnaire.VARIABLES)&&(m_don.getType()==DonneeParse.TypeAnalyse.PCA))
// fin version 1.05
				|| typeAff==Gestionnaire.DATAFRAME);
	}
	
	

	/**
	 * @param d
	 */
	public void setSeuilAffinage(double d) {
		m_seuil_affinage=d;
	}
	
	/**
	 * @return
	 */
	public double getSeuilAffinage() {
		return m_seuil_affinage;
	}
	
	/**
	 * @param cont
	 */
	public void setVarAffinage(Continue cont){
		m_var_affinage=cont;
	}

// version 1.05_light
//	private void setAffinageDefaut() {
//		if(getAxeX() instanceof Dimension && getAxeY() instanceof Dimension){
//			if((m_don.isVarActifNonOrtho())&&(m_don.getType()==DonneeParse.TypeAnalyse.PCA)){
//				setVarAffinage(new Contrib((Dimension)getAxeX(),(Dimension)getAxeY()));
//				setSeuilAffinage(Constantes.VALEUR_AFFINAGE_DEFAUT);
//			}
//			else{
//				setVarAffinage(new Cos2((Dimension)getAxeX(),(Dimension)getAxeY()));
//				setSeuilAffinage(Constantes.VALEUR_AFFINAGE_DEFAUT);
//			}
//		}
//	}
// fin version 1.05_light

// version 1.05_reverse = version 1.0
	private void setAffinageDefaut()
    {
        if((getAxeX() instanceof Dimension) && (getAxeY() instanceof Dimension))
        {
            setVarAffinage(new Cos2((Dimension)getAxeX(), (Dimension)getAxeY()));
            setSeuilAffinage(Constantes.VALEUR_AFFINAGE_DEFAUT);
        }
    }
// fin version 1.05_reverse

	/**
	 * @param don
	 * @return
	 */
	public boolean isSousSeuilAffinage(DonneeID don) {
		return m_don.getValeur(don, m_var_affinage)<m_seuil_affinage;
	}
}
