package ihm;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.Hashtable;

import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JSlider;
import javax.swing.JTabbedPane;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.UIManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jfree.chart.ChartPanel;
import org.jfree.ui.RefineryUtilities;
import org.jvnet.substance.SubstanceLookAndFeel;
import org.jvnet.substance.skin.SubstanceOfficeSilver2007LookAndFeel;
import org.jvnet.substance.theme.SubstanceSteelBlueTheme;

import outils.Constantes;
import outils.Fichier;
import outils.XML;
import actions.ActionChangerAnnotations;
import actions.ActionChoixAxes;
import actions.ActionChoixTailleAnnotations;
import actions.ActionEnregistrer;
import actions.ActionExporter;
import actions.ActionFermer;
import actions.ActionGestionnaire;
import actions.ActionInternalClosed;
import actions.ActionLangue;
import actions.ActionOuvrir;
import don.DonneeParse;
import don.NullDonneeParse;
import drawing.Registre;
import exceptions.ExceptionIntegrite;

/** Interface graphique
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class IHM extends JFrame
{
	private static final long serialVersionUID = -783853601094190381L;

	/** répertoire d'installation */
	public static String m_dir;

	/** éléments graphique  */

	/** la barre de menu */
	private JMenuBar m_menu = null;

	/** le menu fichier */
	private JMenu m_fichier = null;

	/** l'item de menu ouvrir */
	private JMenuItem m_ouvrir = null;

	/** l'item de menu fermer */
	private JMenuItem m_fermer;

	/** l'item de menu enregistrer */
	private JMenuItem m_enregistrer = null;

	/** l'item de menu exporter */
	private JMenuItem m_exporter;

	/** l'item de menu imprimer */
	private JMenuItem m_imprimer = null;

	/** l'item de menu langue */
	private JMenuItem m_langue = null;

	/** l'item de menu quitter */
	private JMenuItem m_quitter = null;

	/** le menu configuration */
	private JMenu m_configuration = null;

	/** l'item de menu modification axe */
	private JMenuItem m_axe = null;
	
	//@trovalet
	/** l'item de menu modification taille des annotations */
	private JMenuItem m_tailleAnnotations = null;

	/** l'item de menu type graphique */
	private JMenuItem m_typeGraphique = null;

	/** type de graphique ici individus */
	protected JRadioButtonMenuItem m_individus;

	/** type de graphique ici variables */
	private JRadioButtonMenuItem m_variables;

	/** type de graphique ici groupes */
	private JRadioButtonMenuItem m_groupes;

	/** type de graphique ici axes partiels */
	private JRadioButtonMenuItem m_axesPart;

	/** type de graphique ici axes partiels */
	private JRadioButtonMenuItem m_dataframe;


	/** l'item de menu fantome */
	protected JCheckBoxMenuItem m_fantome;

	/** l'item de menu afficher les partiels */
	private JCheckBoxMenuItem m_partiels;


	/** l'item de menu grille */
	protected JCheckBoxMenuItem m_grille;

	/** l'item de menu afficher les annotations */
	private JCheckBoxMenuItem m_annotations;

	/** le menu de sélection */
	private JMenu m_selection;

	/** l'item de menu déselection */
	private JMenuItem m_deselect = null;

	/** l'item de menu inverser */
	private JMenuItem m_inverser = null;

	/** l'item de menu selection par liste */
	private JMenuItem m_lstSelection = null;

	/** la barre d'icones */
	private JToolBar m_toolbar;

	private JSlider m_sliderAffinage = null;
	private JToolBar m_toolbarAffinage = null;


	/** icone grille */
	private ImageIcon m_icoGrille;

	/** bouton qui stock l'icone grille */
	private JToggleButton m_btGrille;

	/** icone deselection */
	private ImageIcon m_icoDeselection;

	/**  bouton qui stock l'icone déselection */
	private JButton m_btDeselection;	

	/** icone inverser sélection */
	private ImageIcon m_icoInverserSelection;

	/**  bouton qui stock l'icone inverser sélection */
	private JButton m_btInverserSelection;

	/** icone selection par liste */
	private ImageIcon m_icoSelectionParListe;

	/** bouton qui stock l'icone sélection par liste */
	private JButton m_btSelectionParListe;

	/** icone annotation */
	private ImageIcon m_icoAnnotation;

	/** bouton qui stock l'icone annotation */
	private JToggleButton m_btAnnotation;

	/** icone fantome */
	public ImageIcon m_icoFantome;

	/**  bouton qui stock l'icone fantome */
	private JToggleButton m_btFantome;

	/** icone partiel */
	private ImageIcon m_icoPartiel;

	/**  bouton qui stock l'icone partiel */
	protected JToggleButton m_btPartiel;

	/** attributs */

	/** Le gestionnaire de l'application */
	protected Gestionnaire m_gestionnaire;

	/** Le jeu de donnees sur lequel on travaille */
	protected DonneeParse m_donneesParse;

	/** Le graphique JFreeChart */
	private ChartPanel m_chartPanel;

	/** l'application a-t-elle été lancée depuis R ? */
	private boolean m_goByR;

	/** Fichier de langue */
	private static XML m_xmlLangue;

	/** Fichier qui donne toutes les langues supportees */
	private XML m_xmlGeneral;

	private JCheckBoxMenuItem m_affLegende;

	private JMenuItem m_habillageFenetre;

	private JButton m_btHabillage;

	private ImageIcon m_icoHabillage;

	private JTabbedPane m_tabbedPane;

	protected int[] m_correspondance_tab_types;
	
	private Gestionnaire m_gest;
	
	

	/** Constructeur
	 * @param donnees collection d'objet DonneeID
	 */
	public IHM(DonneeParse donnees)
	{
		super("dynGraph");
		//Mise en place du look'n feel
		try 
		{
			UIManager.setLookAndFeel(new SubstanceOfficeSilver2007LookAndFeel());
			SubstanceLookAndFeel.setCurrentTheme(new SubstanceSteelBlueTheme());
			SubstanceLookAndFeel.hideMenuSearchPanels();
			UIManager.put(SubstanceLookAndFeel.NO_EXTRA_ELEMENTS, Boolean.TRUE);

		} catch (Exception ulafe)
		{System.out.println("Substance failed to set");}
		
		Image icone = Toolkit.getDefaultToolkit().getImage(m_dir+Constantes.ICONE_FENETRE);
		setIconImage(icone);
		// gestion de la langue
		gestionLangue();
		//initialisation donnee et gestionnaire
		m_donneesParse = donnees;
		m_gestionnaire=new Gestionnaire(this);
	}

	/** @return si l'application est lance par R */
	public boolean getGoByR()
	{return m_goByR;}

	/** @return le jeu de donnee */
	public DonneeParse getDonnee()
	{return m_donneesParse;}

	/** Modification du jeu de Donnee
	 * @param donnee
	 */
	public void setDonneeParse(DonneeParse donnee)
	{
		m_donneesParse=donnee;
		redemarrer();
	}

	/** réinitialise le menu */
	public void redemarrer()
	{
		/** la barre de menu */
		m_menu = null;
		m_fichier = null;
		m_ouvrir = null;
		m_fermer=null;
		m_enregistrer = null;
		m_exporter = null;
		m_imprimer = null;
		m_langue = null;
		m_quitter = null;
		m_configuration = null;
		m_habillageFenetre = null;
		m_axe = null;
		//@trovalet
		m_tailleAnnotations = null;
		m_typeGraphique = null;
		m_individus = null;
		m_variables = null;
		m_groupes = null;
		m_axesPart = null;
		m_dataframe=null;
		m_affLegende = null;
		m_fantome=null;
		m_partiels = null;
		m_grille=null;
		m_annotations=null;
		m_selection=null;
		m_deselect=null;
		m_inverser=null;
		m_lstSelection=null;
		/** toolbar */
		m_toolbar = null;
		m_btGrille=null;
		m_btDeselection=null;
		m_btInverserSelection=null;
		m_btSelectionParListe=null;
		m_btHabillage=null;
		m_btAnnotation=null;
		m_btFantome=null;			
		m_btPartiel=null;	
		m_tabbedPane=null;
		m_toolbarAffinage=null;
		m_sliderAffinage=null;
		/** initialisation graphique JFreeChart */
		m_chartPanel=null;
		/** démarrage */
		if(!(m_donneesParse instanceof NullDonneeParse)){
			m_gestionnaire.prepareDonnees();
		}
		updateContenu();
	}

	/** permet d'ouvrir la fenetre */
	public void demarrer()
	{		
		if(!(m_donneesParse instanceof NullDonneeParse)){
			m_gestionnaire.updateGraphe();
		}
		outils.Variables.Variables();
		initialiser();
	}

	/** Initiale le panel */
	private void initialiser()
	{
		//plein écran
		GraphicsEnvironment graphicsEnvironment = GraphicsEnvironment
		.getLocalGraphicsEnvironment();
		Rectangle bounds = graphicsEnvironment.getMaximumWindowBounds();
		setBounds(bounds);
		this.setExtendedState(this.getExtendedState() | this.MAXIMIZED_BOTH);
		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{quitter();}
		});
		RefineryUtilities.centerFrameOnScreen(this);
		if(m_goByR)setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

		updateContenu();

	}

	/** @return le graphique JFreeChart */
	public ChartPanel getChartPanel()
	{return m_chartPanel;}

	/** getteur DonneeParse 
	 * @return les donnees*/
	public DonneeParse getDonneeParse()
	{return m_donneesParse;}

	/** modifie le graphiqe JFreeChart
	 * @param chartPanel
	 */
	public void setChartPanel(ChartPanel chartPanel)
	{m_chartPanel = chartPanel;}

	/** modifie l'attribut renseignant qui lance l'application
	 * @param val */
	public void setGoByR(boolean val)
	{m_goByR = val;}

	/** @return le gestionnaire */
	public Gestionnaire getGest()
	{return m_gestionnaire;}


	/** @return les données sont elles chargees ? */
	private boolean isDonneesChargees()
	{return !(m_donneesParse instanceof NullDonneeParse);}

	/** changement langue
	 * @param index langue choisie
	 * @throws Exception 
	 */
	public void updateLang(int index) throws Exception
	{
		String nom = m_xmlGeneral.getElement("langue", index);
		m_xmlGeneral.setDefaut(m_xmlGeneral.searchByNom(nom));
		String path=m_xmlGeneral.searchByNom(nom) + ".xml";
		m_xmlLangue=new XML(IHM.m_dir+"langue"+File.separator+path);
		if(!(m_donneesParse instanceof NullDonneeParse))
		{
			m_gestionnaire.sauverAnnotations();
			m_gestionnaire.updateGraphe();
		}
		redemarrer();
	}


	/** Met à jour les textes par défaut (JFileChooser en particulier) en fonction de la langue */
	private void updateLangTextesDefaut()
	{
		UIManager.put("FileChooser.saveButtonText",IHM.getXmlLangue().getElement("Tenregistrer"));
		UIManager.put("FileChooser.openButtonText",IHM.getXmlLangue().getElement("Touvrir"));
		UIManager.put("FileChooser.cancelButtonText",IHM.getXmlLangue().getElement("Lannuler"));
		UIManager.put("FileChooser.updateButtonText","");
		UIManager.put("FileChooser.helpButtonText","");
		UIManager.put("FileChooser.saveButtonToolTipText","");
	}

	/** Gestion de la langue */
	private void gestionLangue()
	{
		m_xmlGeneral = new XML(m_dir+"langue"+File.separator+"langue.xml");
		String defaut = m_xmlGeneral.getDefaut();
		int max = m_xmlGeneral.getNbBalise("langue");
		int i = 0;
		boolean test = false;
		while (i < max && !test) {
			String chaine = m_xmlGeneral.getElement("id", i);
			if (defaut.compareTo(chaine) == 0) {
				m_xmlLangue=new XML(m_dir+"langue"+File.separator + chaine + ".xml");
				test = true;
			}
			i++;
		}
	}


	/**
	 * This method initializes jSlider	
	 * 	
	 * @return javax.swing.JSlider	
	 */
	private JSlider getSliderAffinage() {
		if (m_sliderAffinage == null) {
			int val = 0;
			double affinage = m_gestionnaire.getSeuilAffinage();
			if(affinage>0){
				val=(int)(affinage*100.0);
			}
			m_sliderAffinage = new JSlider(0,100,val);
			m_sliderAffinage.setOrientation(JSlider.VERTICAL);
			m_sliderAffinage.setAlignmentY(Component.CENTER_ALIGNMENT);
			m_sliderAffinage.setPaintTicks(true);
			m_sliderAffinage.setPaintLabels(true);
			m_sliderAffinage.setMajorTickSpacing(10);
			Hashtable<Integer, JLabel> table=new Hashtable<Integer, JLabel>();
			for(int i=0;i<=100;i+=10){
				table.put(i, new JLabel(String.valueOf(i/100.0)));
			}
			m_sliderAffinage.setLabelTable(table);
			m_sliderAffinage.addChangeListener(new ChangeListener(){
				public void stateChanged(ChangeEvent e) {
					JSlider source = (JSlider)e.getSource();
					if (!source.getValueIsAdjusting() && m_gestionnaire.isAffinagePossible()) {
						int val = (int)source.getValue();
						m_gestionnaire.setAffinage(val/100.0);
					}
				}
			});
		}
		if(!m_gestionnaire.isAffinagePossible()){
			m_sliderAffinage.setValue(0);
			m_sliderAffinage.setEnabled(false);
		}
		return m_sliderAffinage;
	}

	/**
	 * This method initializes jToolBar	
	 * 	
	 * @return javax.swing.JToolBar	
	 */
	private JToolBar getToolBarAffinage() {
		if (m_toolbarAffinage == null) {
			m_toolbarAffinage = new JToolBar();
			m_toolbarAffinage.setOrientation(JToolBar.VERTICAL);
			JLabel label = new JLabel(IHM.getXmlLangue().getElement("LAffinageCos2"));
			label.setAlignmentX(Component.CENTER_ALIGNMENT);
			m_toolbarAffinage.add(label);
			m_toolbarAffinage.add(getSliderAffinage());
			m_toolbarAffinage.setFloatable(false);
		}
		return m_toolbarAffinage;
	}

	/** @return the toolbar */
	private JToolBar getToolbar()
	{
		if(m_toolbar==null)m_toolbar = new JToolBar();

		/** bouton grille */
		getBtGrid();

		/** bouton gabillage */
		getBtHabillage();

		m_toolbar.addSeparator();

		/** bouton tout deselectionner */
		getBtDeselect();

		/** bouton inverser la selection */
		getBtInverserSelect();

		/** bouton sélection par liste */
		getBtSelectListe();

		m_toolbar.addSeparator();

		/** bouton annotation */
		getBtAnnot();

		/** bouton fantome */
		getBtFantome();

		/** bouton partiel */
		getBtPartiel();

		m_toolbar.validate();
		m_toolbar.setFloatable(false);

		return m_toolbar;
	}

	private JButton getBtHabillage() {
		if(m_btHabillage==null)
		{
			m_icoHabillage = new ImageIcon(m_dir+Constantes.ICONE_HABILLAGE);
			m_btHabillage=new JButton(m_icoHabillage);
			m_btHabillage.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					new FenetreHabillage(IHM.this,m_gestionnaire);
				}
			});
			m_toolbar.add(m_btHabillage);
		}
		m_btHabillage.setEnabled(isDonneesChargees() 
				&& m_gestionnaire.getHabilleur().isHabillagePossible(m_gestionnaire.getType()));
		m_btHabillage.setToolTipText(IHM.getXmlLangue().getElement("Mhabillage"));
		return m_btHabillage;

	}

	/** @return la barre de menu */
	private JMenuBar getMenu()
	{
		if(m_menu==null)m_menu = new JMenuBar();
		return m_menu;
	}

	/** @return le bouton annotation */
	protected JToggleButton getBtAnnot()
	{
		if(m_btAnnotation==null)
		{
			m_icoAnnotation = new ImageIcon(m_dir+Constantes.ICONE_ANNOTATIONS);
			m_btAnnotation = new JToggleButton(m_icoAnnotation);
			m_btAnnotation.addActionListener(new ActionChangerAnnotations(m_gestionnaire,m_btAnnotation));
			m_toolbar.add(m_btAnnotation);
		}
		m_btAnnotation.setEnabled(isActionAnnotPossible());
		m_btAnnotation.setSelected(m_gestionnaire.isAnnotationsDonnesEnCours());
		m_btAnnotation.setToolTipText(IHM.getXmlLangue().getElement("Mannotations"));
		return m_btAnnotation;
	}

	/** @return si l'action d'affichage des annotations est possible */
	private boolean isActionAnnotPossible()
	{return isDonneesChargees() && !m_gestionnaire.isFantome(m_gestionnaire.getSelectionnes());}

	/** @return le bouton grille */
	private JToggleButton getBtGrid()
	{
		if(m_btGrille==null)
		{
			m_icoGrille = new ImageIcon(m_dir+Constantes.ICONE_GRID);
			m_btGrille=new JToggleButton(m_icoGrille);
			m_btGrille.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					m_gestionnaire.setAffGrid(!m_gestionnaire.isAffGrid());
					updateMenuBarre();
				}
			});
			m_toolbar.add(m_btGrille);
		}
		m_btGrille.setSelected(m_gestionnaire.isAffGrid());
		m_btGrille.setEnabled(isDonneesChargees());
		m_btGrille.setToolTipText(IHM.getXmlLangue().getElement("Mgrid"));
		return m_btGrille;
	}

	/** @return le bouton deselection */
	private JButton getBtDeselect()
	{
		if(m_btDeselection==null)
		{
			m_icoDeselection = new ImageIcon(m_dir+Constantes.ICONE_DESELECTION);
			m_btDeselection=new JButton(m_icoDeselection);
			m_btDeselection.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{m_gestionnaire.rmSelectionnes();}
			});
			m_toolbar.add(m_btDeselection);
		}
		m_btDeselection.setEnabled(isDonneesChargees());
		m_btDeselection.setToolTipText(IHM.getXmlLangue().getElement("Mdeselectionner"));
		return m_btDeselection;
	}

	/** @return le bouton inverser sélection */
	private JButton  getBtInverserSelect()
	{
		if(m_btInverserSelection==null)
		{
			m_icoInverserSelection = new ImageIcon(m_dir+Constantes.ICONE_INVERSER);
			m_btInverserSelection=new JButton(m_icoInverserSelection);
			m_btInverserSelection.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					m_gestionnaire.inverserSelectionnes();
					updateMenuBarre();
				}
			});
			m_toolbar.add(m_btInverserSelection);
		}
		m_btInverserSelection.setEnabled(isDonneesChargees());
		m_btInverserSelection.setToolTipText(IHM.getXmlLangue().getElement("MinverserSelection"));
		return m_btSelectionParListe; 
	}

	/** @return le bouton sélection par liste */
	private JButton getBtSelectListe()
	{
		if(m_btSelectionParListe==null)
		{
			m_icoSelectionParListe = new ImageIcon(m_dir+Constantes.ICONE_SELECTION_LISTE);
			m_btSelectionParListe=new JButton(m_icoSelectionParListe);
			m_btSelectionParListe.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e) {
					new FenetreSelection(IHM.this,m_donneesParse,m_gestionnaire);
				}
			});
			m_toolbar.add(m_btSelectionParListe);
		}
		m_btSelectionParListe.setEnabled(isDonneesChargees());
		m_btSelectionParListe.setToolTipText(IHM.getXmlLangue().getElement("MselectListe"));
		return m_btSelectionParListe;
	}

	/** @return le bouton fantome */
	private JToggleButton getBtFantome()
	{
		if(m_btFantome==null)
		{
			m_icoFantome = new ImageIcon(m_dir+Constantes.ICONE_FANTOME);
			m_btFantome=new JToggleButton(m_icoFantome);
			m_btFantome.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					boolean nouvel_etat=!m_gestionnaire.isContientFantomeDonnesEnCours();
					m_gestionnaire.setFantomeDonnesEnCours(nouvel_etat);
					updateMenuBarre();
				}
			});
			m_toolbar.add(m_btFantome);
		}
		m_btFantome.setSelected(m_gestionnaire.isContientFantomeDonnesEnCours());
		m_btFantome.setEnabled(isDonneesChargees());
		m_btFantome.setToolTipText(IHM.getXmlLangue().getElement("Mfantome"));
		return m_btFantome;
	}

	/** @return le bouton partiel */
	private JToggleButton getBtPartiel()
	{
		if(m_btPartiel==null)
		{
			m_icoPartiel = new ImageIcon(m_dir+Constantes.ICONE_PARTIELS);
			m_btPartiel=new JToggleButton(m_icoPartiel);
			m_btPartiel.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					boolean nouvel_etat=!m_gestionnaire.isAffPartielsDonnesEnCours();
					m_gestionnaire.setAffPartielsDonnesEnCours(nouvel_etat);
					updateMenuBarre();
				}
			});
			m_toolbar.add(m_btPartiel);
		}
		m_btPartiel.setEnabled(isActionPartielPossible());
		m_btPartiel.setSelected(m_gestionnaire.isAffPartielsDonnesEnCours());
		m_btPartiel.setToolTipText(IHM.getXmlLangue().getElement("MPartiels"));
		return m_btPartiel;
	}

	/** @return si l'action d'affichage des partiels est possible */
	private boolean isActionPartielPossible()
	{
		return !m_gestionnaire.isFantome(m_gestionnaire.getSelectionnes()) && m_donneesParse.isIndPart() && m_gestionnaire.getType()==Gestionnaire.INDIVIDUS ||
		m_donneesParse.isVarPart() && m_gestionnaire.getType()==Gestionnaire.VARIABLES;
	}

	/** @return le menu fichier */
	private JMenu getFichier()
	{
		if(m_fichier==null)m_fichier = new JMenu();	
		m_fichier.setText(getXmlLangue().getElement("Mfichier"));
		//ouvrir
		getOuvrir();
		//fermer
		getFermer();
		//enregistrer
		getEnregistrer();
		//enregistrer sous
		getExporter();
		//imprimer
		getPrint();
		//Choix de la langue 	
		getLang();
		// Mode debug 
		//getError();
		//Quitter
		getQuitter();
		return m_fichier;
	}

	/** @return le menu langue */
	private JMenuItem getLang()
	{
		if(m_langue==null)
		{
			m_langue = new JMenu();
			ButtonGroup types=new ButtonGroup();
			String langueDefaut=m_xmlGeneral.getLangueDefaut();
			for(int i=0;i<m_xmlGeneral.getNbBalise("langue");i++)
			{
				String langueItem=m_xmlGeneral.getElement("langue", i);
				JMenuItem item  = new JRadioButtonMenuItem(langueItem);
				item.addActionListener(new ActionLangue(this, i));
				types.add(item);
				m_langue.add(item);
				if(langueItem.equals(langueDefaut))item.setSelected(true);
			}
			m_fichier.add(m_langue);
		}
		m_langue.setText(getXmlLangue().getElement("Mlang"));
		return m_langue;
	}

	/** @return le menu ouvrir */
	private JMenuItem getOuvrir()
	{
		if(m_ouvrir==null)
		{
			m_ouvrir = new JMenuItem();
			m_ouvrir.addActionListener(new ActionOuvrir(m_gestionnaire));
			m_fichier.add(m_ouvrir);
		}
		m_ouvrir.setText(getXmlLangue().getElement("Mouvrir"));
		return m_ouvrir;
	}

	/** @return le menu fermer */
	private JMenuItem getFermer()
	{
		if(m_fermer==null)
		{
			m_fermer = new JMenuItem();
			m_fermer.addActionListener(new ActionInternalClosed(m_gestionnaire));
			m_fichier.add(m_fermer);
		}
		m_fermer.setEnabled(!(m_donneesParse instanceof NullDonneeParse));
		m_fermer.setText(getXmlLangue().getElement("Mfermer"));
		return m_ouvrir;
	}

	/** @return le menu enregistrer */
	private JMenuItem getEnregistrer()
	{
		if(m_enregistrer==null)
		{
			m_enregistrer = new JMenuItem();
			m_enregistrer.addActionListener(new ActionEnregistrer(m_gestionnaire));
			m_fichier.add(m_enregistrer);
		}
		m_enregistrer.setEnabled(!(m_donneesParse instanceof NullDonneeParse));
		m_enregistrer.setText(getXmlLangue().getElement("Menregistrer"));
		return m_enregistrer;
	}

	/** @return le menu imprimer */
	private JMenuItem getPrint()
	{
		if(m_imprimer==null)
		{
			m_imprimer = new JMenuItem();
			m_imprimer.addActionListener(new ActionGestionnaire(m_gestionnaire)
			{
				public void actionPerformed(ActionEvent e) {
					m_gest=getM_gest();
					m_gest.getPanel().createChartPrintJob();}
			});
			m_fichier.add(m_imprimer);
		}
		m_imprimer.setEnabled(!(m_donneesParse instanceof NullDonneeParse));
		m_imprimer.setText(getXmlLangue().getElement("Print..."));
		return m_imprimer;
	}

	/** @return le menu exporter */
	private JMenuItem getExporter()
	{
		if(m_exporter==null)
		{
			m_exporter = new JMenuItem();
			m_exporter.addActionListener(new ActionExporter(m_gestionnaire));
			m_fichier.add(m_exporter);
		}
		m_exporter.setEnabled(isDonneesChargees());
		m_exporter.setText(getXmlLangue().getElement("Mexporter..."));
		return m_exporter;
	}

	/** @return le menu quitter */
	private JMenuItem getQuitter()
	{	
		if(m_quitter==null)
		{
			m_quitter = new JMenuItem();
			m_quitter.addActionListener(new ActionFermer(this));
			m_fichier.add(m_quitter);
			m_menu.add(m_fichier);
		}
		m_quitter.setText(getXmlLangue().getElement("Mquitter"));
		return m_quitter;
	}

	/** @return le menu configuration */
	private JMenu getConfig()
	{
		boolean construction=(m_configuration==null);
		if(construction)
		{
			m_configuration = new JMenu();
			m_menu.add(m_configuration);
		}
		m_configuration.setText(getXmlLangue().getElement("Mgraphe"));		
		if(isDonneesChargees())
		{
			m_configuration.setEnabled(true);
			//choix des axes
			getAxe();
			//@trovalet
			//choix de la taille des annotations
			getTailleAnnotations();
			//type de graphe
			getTypeGr();

			if(construction)  m_configuration.addSeparator();
			//habillage
			getHabillageFenetre();
			//afficher la légende
			getAfficherLegende();
			//affiche ou non la grille
			getGrille();

			if(construction)  m_configuration.addSeparator();
			//affiche les annotations
			getAnnot();
			//Affichage des fantome
			getFantome();
			//Affichage des partiels
			getPartiels();

		}
		else m_configuration.setEnabled(false);
		return m_configuration;
	}


	private JMenuItem getHabillageFenetre() {
		if(m_habillageFenetre==null){
			m_habillageFenetre=new JMenuItem(getXmlLangue().getElement("Mhabillage")+"...");
			m_habillageFenetre.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent e) {
					new FenetreHabillage(IHM.this,m_gestionnaire);
				}

			});
			m_habillageFenetre.setEnabled(isDonneesChargees() 
					&& m_gestionnaire.getHabilleur().isHabillagePossible(m_gestionnaire.getType()));
			m_configuration.add(m_habillageFenetre);
		}
		return m_habillageFenetre;

	}

	/** @return le menu type graphique */
	private JMenuItem getTypeGr()
	{
		if(m_typeGraphique==null)
		{
			m_typeGraphique = new JMenu();
			m_configuration.add(m_typeGraphique);
		}
		//ne l'activer que si plusieurs types de graphique sont envisageables
		int nbRepPossibles=0;
		if(m_donneesParse.getGroupes().size() > 1)nbRepPossibles++;
		if(m_donneesParse.getIndividusMoyens().size() > 0)nbRepPossibles++;
		if(m_donneesParse.getVariablesMoyennes().size() > 0)nbRepPossibles++;
		if(m_donneesParse.getType()==DonneeParse.TypeAnalyse.DATAFRAME)nbRepPossibles=0;

		if(nbRepPossibles > 1)m_typeGraphique.setEnabled(true);
		else m_typeGraphique.setEnabled(false);

		ButtonGroup types=new ButtonGroup();

		//type graphique: Individu
		types.add(getIndividus());

		//type graphique: Variable
		if(m_donneesParse.getVariablesMoyennes().size() > 0)types.add(getVariables());

		//type graphique: Groupe
		if(m_donneesParse.getGroupes().size()>1) types.add(getGroupes());

		//type graphique: Axes partiels
		if(m_donneesParse.isAxesPart()) types.add(getAxesPart());


		//type graphique: dataframe			
		if(m_donneesParse.isRepDataframePossible())types.add(getDataframe());

		if(m_gestionnaire.getType()==Gestionnaire.INDIVIDUS) m_individus.setSelected(true);
		else if(m_gestionnaire.getType()==Gestionnaire.VARIABLES) m_variables.setSelected(true);
		else if(m_gestionnaire.getType()==Gestionnaire.GROUPES) m_groupes.setSelected(true);
		else if(m_gestionnaire.getType()==Gestionnaire.AXES_PARTIELS) m_axesPart.setSelected(true);
		else if(m_gestionnaire.getType()==Gestionnaire.DATAFRAME) m_dataframe.setSelected(true);

		m_typeGraphique.setText(getXmlLangue().getElement("MtypeGR"));
		return m_typeGraphique;
	}


	/** @return la radio individu */
	private JRadioButtonMenuItem getIndividus() {
		if(m_individus==null) {
			m_individus = new JRadioButtonMenuItem();
			m_individus.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent arg0)
				{
					m_gestionnaire.setType(Gestionnaire.INDIVIDUS);
					updateMenuBarre();			
				}
			});
			m_typeGraphique.add(m_individus);
		}
		m_individus.setSelected(m_gestionnaire.getType()==Gestionnaire.INDIVIDUS);
		m_individus.setText(getXmlLangue().getElement("MtypeGRInd"));
		return m_individus;
	}

	/** @return la radio variable */
	private JRadioButtonMenuItem getVariables()
	{
		if(m_variables==null)
		{
			m_variables = new JRadioButtonMenuItem();
			m_variables.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent arg0)
				{
					m_gestionnaire.setType(Gestionnaire.VARIABLES);
					updateMenuBarre();
				}
			});
			m_typeGraphique.add(m_variables);

		}
		m_variables.setSelected(m_gestionnaire.getType()==Gestionnaire.VARIABLES);
		m_variables.setText(getXmlLangue().getElement("MtypeGRVar"));
		return m_variables;
	}

	/** @return la radio groupe */
	private JRadioButtonMenuItem getGroupes()
	{
		if(m_groupes==null)
		{
			m_groupes = new JRadioButtonMenuItem();
			m_groupes.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent arg0)
				{
					m_gestionnaire.setType(Gestionnaire.GROUPES);
					updateMenuBarre();			
				}
			});
			m_typeGraphique.add(m_groupes);
		}
		m_groupes.setSelected(m_gestionnaire.getType()==Gestionnaire.GROUPES);
		m_groupes.setText(getXmlLangue().getElement("MtypeGRGr"));
		return m_groupes;
	}

	/** @return la radio axes partiels */
	private JRadioButtonMenuItem getAxesPart(){
		if(m_axesPart==null)
		{
			m_axesPart = new JRadioButtonMenuItem();
			m_axesPart.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent arg0)
				{
					m_gestionnaire.setType(Gestionnaire.AXES_PARTIELS);
					updateMenuBarre();			
				}
			});
			m_typeGraphique.add(m_axesPart);
		}
		m_axesPart.setSelected(m_gestionnaire.getType()==Gestionnaire.AXES_PARTIELS);
		m_axesPart.setText(getXmlLangue().getElement("MtypeGRAxesPart"));
		return m_axesPart;
	}


	private AbstractButton getDataframe() {
		if(m_dataframe==null)
		{
			m_dataframe = new JRadioButtonMenuItem();
			m_dataframe.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent arg0)
				{
					m_gestionnaire.setType(Gestionnaire.DATAFRAME);
					updateMenuBarre();			
				}
			});
			m_typeGraphique.add(m_dataframe);
		}
		m_dataframe.setSelected(m_gestionnaire.getType()==Gestionnaire.DATAFRAME);
		m_dataframe.setText(IHM.getXmlLangue().getElement("MtypeGRAxesDataframe"));
		return m_dataframe;
	}

	/** @return le menu axe */
	private JMenuItem getAxe() {
		if(m_donneesParse.getNbAxe() > 2 || m_gestionnaire.getType()==Gestionnaire.DATAFRAME)
		{
			if(m_axe==null)
			{
				m_axe = new JMenuItem();
				m_axe.addActionListener(new ActionChoixAxes(this, m_donneesParse.getNbAxe(), m_donneesParse.getInerties(),m_gestionnaire));
				m_configuration.add(m_axe);
			}
			m_axe.setEnabled(true);
			m_axe.setText(getXmlLangue().getElement("Maxe"));
		}
		else m_axe = null;
		return m_axe;
	}
	
	//@trovalet
	/** @return le menu taille annotations */
	private JMenuItem getTailleAnnotations() {
		if (m_tailleAnnotations==null) {
			m_tailleAnnotations = new JMenuItem();
			m_tailleAnnotations.addActionListener(new ActionChoixTailleAnnotations(this, m_gestionnaire));
			// ajoute dans le menu configuration
			m_configuration.add(m_tailleAnnotations);
		}
		m_tailleAnnotations.setEnabled(true);
		m_tailleAnnotations.setText(getXmlLangue().getElement("MtailleAnnotations"));
		return m_tailleAnnotations;
	}


	private JCheckBoxMenuItem getAfficherLegende() {
		if(m_affLegende==null){
			m_affLegende=new JCheckBoxMenuItem();
			m_affLegende.addActionListener(new ActionListener(){

				public void actionPerformed(ActionEvent arg0) {
					m_gestionnaire.setAffLegende(!m_gestionnaire.isAffLegende());
				}

			});
			m_configuration.add(m_affLegende);
		}
		m_affLegende.setText(getXmlLangue().getElement("Mlegende"));
		m_affLegende.setSelected(m_gestionnaire.isAffLegende());
		return m_affLegende;
	}

	/** @return la case é cocher afficher partiel */
	protected JCheckBoxMenuItem getPartiels()
	{

		if(m_partiels==null) {
			m_partiels = new JCheckBoxMenuItem();
			m_partiels.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent arg0)
				{
					boolean nouvel_etat=!m_gestionnaire.isAffPartielsDonnesEnCours();
					m_gestionnaire.setAffPartielsDonnesEnCours(nouvel_etat);
					updateMenuBarre();
				}
			});
			m_configuration.add(m_partiels);
		}
		m_partiels.setText(getXmlLangue().getElement("MPartiels"));
		m_partiels.setEnabled(isActionPartielPossible());
		m_partiels.setSelected(m_gestionnaire.isAffPartielsDonnesEnCours());
		return m_partiels;
	}

	/** @return la case é cocher afficher fantome */
	private JCheckBoxMenuItem getFantome() {
		if(m_fantome==null) {
			m_fantome = new JCheckBoxMenuItem();
			m_fantome.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					boolean nouvel_etat=!m_gestionnaire.isContientFantomeDonnesEnCours();
					m_gestionnaire.setFantomeDonnesEnCours(nouvel_etat);
					updateMenuBarre();
				}
			});
			m_configuration.add(m_fantome);
		}
		m_fantome.setSelected(m_gestionnaire.isContientFantomeDonnesEnCours());
		m_fantome.setText(getXmlLangue().getElement("Mfantome"));
		m_fantome.setEnabled(isDonneesChargees());
		return m_fantome;
	}


	/** @return la case é cocher afficher grille */
	private JCheckBoxMenuItem getGrille()
	{
		if(isDonneesChargees())
		{
			if(m_grille==null)
			{
				m_grille = new JCheckBoxMenuItem();
				m_grille.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						m_gestionnaire.setAffGrid(!m_gestionnaire.isAffGrid());
						updateMenuBarre();
					}
				});
				m_configuration.add(m_grille);
			}
			m_grille.setSelected(m_gestionnaire.isAffGrid());
			m_grille.setEnabled(m_gestionnaire.getType()!=Gestionnaire.GROUPES);
			m_grille.setText(getXmlLangue().getElement("Mgrid"));
		}
		return m_grille;
	}

	/** @return lla case é cocher afficher annotation */
	private JCheckBoxMenuItem getAnnot()
	{
		if(isDonneesChargees())
		{
			if(m_annotations==null)
			{
				m_annotations = new JCheckBoxMenuItem();
				m_annotations.addActionListener(new ActionChangerAnnotations(m_gestionnaire,m_annotations));
				m_configuration.add(m_annotations);
			}
			m_annotations.setEnabled(isActionAnnotPossible());
			m_annotations.setSelected(m_gestionnaire.isAnnotationsDonnesEnCours());
			m_annotations.setText(getXmlLangue().getElement("Mannotations"));
		}
		return m_annotations;
	}


	/** @return le menu déselection */
	private JMenuItem getDeselect()
	{
		if(isDonneesChargees())
		{
			if(m_deselect==null)
			{
				m_deselect = new JMenuItem();
				m_deselect.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{m_gestionnaire.rmSelectionnes();}
				});
				m_selection.add(m_deselect);
			}
			m_deselect.setText(getXmlLangue().getElement("Mdeselectionner"));
		}
		return m_deselect;	
	}

	/** @return le menu inverser selection */
	private JMenuItem getInverser()
	{
		if(isDonneesChargees())
		{
			if(m_inverser==null)
			{
				m_inverser = new JMenuItem();
				m_inverser.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						getBtAnnot().setEnabled(true);
						if(m_gestionnaire.getType()!=Gestionnaire.INDIVIDUS)
							m_btPartiel.setEnabled(false);
						else m_btPartiel.setEnabled(true);
						m_gestionnaire.inverserSelectionnes();
					}
				});
				m_selection.add(m_inverser);
			}
			m_inverser.setText(getXmlLangue().getElement("MinverserSelection"));
		}
		return m_inverser;	
	}

	/** @return le menu selection par liste */
	private JMenuItem getSelectParListe()
	{
		if(isDonneesChargees())
		{
			if(m_lstSelection==null)
			{
				m_lstSelection = new JMenuItem();
				m_lstSelection.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{new FenetreSelection(IHM.this,m_donneesParse,m_gestionnaire);}
				});
				m_selection.add(m_lstSelection);
			}
			m_lstSelection.setText(getXmlLangue().getElement("MselectListe")+"...");
		}
		return m_lstSelection;		
	}

	/** @return le menu selection */
	private JMenu getSelection()
	{
		if(m_selection==null)
		{
			m_selection = new JMenu();
			m_menu.add(m_selection);
		}
		m_selection.setText(getXmlLangue().getElement("Tselect"));

		if(isDonneesChargees())
		{
			m_selection.setEnabled(true);
			getDeselect();
			getInverser();
			getSelectParListe();
		}
		else
			m_selection.setEnabled(false);
		return m_selection;
	}

	/** instancie ou met à jour la barre de menu 
	 * @return la barre de menu
	 */
	public JMenuBar updateMenuBarre()
	{
		/** Création de la barre de menu */
		getMenu();

		/** Menu fichier */
		getFichier();

		/** Menu configuration */
		getConfig();


		/** Menu Selection */
		getSelection();

		/** Barre d'icones */
		getToolbar();

		getToolBarAffinage();

		return m_menu;
	}

	private void updateContenu() {
		int w = 0,h = 0;
		w=(int)getWidth()-100;
		h=(int)getHeight()-100;

		setContentPane(getContenu());
		// show
		if(!(m_donneesParse instanceof NullDonneeParse))
		{
			getChartPanel().setMaximumDrawWidth(w);
			getChartPanel().setMaximumDrawHeight(h-m_toolbar.getHeight()-m_menu.getHeight());
		}
		setVisible(true);
	}

	private JPanel getContenu() {
		// menu
		setJMenuBar(updateMenuBarre());
		updateLangTextesDefaut();
		//paneau
		JPanel contenu = new JPanel();
		contenu.setLayout(new BorderLayout());	
		if(!(m_donneesParse instanceof NullDonneeParse))
		{
			m_gestionnaire.updateGraphe();
			//FenetreErreur.display("changement onglet"); @trovalet 2012-03-06
			m_tabbedPane = getJTabbedPane();
			contenu.add(m_tabbedPane, BorderLayout.CENTER);
		}
		contenu.add(m_toolbar, BorderLayout.NORTH);
		contenu.add(m_toolbarAffinage,BorderLayout.WEST);
		return contenu;
	}


	private JTabbedPane getJTabbedPane() {
		if(m_tabbedPane==null){
			m_tabbedPane = new JTabbedPane();
			m_correspondance_tab_types = new int[Gestionnaire.NB_TYPES_MAX];
			int i=0;

			//type graphique: Individu
			if(m_donneesParse.getType()!=DonneeParse.TypeAnalyse.DATAFRAME){
				JPanel panelInd = new JPanel();
				m_tabbedPane.addTab(getXmlLangue().getElement("MtypeGRInd"),panelInd);
				m_correspondance_tab_types[i]=Gestionnaire.INDIVIDUS;
				i++;

				//type graphique: Variable
				if(m_donneesParse.getVariablesMoyennes().size() > 0){
					JPanel panelVar = new JPanel();
					m_tabbedPane.addTab(getXmlLangue().getElement("MtypeGRVar"),panelVar);
					m_correspondance_tab_types[i]=Gestionnaire.VARIABLES;
					i++;
				}

				//type graphique: Groupe
				if(m_donneesParse.getGroupes().size()>1){
					JPanel panelGr = new JPanel();			
					m_tabbedPane.addTab(getXmlLangue().getElement("MtypeGRGr"),panelGr);
					m_correspondance_tab_types[i]=Gestionnaire.GROUPES;
					i++;
				}

				//type graphique: Axes partiels			
				if(m_donneesParse.isAxesPart()){
					JPanel panelAxes = new JPanel();
					m_tabbedPane.addTab(getXmlLangue().getElement("MtypeGRAxesPart"),panelAxes);
					m_correspondance_tab_types[i]=Gestionnaire.AXES_PARTIELS;
					i++;
				}
			}

			//type graphique: dataframe			
			if(m_donneesParse.isRepDataframePossible()){
				JPanel panelDataframe = new JPanel();			
				m_tabbedPane.addTab(IHM.getXmlLangue().getElement("MtypeGRAxesDataframe"),panelDataframe);
				m_correspondance_tab_types[i]=Gestionnaire.DATAFRAME;
				i++;
			}


			int index=-1;
			for (int j = 0; j < m_correspondance_tab_types.length && index==-1; j++) {
				if(m_gestionnaire.getType()==m_correspondance_tab_types[j]) index=j;
			}

			m_tabbedPane.setComponentAt(index, m_chartPanel);
			m_tabbedPane.setSelectedIndex(index);
			m_tabbedPane.addChangeListener(new ChangeListener(){								
				public void stateChanged(ChangeEvent e) {					
					int index =((JTabbedPane)e.getSource()).getSelectedIndex();
					if(index>=0){					
						m_gestionnaire.setType(m_correspondance_tab_types[index]);
						//2012-03-14 @trovalet supprimer les fenetres visibles venant autre onglet (FenetreDescriptionContinue)
						FenetreDescriptionContinue.masquer();
						//2012-03-16 remettre l'habillage car aucune fenetre ouverte
						m_gestionnaire.setDescription(null);
						m_gestionnaire.sauverAnnotations();
						m_gestionnaire.updateGraphe();
					}
				}

			});
		}
		return m_tabbedPane;
	}


	/**
	 * @return le décalage entre le bord de la fenêtre et le début du graphe
	 */
	public Point getDecGraphe() {
		if(m_toolbar!=null){
			Rectangle b=getChartPanel().getBounds();
			return b.getLocation();
		}else{
			return new Point(0,0);
		}
	}

	/** arréte dynGraph */
	public void quitter()
	{
		if (getTitle().equals("dynGraph")) {
			dispose();
			new RuntimeException("exit");
		}
		if(!m_goByR)System.exit(0);	    
	}

	/**
	 * main
	 * @param args
	 */
	public static void main(String[] args)
	{
		try{
			//Le programme regarde où il se trouve
			String dir;
			if(args.length<1) {
				dir=new File("").getAbsolutePath()+File.separator;
			}else
				dir=args[0];
			IHM.m_dir=dir;
			//Le programme regarde le fichier avec lequel il doit s'ouvrir
			String fichier;
			if(args.length<2) {
				fichier=null;
			}else
				fichier=args[1];
			//Le programme charge les données du fichier
			DonneeParse donnees;
			if(fichier!=null) {
				Registre reg=Fichier.chargerDonnees(fichier);
				donnees=reg.getDonnee();
				try {
					donnees.verifierIntegrite();
				} catch (ExceptionIntegrite e) {
					new FenetreErreur(e);
				}
				if(args.length==3 && args[2].equals("-t")){
					File f=new File(fichier);
					f.delete();
				}
			}else {
				donnees=new NullDonneeParse();
			}
			IHM ihm=new IHM(donnees);
			ihm.m_goByR=false;
			ihm.demarrer();			
		}catch(Exception e){
			new FenetreErreur(e);
		}
	}

	/**
	 * @return la langue en cours
	 */
	public static XML getXmlLangue() {
		if(m_xmlLangue==null){
			XML xmlGeneral = new XML(m_dir+"langue"+File.separator+"langue.xml");
			String defaut = xmlGeneral.getDefaut();
			int max = xmlGeneral.getNbBalise("langue");
			int i = 0;
			boolean test = false;
			while (i < max && !test) {
				String chaine = xmlGeneral.getElement("id", i);
				if (defaut.compareTo(chaine) == 0) {
					m_xmlLangue=new XML(m_dir+"langue"+File.separator + chaine + ".xml");
					test = true;
				}
				i++;
			}
		}
		return m_xmlLangue;
	}
	
}

