package ihm;

import java.awt.Component;
import java.awt.FlowLayout;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;

import outils.Constantes;
import don.DonneeID;
import don.Modalite;

/** Classe pour afficher des DonneeID sous forme de JLabel dans une JListe
 * 
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class RenduDonneeID extends JPanel implements ListCellRenderer
{
    private static final long serialVersionUID = -4503481361185338287L;
    private Gestionnaire m_gestionnaire;
   
    private JLabel icone_fantome;
    private JLabel icone_illus;
    private JLabel texte;
//    private String texteModaliteVariable=null;
        
    /**
     * @param gestionnaire
     */
    public RenduDonneeID(Gestionnaire gestionnaire) {
	m_gestionnaire=gestionnaire;
	setLayout(new FlowLayout(FlowLayout.LEADING,0,0));
	icone_fantome    = new JLabel();
	icone_illus   = new JLabel();
	texte    = new JLabel();
        icone_fantome.setOpaque(true);
        icone_illus.setOpaque(true);
        texte.setOpaque(true);
        texte.setHorizontalTextPosition(JLabel.LEFT);
        add(icone_fantome);
        add(icone_illus);
        add(texte); 
    }

    /** @return un label */
    public Component getListCellRendererComponent(JList arg0, Object element,
	    int arg2, boolean arg3, boolean arg4)
    {
	if(element instanceof DonneeID) {
	    String statut="";
	    if(m_gestionnaire.isFantome(((DonneeID)element))) {
	    	icone_fantome.setIcon(new ImageIcon(IHM.m_dir+Constantes.ICONE_FANTOME));
	    	statut+=IHM.getXmlLangue().getElement("LInvisible");
	    }else {
	    	icone_fantome.setIcon(new ImageIcon(IHM.m_dir+Constantes.ICONE_NON_FANTOME));
	    	statut+=IHM.getXmlLangue().getElement("LVisible");
	    }
	    statut+=" "+IHM.getXmlLangue().getElement("et")+" ";
	    
	    if(((DonneeID)element).isActive()) {
	    	icone_illus.setIcon(new ImageIcon(IHM.m_dir+Constantes.ICONE_ACTIF));
	    	statut+=IHM.getXmlLangue().getElement("CactifMF");
	    }else {
	    	icone_illus.setIcon(new ImageIcon(IHM.m_dir+Constantes.ICONE_ILLUSTRATIF));
	    	statut+=IHM.getXmlLangue().getElement("CillusMF");
	    }
	    this.setToolTipText(statut);
	    
	    if(element instanceof Modalite) {
	    	Modalite mod=(Modalite)element;
	    	texte.setText(mod.getNominale().getAnnotation()+" : "+mod.getAnnotation());
//	    	texteModaliteVariable = mod.getAnnotation()+" : "+mod.getNominale().getAnnotation();		
	    }else
	    	texte.setText(((DonneeID)element).getAnnotation());
	}
	return this;
    }

//    /**
//     * @return annotation modalité puis variable
//     */
//    public String getTexteModaliteVariable() {
//		return texteModaliteVariable;
//	}

	/**
	 * @return annotation
	 */
	public String getText() {
	return texte.getText();
    }
}
