package org.jfree.chart.annotations;

import java.awt.Color;
import java.io.Serializable;
import org.jfree.util.PublicCloneable;

/** libelle avec xy
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class XYIDAnnotation extends XYTextAnnotation implements Cloneable, PublicCloneable, Serializable
{
	private static final long serialVersionUID = -2166877245792880830L;
	
	private Color m_couleur;
	
	/** the x point coordinate */
	private double m_Ptsx;
	
	/** the y point coordinate */
	private double m_Ptsy;
	
	/** modify the x label coordinate 
	 * @param lblX x of the label (double)
	 * */
	public void setLblx(double lblX){setX(lblX);}
	
	/** modify the y label coordinate 
	 * @param lblY y of the label (double)
	 * */
	public void setLbly(double lblY){setY(lblY);}
	
	/** return the x label coordinate 
	 * @return un double*/
	public double getLblx(){return getX();}
	
	/** return the y label coordinate 
	 * @return un double*/
	public double getLbly(){return getY();}
	
	/** modify the x label coordinate 
	 * @param ptsX x of the label (double)
	 * */
	public void setPtsx(double ptsX){m_Ptsx=ptsX;}
	
	/** modify the y label coordinate 
	 * @param ptsY y of the label (double)
	 * */
	public void setPtsy(double ptsY){m_Ptsy=ptsY;}
	
	/** return the x point coordinate 
	 * @return un double*/
	public double getPtsx(){return m_Ptsx;}
	
	/** return the y point coordinate 
	 * @return un double*/
	public double getPtsy(){return m_Ptsy;}
	
	/** setteur annotation 
	 * @param annot */
	public void setAnnot(String annot){super.setText(annot);}
	
	/** getteur annotation 
	 * @return une chaine de charactere */
	public String getAnnot(){return super.getText();}
	
	/**
	 * @return la couleur
	 */
	public Color getColor(){return m_couleur;}//(Color)getPaint();}
	
	/**
	 * @param couleur
	 */
	public void setColor(Color couleur){m_couleur=couleur;setPaint(couleur);}
	
	/**
	* Creates a new label and arrow annotation.
	*
	* @param label  the label (<code>null</code> permitted).
	* @param lblX  the x label coordinate (measured against the chart's domain axis).
	* @param lblY  the y label coordinate (measured against the chart's range axis).
	* @param ptsX  the x point coordinate (measured against the chart's domain axis).
	* @param ptsY  the y point coordinate (measured against the chart's range axis).
	 * @param couleur de l'annotation
	*/
	public XYIDAnnotation(String label, double lblX, double lblY, double ptsX, double ptsY, Color couleur)
	{	
		super(label, lblX, lblY);
		setPtsx(ptsX);setPtsy(ptsY);
		setColor(couleur);
	}
	
	/**
	* Creates a new label and arrow annotation.
	*
	* @param label  the label (<code>null</code> permitted).
	* @param lblX  the x label coordinate (measured against the chart's domain axis).
	* @param lblY  the y label coordinate (measured against the chart's range axis).
	* @param ptsX  the x point coordinate (measured against the chart's domain axis).
	* @param ptsY  the y point coordinate (measured against the chart's range axis).
	*/
	public XYIDAnnotation(String label, double lblX, double lblY, double ptsX, double ptsY)
	{this(label,lblX,lblY,ptsX,ptsY,(Color)XYTextAnnotation.DEFAULT_PAINT);}
}

