package org.jfree.chart.demo;

import java.awt.Dimension;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.PolarChartPanel;
import org.jfree.chart.plot.PolarPlot;
import org.jfree.chart.renderer.DefaultPolarItemRenderer;
import org.jfree.data.xy.XYDataset;
import org.jfree.experimental.data.xy.VectorSeries;
import org.jfree.experimental.data.xy.VectorSeriesCollection;
import org.jfree.ui.ApplicationFrame;
import org.jfree.ui.RefineryUtilities;

/**
 * <code>PolarChartDemo</code> demonstrates the capabilities of the
 * {@link PolarPlot}.
 * 
 * @author Daniel Bridenbecker, Solution Engineering, Inc.
 */
public class PolarChartDemo extends ApplicationFrame {

	private static final long serialVersionUID = 2019237349040333685L;

	/**
	 * Creates a new instance of the demo.
	 * 
	 * @param title
	 *            the frame title.
	 */
	public PolarChartDemo(final String title) {
		super(title);
		final XYDataset dataset = createDataset();
		final JFreeChart chart = createChart(dataset);
		final ChartPanel chartPanel = new PolarChartPanel(chart);
		chartPanel.setPreferredSize(new Dimension(500, 270));
		chartPanel.setEnforceFileExtensions(false);
		setContentPane(chartPanel);
	}

	/**
	 * Creates a sample dataset.
	 * 
	 * @return A sample dataset.
	 */
	private XYDataset createDataset() {
		/*
		 * final XYSeriesCollection data = new XYSeriesCollection(); final
		 * XYSeries series1 = createRandomData("Series 1", 75.0, 10.0); final
		 * XYSeries series2 = createRandomData("Series 2", 50.0, 5.0); final
		 * XYSeries series3 = createRandomData("Series 3", 25.0, 1.0);
		 * data.addSeries(series1); data.addSeries(series2);
		 * data.addSeries(series3); return data;
		 */
		VectorSeries mySerie = new VectorSeries("");
		mySerie.add(0, 0, 1, 0);
		VectorSeriesCollection myData = new VectorSeriesCollection();
		return myData;
	}

	/**
	 * Creates a sample chart.
	 * 
	 * @param dataset
	 *            the dataset.
	 * 
	 * @return A sample chart.
	 */
	private JFreeChart createChart(final XYDataset dataset) {
		final JFreeChart chart = ChartFactory.createPolarChart(
				"Polar Chart Demo", dataset, true, true, false);
		final PolarPlot plot = (PolarPlot) chart.getPlot();
		final DefaultPolarItemRenderer renderer = (DefaultPolarItemRenderer) plot
				.getRenderer();
		renderer.setSeriesFilled(0, false);
		return chart;
	}

//	/** main
//	 * @param argv
//	 */
//	public static void main(final String[] argv) {
//
//		final PolarChartDemo demo = new PolarChartDemo("Polar Chart Demo");
//		demo.pack();
//		RefineryUtilities.centerFrameOnScreen(demo);
//		demo.setVisible(true);
//
//	}

}
