package org.jfree.chart.plot;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.Serializable;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.data.Range;
import org.jfree.data.xy.XYDataset;

/** Classe qui cr�er le cercle de corr�lation
 *  @author michaelory@gmail.com
 *  @author julien.durand.eu@gmail.com
 *
 */
public class VectPiePlot extends XYPlot implements Cloneable, Serializable
{
    private static final long serialVersionUID = -477525209363105945L;

    /** trace ou pas le cerlce */
    private boolean m_pie;
    
    /** Creates a new plot.
     *
     * @param dataset  the dataset (<code>null</code> permitted)
     * @param domainAxis  the domain axis (<code>null</code> permitted)
     * @param rangeAxis  the range axis (<code>null</code> permitted)
     * @param pie variable orthonorm� ou non
     */
    public VectPiePlot(XYDataset dataset,ValueAxis domainAxis,ValueAxis rangeAxis, boolean pie)
    {
    	super(dataset,domainAxis,rangeAxis,null);
    	m_pie=pie;
    }

    /**
     * Draws a pie
     * @param g2 graphics
     * @param dataArea  the data plot area.
     */
    public void drawPie(Graphics2D g2,Rectangle2D dataArea)
    {
		//couleur cerlce
		g2.setPaint(Color.black);
		//on trace le cercle tjrs en fonction de la marge
		int x=(int)getDomainAxis().valueToJava2D(-1, dataArea, getDomainAxisEdge());
		int y=(int)getRangeAxis().valueToJava2D(1, dataArea, getRangeAxisEdge());
		int d=(int)getDomainAxis().valueToJava2D(1, dataArea, getDomainAxisEdge())-x;
		g2.drawOval(x, y, d, d);		
    }

    public void draw(Graphics2D g2, Rectangle2D area, Point2D anchor, PlotState parentState, PlotRenderingInfo info) {
		draw(g2, area, anchor, parentState, info, 1);
	}

	public void draw(Graphics2D g2, Rectangle2D area, Point2D anchor, PlotState parentState, PlotRenderingInfo info, double rapport) {
		ValueAxis horizontalAxis = getDomainAxis();
		ValueAxis verticalAxis = getRangeAxis();
		if(m_pie)
		{
			verticalAxis.setRange(new Range(-1.1, 1.1), false, false);
			horizontalAxis.setRange(new Range(-1.1, 1.1), false, false);
		}
		super.draw(g2, area, anchor, parentState, info, rapport);
		if(m_pie)drawPie(g2, dataArea);
    }
}