/* ===========================================================
 * JFreeChart : a free chart library for the Java(tm) platform
 * ===========================================================
 *
 * (C) Copyright 2004, by Michel Santos
 *
 * Project Info:  http://www.jfree.org/jfreechart/index.html
 *
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 *
 * Created on May 19, 2004, 11:19 AM
 */


package org.jfree.chart.plot.util;


import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.axis.CategoryAxis;



/**
 * A class intended for very easy plotting of XY data
 *
 * @author  Michel Santos
 * @since 0.9.20
 */
public class SimpleBarChart {
  
  public SimpleBarChart(org.jfree.data.category.CategoryDataset dataset) {
    init(dataset);
  }
  
  public SimpleBarChart(double[] values, String[] categoryNames, String seriesName) {
    // Safety-check
    if (values == null || categoryNames == null) {
      throw new IllegalArgumentException("Valid values and category names must be provided");      
    }
    if (values.length != categoryNames.length) {
      throw new IllegalArgumentException("The number of values must equal the number of category names");
    }
    
    // create the dataset ...
    DefaultCategoryDataset dataset = new DefaultCategoryDataset();

    int iNbrValues = values.length;
    for (int i = 0; i < iNbrValues; i++) {
      double dValue = values[i];
      String strCategoryName = categoryNames[i];
      dataset.addValue(dValue, seriesName, strCategoryName);
    }
    
    init(dataset);
  }

  public SimpleBarChart(double[][] values, String[][] categoryNames, String[] seriesNames) {
    // Safety-check
    if (values == null || categoryNames == null || seriesNames == null) {
      throw new IllegalArgumentException("Valid values, category, and series names must be provided");      
    }
    if (seriesNames.length != values.length) {
      throw new IllegalArgumentException("The number of values must equal the number of series names");
    }
    if (seriesNames.length != categoryNames.length) {
      throw new IllegalArgumentException("The number of values must equal the number of series names");
    }
    int iNbrSeries = seriesNames.length;
    for (int i = 0; i < iNbrSeries; i++) {
      int iNbrSeriesValues = values[i].length;      
      int iNbrCategoryValues = categoryNames[i].length;
      
      if (iNbrSeriesValues != iNbrCategoryValues) {
        throw new IllegalArgumentException("In series " + i + " the number of values must equal the number of series names");
      }
    }

    
    // create the dataset ...
    DefaultCategoryDataset dataset = new DefaultCategoryDataset();
    
    for (int i = 0; i < iNbrSeries; i++) {
      double[] dAryValues = values[i];
      String[] strAryCategoryNames = categoryNames[i];
      String strSeriesName = seriesNames[i];
      
      int iNbrValues = dAryValues.length;
      for (int j = 0; j < iNbrValues; j++) {              
        double dValue = dAryValues[j];
        String strCategoryName = strAryCategoryNames[j];
        dataset.addValue(dValue, strSeriesName, strCategoryName);
      }    
    }

    
    init(dataset);
  }  
  
  
  
  public org.jfree.chart.JFreeChart chart;
  private void init(org.jfree.data.category.CategoryDataset dataset) {
    String strTitle = "";
    chart = org.jfree.chart.ChartFactory.createBarChart(strTitle, "Category", "Value", dataset, org.jfree.chart.plot.PlotOrientation.VERTICAL, true, true, false);
//    chart = createLineShapePlot(strTitle, strXLabel, strYLabel, colxn, org.jfree.chart.plot.PlotOrientation.VERTICAL, false, false, false);
//    chart.setBackgroundPaint(java.awt.Color.WHITE);

  }

  
  public void setTitle(String title) {
    chart.setTitle(title);    
    
    if (frame != null) {
      frame.setTitle(title);
    }
  }

  private SimpleFrame frame = null;
  public void display() {
    if (frame == null) {
      frame = new SimpleFrame(chart);
    }      
    //frame.pack();
    
    frame.setSize(600, 350);
    /**
     * Since the current version of SGT Beans doesn't directly support
     * resizing disable resizing and make the frame visible.
     */
    // frame.setResizable(false);
    frame.setVisible(true);
    
    frame.setTitle(chart.getTitle().getText());
  }
  
  public void saveAs(String fileName) throws java.io.IOException {
    java.io.File file = new java.io.File(fileName);
    
    /**
     * Size is measured in pixels
     * The smaller the image, the fewer tickmarks and labels from the axis are shown
     */
    // org.jfree.chart.ChartUtilities.saveChartAsPNG(file, chart, 300, 200);
    org.jfree.chart.ChartUtilities.saveChartAsPNG(file, chart, 600, 400);
  }  

  
  public void setCategoryLabel(String label) {
    ((CategoryPlot) chart.getPlot()).setDomainAxis(new CategoryAxis(label));
  }
  
  public void setValuesLabel(String label) {
    ((CategoryPlot) chart.getPlot()).setRangeAxis(new org.jfree.chart.axis.NumberAxis(label));
  }
  
  public void setWindowSize(int width, int height) {
    if (frame == null) {
      return;      
    }
    
    frame.setSize(width, height);
  }
}

