package outils;
import java.awt.Color;
import java.awt.Font;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Locale;

import org.jfree.chart.axis.NumberTickUnit;
import org.jfree.chart.axis.TickUnitSource;
import org.jfree.chart.axis.TickUnits;
import org.jfree.ui.RectangleEdge;


/**
 *  @author julien.durand.eu@gmail.com
 *  
 *  Classe qui contient toutes les constantes utiles
 *
 */
public class Constantes {

    /**
     * Couleurs par défaut dans les graphes 
     */    


	/** La couleur des variables quantitatives actives*/
    public static final Color COULEUR_VARIABLES_ACTIVES=Color.black;
    
    /** La couleur des variables quantitatives illustratives*/
    public static final Color COULEUR_VARIABLES_ILLUSTRATIVES = Color.blue;

    /** La couleur des groupes actifs*/
    public static final Color COULEUR_GROUPES_ACTIFS = Color.black;

    /** La couleur des groupes illustratifs*/
    public static final Color COULEUR_GROUPES_ILLUSTRATIFS = Color.blue;

    /** La couleur des individus actifs*/
    public static final Color COULEUR_INDIVIDUS_ACTIFS = Color.black;
    
    /** La couleur des individus illustratifs */
    public static final Color COULEUR_INDIVIDUS_ILLUSTRATIFS = Color.blue;

    /** La couleur des points sélectionnés */
    public static final Color COULEUR_SELECTION = Color.GREEN;

    /** La couleur des lignes actives en AC */
	public static final Color COULEUR_AC_LIGNE_ACTIVE = Color.blue;
	
    /** La couleur des colonnes actives en AC */
	public static final Color COULEUR_AC_COLONNE_ACTIVE = Color.red;
	
    /** La couleur des lignes illustratives en AC */
	public static final Color COULEUR_AC_LIGNE_ILLUSTRATIVE = new Color(8, 37, 103); //bleu foncé "saphire"
	
	/** La couleur des colonnes illustratives en AC */
	public static final Color COULEUR_AC_COLONNE_ILLUSTRATIVE = new Color(200, 8, 21);//rouge (foncé) "vénétien"

	/** La couleur des valeurs positives pour un habillage par continue*/
	public static final Color COULEUR_HABILLAGE_CONTINUE_POSITIF = Color.red;

	/** La couleur des valeurs negatives pour un habillage par continue*/
	public static final Color COULEUR_HABILLAGE_CONTINUE_NEGATIF = Color.blue;
	
	/** La couleur des valeurs égales à zéro pour un habillage par continue */
	public static final Color COULEUR_HABILLAGE_CONTINUE_ZERO = Color.gray;
    
	
    /**
     * Formes des points
     */
    
    /** spécifie la forme des individus	
     * -> cercle 
     */
    public final static Shape FORME_INDIVIDUS = new Ellipse2D.Double(-2, -2, 4, 4);

    /** spécifie la forme des modalite 
     * -> carré 
     */
    public final static Shape FORME_MODALITES = new Rectangle2D.Double(-2, -2, 4, 4);

    /** spécifie la forme des groupe actifs 
     * ->triangle à l'endroit
     */
    public final static Shape FORME_GROUPES_ACTIFS = new Polygon(new int[] {3,6,0},new int[]{0,5,5},3);

    /** spécifie la forme des groupe illustratifs 
     * -> triangle à l'envers
     */
    public final static Shape FORME_GROUPES_ILLUSTRATIFS = new Polygon(new int[] {0,3,6},new int[]{0,4,0},3);
    
    /**
     * 
     */
    public final static Shape FORME_ELEMENT_SOUS_SEUIL_AFFINAGE = new Rectangle2D.Double(0,0,1,1);
    
    
    
    /**
     * Formats
     */
    
    /** Le format des nombres sur les graphes*/
    public static final NumberFormat FORMAT_NOMBRES = NumberFormat.getNumberInstance(new Locale("en","US"));

    /** La position de la légende*/
    public static final RectangleEdge POSITION_LEGENDE = RectangleEdge.LEFT;

    /** La police des graduations des axes*/
    public static final Font POLICE_AXES = new Font("Arial", Font.PLAIN, 15);

    /** La police du titre du graphe*/
    public static final Font POLICE_TITRE = new Font("SansSerif", Font.BOLD, 18);    

    /** Taille maximale d'un cercle en pourcentage de la taille du graphe */
	public static final double TAILLE_MAX_CERCLE_EN_POURCENTAGE=0.04;

    
    /**
	 * Le nombre maximal de graduations 6
	 */
	public static final int NOMBRE_MAX_GRADUATIONS = 8;

	
	/**
	 * @return La liste des unités de graduations possibles
	 */
	public static TickUnitSource createStandardTickUnits() {
	
		TickUnits units = new TickUnits();
		DecimalFormat df0 = new DecimalFormat("0.00000000");
		DecimalFormat df1 = new DecimalFormat("0.0000000");
		DecimalFormat df2 = new DecimalFormat("0.000000");
		DecimalFormat df3 = new DecimalFormat("0.00000");
		DecimalFormat df4 = new DecimalFormat("0.0000");
		DecimalFormat df5 = new DecimalFormat("0.000");
		DecimalFormat df6 = new DecimalFormat("0.00");
		DecimalFormat df7 = new DecimalFormat("0.0");
		DecimalFormat df8 = new DecimalFormat("#,##0");
		DecimalFormat df9 = new DecimalFormat("#,###,##0");
		DecimalFormat df10 = new DecimalFormat("#,###,###,##0");
	
		// we can add the units in any order, the TickUnits collection will
		// sort them...
		units.add(new NumberTickUnit(0.0000001, df1));
		units.add(new NumberTickUnit(0.000001, df2));
		units.add(new NumberTickUnit(0.00001, df3));
		units.add(new NumberTickUnit(0.0001, df4));
		units.add(new NumberTickUnit(0.001, df5));
		units.add(new NumberTickUnit(0.01, df6));
		units.add(new NumberTickUnit(0.1, df7));
		units.add(new NumberTickUnit(1, df8));
		units.add(new NumberTickUnit(10, df8));
		units.add(new NumberTickUnit(100, df8));
		units.add(new NumberTickUnit(1000, df8));
		units.add(new NumberTickUnit(10000, df8));
		units.add(new NumberTickUnit(100000, df8));
		units.add(new NumberTickUnit(1000000, df9));
		units.add(new NumberTickUnit(10000000, df9));
		units.add(new NumberTickUnit(100000000, df9));
		units.add(new NumberTickUnit(1000000000, df10));
		units.add(new NumberTickUnit(10000000000.0, df10));
		units.add(new NumberTickUnit(100000000000.0, df10));
	
		units.add(new NumberTickUnit(0.00000015, df0));
		units.add(new NumberTickUnit(0.0000015, df1));
		units.add(new NumberTickUnit(0.000015, df1));
		units.add(new NumberTickUnit(0.00015, df3));
		units.add(new NumberTickUnit(0.0015, df4));
		units.add(new NumberTickUnit(0.015, df5));
		units.add(new NumberTickUnit(0.15, df6));
		units.add(new NumberTickUnit(1.5, df7));
		units.add(new NumberTickUnit(15, df8));
		units.add(new NumberTickUnit(150, df8));
		units.add(new NumberTickUnit(1500, df8));
		units.add(new NumberTickUnit(15000, df8));
		units.add(new NumberTickUnit(150000, df8));
		units.add(new NumberTickUnit(1500000, df9));
		units.add(new NumberTickUnit(15000000, df9));
		units.add(new NumberTickUnit(150000000, df9));
		units.add(new NumberTickUnit(1500000000.0, df10));
		units.add(new NumberTickUnit(15000000000.0, df10));
		units.add(new NumberTickUnit(150000000000.0, df10));
	
		units.add(new NumberTickUnit(0.0000002, df0));
		units.add(new NumberTickUnit(0.000002, df1));
		units.add(new NumberTickUnit(0.00002, df2));
		units.add(new NumberTickUnit(0.0002, df3));
		units.add(new NumberTickUnit(0.002, df4));
		units.add(new NumberTickUnit(0.02, df5));
		units.add(new NumberTickUnit(0.2, df6));
		units.add(new NumberTickUnit(2, df8));
		units.add(new NumberTickUnit(20, df8));
		units.add(new NumberTickUnit(200, df8));
		units.add(new NumberTickUnit(2000, df8));
		units.add(new NumberTickUnit(20000, df8));
		units.add(new NumberTickUnit(200000, df9));
		units.add(new NumberTickUnit(2000000, df9));
		units.add(new NumberTickUnit(20000000, df9));
		units.add(new NumberTickUnit(200000000.0, df10));
		units.add(new NumberTickUnit(2000000000.0, df10));
		units.add(new NumberTickUnit(20000000000.0, df10));
	
		units.add(new NumberTickUnit(0.00000025, df0));
		units.add(new NumberTickUnit(0.0000025, df1));
		units.add(new NumberTickUnit(0.000025, df2));
		units.add(new NumberTickUnit(0.00025, df3));
		units.add(new NumberTickUnit(0.0025, df4));
		units.add(new NumberTickUnit(0.025, df5));
		units.add(new NumberTickUnit(0.25, df6));
		units.add(new NumberTickUnit(2.5, df7));
		units.add(new NumberTickUnit(25, df8));
		units.add(new NumberTickUnit(250, df8));
		units.add(new NumberTickUnit(2500, df8));
		units.add(new NumberTickUnit(25000, df8));
		units.add(new NumberTickUnit(250000, df8));
		units.add(new NumberTickUnit(2500000, df9));
		units.add(new NumberTickUnit(25000000, df9));
		units.add(new NumberTickUnit(250000000, df9));
		units.add(new NumberTickUnit(2500000000.0, df10));
		units.add(new NumberTickUnit(25000000000.0, df10));
		units.add(new NumberTickUnit(250000000000.0, df10));
	
		units.add(new NumberTickUnit(0.0000005, df1));
		units.add(new NumberTickUnit(0.000005, df2));
		units.add(new NumberTickUnit(0.00005, df3));
		units.add(new NumberTickUnit(0.0005, df4));
		units.add(new NumberTickUnit(0.005, df5));
		units.add(new NumberTickUnit(0.05, df6));
		units.add(new NumberTickUnit(0.5, df7));
		units.add(new NumberTickUnit(5L, df8));
		units.add(new NumberTickUnit(50L, df8));
		units.add(new NumberTickUnit(500L, df8));
		units.add(new NumberTickUnit(5000L, df8));
		units.add(new NumberTickUnit(50000L, df8));
		units.add(new NumberTickUnit(500000L, df8));
		units.add(new NumberTickUnit(5000000L, df9));
		units.add(new NumberTickUnit(50000000L, df9));
		units.add(new NumberTickUnit(500000000L, df9));
		units.add(new NumberTickUnit(5000000000L, df10));
		units.add(new NumberTickUnit(50000000000L, df10));
		units.add(new NumberTickUnit(500000000000L, df10));
	
		units.add(new NumberTickUnit(0.00000075, df0));
		units.add(new NumberTickUnit(0.0000075, df1));
		units.add(new NumberTickUnit(0.000075, df2));
		units.add(new NumberTickUnit(0.00075, df3));
		units.add(new NumberTickUnit(0.0075, df4));
		units.add(new NumberTickUnit(0.075, df5));
		units.add(new NumberTickUnit(0.75, df6));
		units.add(new NumberTickUnit(7.5, df7));
		units.add(new NumberTickUnit(75, df8));
		units.add(new NumberTickUnit(750, df8));
		units.add(new NumberTickUnit(7500, df8));
		units.add(new NumberTickUnit(75000, df8));
		units.add(new NumberTickUnit(750000, df8));
		units.add(new NumberTickUnit(7500000, df9));
		units.add(new NumberTickUnit(75000000, df9));
		units.add(new NumberTickUnit(750000000, df9));
		units.add(new NumberTickUnit(7500000000.0, df10));
		units.add(new NumberTickUnit(75000000000.0, df10));
		units.add(new NumberTickUnit(750000000000.0, df10));
	
		return units;
	
	}
    
    /**
     * Chemins des icones
     */
    
    /** Icone de la fenêtre de dynGraph*/
    public static final String ICONE_FENETRE = "ico/Rlogo-5.jpg";

    /** Icone du bouton d'annotations*/
    public static final String ICONE_ARROW_UP = "ico"+ File.separator +"arrow-up.gif";
    
    /** Icone du bouton d'annotations*/
    public static final String ICONE_ARROW_DOWN = "ico"+ File.separator +"arrow-down.gif";
    
    /** Icone du bouton d'annotations*/
    public static final String ICONE_ANNOTATIONS = "ico"+ File.separator +"Annot.gif";

    /** Icone du bouton afficher/masquer la grille*/
    public static final String ICONE_GRID = "ico"+ File.separator +"Grid.gif";
    
    /** Icone du bouton habillage*/
	public static final String ICONE_HABILLAGE = "ico"+ File.separator +"potpeinture.jpg";

    /** Icone du bouton désélectionner tout*/
    public static final String ICONE_DESELECTION = "ico"+ File.separator +"ensemble__vide.jpg";

    /** Icone du bouton inverser la sélection*/
    public static final String ICONE_INVERSER = "ico"+ File.separator +"inverser.png";

    /** Icone du bouton sélectionner par liste*/
    public static final String ICONE_SELECTION_LISTE = "ico"+ File.separator +"liste.jpg";

    /** Icone du bouton masquer/afficher le point
     * Icone des données visibles 
     */
    public static final String ICONE_FANTOME = "ico"+ File.separator +"fantome2.jpg";

    /** Icone des données qui ne sont pas fantome */
    public static final String ICONE_NON_FANTOME = "ico"+ File.separator +"non_fantome2.jpg";
    
    
    /** Icone du bouton afficher/masquer les points partiels*/
    public static final String ICONE_PARTIELS = "ico"+ File.separator +"Partiel.gif";

    /** Icone des données actives */
    public static final String ICONE_ACTIF = "ico"+ File.separator +"actif.jpg";

    /** Icone des données illustratives */
    public static final String ICONE_ILLUSTRATIF = "ico"+ File.separator +"illustratif.jpg";

	/**
	 * Espace entre une annotation et son point
	 */
	public static final float ESPACE_ANNOTATION_POINT = 2;
	
	/**
	 * Le temps pendant lequel il faut laisser la souris sur une modalités pour habiller les autres selon le catdes
	 */
	public static final int TEMPS_AVANT_DESCRIPTION_PAR_QUALIFIABLE = 300;

	/**
	 * La valeur par défaut pour l'affinage
	 */
	public static final double VALEUR_AFFINAGE_DEFAUT = 0;

	/**
	 * 
	 */
	public static final double SEUIL_VTEST_0 = 0;	
	
}
