
import java.io.File;
import outils.Fichier;
import ihm.FenetreErreur;
import ihm.IHM;
import don.*;
import don.DonneeParse.TypeAnalyse;
import drawing.Registre;

/** 
 * classe qui s'occupe de construire DonneeParse
 * @author michaelory@gmail.com
 * @author julien.durand.eu@gmail.com
 * 
 * le srcipt initDynGraphe.R appelle les fonctions dans cet ordre 
 * 1- new(nom colonne Dataset,nom colonne Annot) 
 * 2- addVal(annot,donnée) 
 * 3- addHabVal(modalité) 
 * 4- generate
 */
public class parse
{
	/** stock les données */
	private DonneeParse m_don;

	/** tableau de nom de colonne */
	private String[] m_annotColName;

	/** numèro du groupe en cours de lecture */
	private int num_groupe_courant =1;	

	/** dernier individu moyen rencontré */
	private IndividuMoyen m_individuMoyen_old;

	/** dernière variable moyenne rencontrée */
	private ContinueMoyenne m_variableMoyenne_old;

	/** dernière modalité moyenne ajouté */
	private ModaliteMoyenne m_modaliteMoyenne_old;

	/** 
	 * constructeur (initialise le parser)
	 * 
	 * @param colName tableau de string qui contient le nom des colonnes
	 * @param dir 
	 */
	public parse(String colName[],String dir)
	{this(colName, null,dir);}

	/**
	 * constructeur (initialise le parser)
	 * 
	 * @param colName tableau de string qui contient le nom des colonnes
	 * @param annotColName tableau de string qui contient le nom des colonnes
	 * @param dir 
	 */
	public parse(String colName[], String annotColName[], String dir)
	{
		try
		{
			IHM.m_dir=dir;
			//initialisation du jeu de donnees
			m_don=new DonneeParse(colName, 0);
			m_don.setColName(colName);
			if (annotColName != null && annotColName.length >1)
			{
				// S'il y a un habillage on recupere en ignorant la premiere colonne
				m_annotColName = new String[annotColName.length - 1];
				for (int i = 0; i < annotColName.length - 2; i++)
				{
					m_annotColName[i] = annotColName[i + 1];
					m_don.addNominale(new Nominale(m_annotColName[i]));
				}
			}
		}
		catch(Exception e)
		{FenetreErreur.display(e);}
	}

	/**
	 * Fixe le type de l'analyse : PCA, MCA, MFA, CA, AFDM, dataframe
	 * AUTRE si le type n'est pas reconnu
	 * @param t
	 */
	public void setType(String t){
		if(t.equals("PCA")){
			m_don.setType(TypeAnalyse.PCA);
		}else if(t.equals("MCA")){
			m_don.setType(TypeAnalyse.MCA);
		}else if(t.equals("MFA")){
			m_don.setType(TypeAnalyse.MFA);
		}else if(t.equals("CA")){
			m_don.setType(TypeAnalyse.CA);
		}else if(t.equals("AFDM")){
			m_don.setType(TypeAnalyse.AFDM);
		}else if(t.equals("dataframe")){
			m_don.setType(TypeAnalyse.DATAFRAME);
		}else{
			m_don.setType(TypeAnalyse.DATAFRAME);
		}
	}



	/** 
	 * ajoute des modalités à un individu
	 * 
	 * @param val tableau des nominales
	 */
	public void addHabVal(String val[])
	{
		try
		{
			//on récupère la donnée
			String annot = val[0];
			//on recherche si c'est un individu
			Individu donnee=m_don.getIndividu(annot);
			if(donnee==null) {
				FenetreErreur.display("impossible de trouver la donnée \""+annot+"\"");
			}else
				for (int i = 0; i < val.length-2; i++)//on ignore la dernière case, dont le contenu est nul pour des soucis de transfert de données
				{
					if(!val[i+1].equals("NA"))
					{
						Modalite mod=m_don.getModalite(val[i+1]);		    
						if(mod!=null)
							donnee.addModalite(mod);
						else 
							FenetreErreur.display("impossible de trouver la modalité : \""+val[i+1]+"\"\n");
					}
					else
						FenetreErreur.display("la case "+(i+1)+" / "+val.length+" du tableau d'annotations est vide");	
				}
		}
		catch(Exception e)
		{FenetreErreur.display(e);}
	}


	/** ajoute une donnée
	 * @param annot
	 * @param val la ligne du tableau
	 * 
	 * Format des tableaux de données à construire en R <br>    
	 * Les résultats des analyses effectuées par les fonctions de FactoMineR doivent 
	 * être traduits sous un format particulier, afin de pouvoir être transmis au module 
	 * Java dynGraphe, qui les interprètera et les mettra en forme. <br>
	 * <b>Données représentées dans les deux tableaux<b> <br> 
	 * <ul><li>Individus actifs (statut 1)</li>
	 * <li>Individus illustratifs (statut 2)</li> 
	 * <li>Variables continues actives orthonormées (statut 3)</li> 
	 * <li>Variables continues illustratives orthonormées (statut 4)</li> 
	 * <li>Variables continues actives non orthonormées (statut 5)</li> 
	 * <li>Variables continues illustratives non orthonormées (statut 6)</li> 
	 * <li>Modalités actives (statut 7)</li> 
	 * <li>Modalités illustratives (statut 8)</li> 
	 * <li>Groupes actifs (statut 9)</li> 
	 * <li>Groupes illustratifs (statut 10)</li> 
	 * <li>Inerties des axes (statut 0)</li></ul>
	 * 
	 * <b>Tableau de valeurs</b>  <br>
	 * Les données sont représentées en lignes. <br>
	 * Les cellules ne sont remplies que si elles sont pertinentes pour la donnée.<br>
	 * Description des colonnes :  <br>
	 * <ul><li>Statut : le code qui décrit le type de la donnée. <br>
	 * Dans le cas des modalités, le code est suivit du numéro de la nominale à laquelle 
	 * appartient la modalité.</li> 
	 * <li>Groupe : le numéro du groupe auquel appartient la donnée.<br>
	 * 0 dans le cas où la donnée n’a pas de groupe </li>
	 * <li>Nombre_axes : le nombre N d’axes de l’analyse (constant sur toute la colonne) </li>
	 * <li>Axe1 .. AxeN : les coordonnées de la donnée sur les axes 1 à N.<br>
	 * Dans le cas des inerties (statut 0), il s’agit des pourcentages d’inertie portés par 
	 * chaque axe </li>
	 * <li>Contrib1 .. ContribN : les contributions de la donnée aux axes 1 à N</li> 
	 * <li>Cos1 .. CosN : les cos² de la donnée pour les axes 1 à N </li>
	 * <li>Nombre_var : le nombre de variables continues qui décrivent la donnée <li>
	 * <li>"Intensité olfactive", "Qualité aromatique au repos", etc. : les valeurs 
	 * des variables (actives ou illustratives) pour la donnée. <br>
	 * Ces valeurs ne sont présentes que pour les individus moyens (actifs ou illustratifs)  </li></ul>
	 * <b>Tableau des annotations</b><br>  
	 * Les données sont représentées en lignes, elles correspondent aux lignes du tableau des 
	 * valeurs 
	 * (c'est-à-dire que si l’individu X est décrit dans la ligne I du tableau des valeurs, 
	 * alors on 
	 * le retrouve dans la ligne I du tableau des annotations). <br>
	 * Les cellules ne sont remplies que si elles sont pertinentes pour la donnée.<br> 
	 * Description des colonnes :  <br>
	 * <ul><li>Annotation : l’étiquette de la donnée, on pourra retrouver plusieurs fois 
	 * la même étiquette si on a des individus partiels, des modalités partielles ou des 
	 * variables partielles </li>
	 * <li>"Appellation", "Milieu", etc. : les modalités de la donnée pour les variables 
	 * nominales (actives ou illustratives)</li></ul> <br> 
	 * <b>Construction des tableaux</b><br>  
	 * Il n’est pas nécessaire de construire le tableau en entier avant de le transmettre 
	 * au module dynGraphe. 
	 * En effet il est possible de transmettre les données ligne par ligne en envoyant 
	 * les lignes du tableau des valeurs, puis celles du tableau des annotations. 
	 * Par contre il est important de transmettre les données dans l’ordre de la partie 
	 * "Données représentées dans les deux tableaux", de plus si une donnée à des 
	 * représentations partielles, celles-ci doivent absolument se trouver juste après elle. 
	 */
	public void addVal(String annot, double[] val)
	{
		try
		{
			if(annot==null)throw new Exception("Annotation nulle");

			//Si on n'a pas encore recupere le nombre d'axes
			if(m_don.getNbAxe()==0)m_don.setNbAxe((int)val[2]);

			//Recuperation et analyse du code de la donnee
			String I = String.valueOf((int) val[0]);
			char Tval[] = I.toCharArray();
			String T = String.valueOf(Tval[0]);
			String T2 = I.substring(1);

			/**
			 * Signification des codes :
			 * 
			 * 0 : Inerties
			 * 1 : Individu actif
			 * 2 : Individu illustratif
			 * 3 : Variable continue active orthonormée
			 * 4 : Variable continue illustrative orthormée
			 * 5 : Variable continue active non orthonormée
			 * 6 : Variable continue illustrative non orthormée
			 * 7 : Modalité active 
			 * 8 : Modalité illustrative 
			 * 9 : Groupe actif
			 * 10 : Groupe illustratif
			 */


			/**
			 * Individu actif
			 */
 		if (val[0] == 1)
		{
			if((int) val[1] ==0)
			{//Individu moyen
				m_individuMoyen_old = new IndividuMoyen(annot,true,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),getZ(val));
				m_don.addIndividuMoyen(m_individuMoyen_old);
			}
			else
			{//individu partiel		
//				m_individuMoyen_old = new IndividuMoyen(null,true,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),getZ(val));
//				m_don.addIndividuMoyen(m_individuMoyen_old);				
				
//				IndividuPartiel donIndiv =new IndividuPartiel(annot,true,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),getZ(val));
//				donIndiv.setPere(m_individuMoyen_old);
//				m_don.addIndividuPartiel(donIndiv);
				
				Individu donIndiv =new Individu(annot,true,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),getZ(val));
				donIndiv.setPere(m_individuMoyen_old);
			}
		}
			/**
			 * Individu illustratif
			 */
			else if (val[0] == 2)
			{
				if((int) val[1] ==0)
				{//Individu moyen
					m_individuMoyen_old = new IndividuMoyen(annot,false,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),getZ(val));
					m_don.addIndividuMoyen(m_individuMoyen_old);
				}
				else
				{//individu partiel		
//					IndividuPartiel donIndiv =new IndividuPartiel(annot+"("+val[2]+")",false,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),getZ(val));
//					donIndiv.setPere(m_individuMoyen_old);
//					m_don.addIndividuPartiel(donIndiv);
					Individu donIndiv =new Individu(annot+"("+val[2]+")",false,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),getZ(val));
					donIndiv.setPere(m_individuMoyen_old);

				}
			} 
			/**
			 * Variable continue active orthonormée
			 * (les variables partielles ne sont pas gérées)
			 */
			else if (val[0] == 3)
			{
				//if(){
				//Continue moyenne
				m_variableMoyenne_old = new ContinueMoyenne(annot,true,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),true);
				m_don.addContinueMoyenne(m_variableMoyenne_old);
				/*}else{
					//Continue partiellele
					Individu donIndiv =new Individu(annot,false,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),0,getZ(val));
					donIndiv.setPere(individuMoyen_old);

				}*/
			} 
			/**
			 * Variable continue illustrative orthormée
			 * (les variables partielles ne sont pas gérées)
			 */
			else if (val[0] == 4)
			{
				//if(){
				//Continue moyenne
				if(m_don.getType()==TypeAnalyse.MCA){
					m_variableMoyenne_old = new ContinueMoyenne(annot,false,m_don.getGroupe((int)val[1]),getCoord(val),null,null,true);
				}else{
					m_variableMoyenne_old = new ContinueMoyenne(annot,false,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),true);
				}
				m_don.addContinueMoyenne(m_variableMoyenne_old);
				/*}else{
				//Continue partielle
				Individu donIndiv =new Individu(annot,false,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),0,getZ(val));
				donIndiv.setPere(individuMoyen_old);

			}*/
			}
			/**
			 * Variable continue active non orthonormée
			 * (les variables partielles ne sont pas gérées)
			 */
			else if (val[0] == 5)
			{
				//if(){
				//Continue moyenne
				m_variableMoyenne_old = new ContinueMoyenne(annot,true,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),false);
				m_don.addContinueMoyenne(m_variableMoyenne_old);
				/*}else{
					//Continue partielle
					Individu donIndiv =new Individu(annot,false,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),0,getZ(val));
					donIndiv.setPere(individuMoyen_old);

				}*/
			} 
			/**
			 * Variable continue illustrative non orthormée
			 * (les variables partielles ne sont pas gérées)
			 */
			else if (val[0] == 6)
			{
				//Continue illustrative non orthormée
				//if(){
				//Continue moyenne
				if(m_don.getType()==TypeAnalyse.MCA){
					m_variableMoyenne_old = new ContinueMoyenne(annot,false,m_don.getGroupe((int)val[1]),getCoord(val),null,null,false);
				}else{
					m_variableMoyenne_old = new ContinueMoyenne(annot,false,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),false);
				}
				m_don.addContinueMoyenne(m_variableMoyenne_old);
				/*}else{
				//Continue partielle
				Individu donIndiv =new Individu(annot,false,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),0,getZ(val));
				donIndiv.setPere(individuMoyen_old);

			}*/
			} 
			/**
			 * Modalité active 
			 */
			else if (T.compareTo("7") == 0)
			{
				Nominale nom=m_don.getNominale(Integer.valueOf(T2)-1);
				if((int)val[1]==0)
				{//modalité moyenne
					m_modaliteMoyenne_old=new ModaliteMoyenne(nom,annot,true,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),0);
					nom.addModaliteMoyenne(m_modaliteMoyenne_old);
				}
				else
				{//modalité partielle
					// traiteModalitePartielle(annot,val,true);
					m_modaliteMoyenne_old.addBranche(new Modalite(nom,annot,true,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),0));
				}
			} 	
			/**
			 * Modalité illustrative 
			 */
			else if (T.compareTo("8") == 0)
			{
				Nominale nom=m_don.getNominale(Integer.valueOf(T2)-1);
				if((int)val[1]==0)
				{//modalité moyenne
					m_modaliteMoyenne_old=new ModaliteMoyenne(nom,annot,false,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),0);
					nom.addModaliteMoyenne(m_modaliteMoyenne_old);
				}
				else
				{//modalité partielle
					//traiteModalitePartielle(annot,val,false);
					m_modaliteMoyenne_old.addBranche(new Modalite(nom,annot,false,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),0));
				}
			} 
			/**
			 * Groupe actif
			 */
			else if (val[0] == 9)
			{
				Groupe g=m_don.getGroupe(num_groupe_courant);
				g.setAnnotation(annot);
				g.setActive(true);
				g.setCoord(getCoord(val));
				g.setContrib(getContrib(val));
				g.setCos2(getCos2(val));
				num_groupe_courant++;
			} 
			/**
			 * Groupe illustratif
			 */
			else if (val[0] == 10)
			{
				Groupe g=m_don.getGroupe(num_groupe_courant);
				g.setAnnotation(annot);
				g.setActive(false);
				g.setCoord(getCoord(val));
				g.setContrib(getContrib(val));
				g.setCos2(getCos2(val));
				num_groupe_courant++;
			} 
			/**
			 * Axes
			 */
			else if (val[0] == 0){
				if((int)val[1]==0){
					/** Inerties */
					m_don.setInerties(getCoord(val));
				}else{
					/** Axes Partiels */
					AxePartiel axePart=new AxePartiel(annot,true,m_don.getGroupe((int)val[1]),getCoord(val),getCos2(val),getContrib(val),true);
					m_don.addAxePartiel(axePart);
				}
			}
			else FenetreErreur.display("donnée illisible");
		}
		catch(Exception e)
		{FenetreErreur.display(e);}
	}


	/** 
	 * génération du graphique, lancement de dynGraph 
	 */
	public void generate()
	{		
		try
		{
			m_don.verifierIntegrite();
			serialiser(IHM.m_dir+"tmp.ser");
			File fichier = new File(IHM.m_dir+"tmp.ser");
			if(fichier.exists()) {
				String commande="java -jar "+IHM.m_dir+"dynGraph.jar "+IHM.m_dir+" "+IHM.m_dir+"tmp.ser -t";
				Runtime.getRuntime().exec(commande);
			}
			else {
				FenetreErreur.display("fichier temporaire introuvable !");
			}
		} 
		catch (Exception e)
		{FenetreErreur.display(e);}
	}

	/** 
	 * serialisation (sauvegarde) du jeu de données dans le fichier <b><i>"chemin"</i></b> directement depuis r
	 * @param chemin 
	 */
	public void serialiser(String chemin)
	{
		try
		{
			Registre reg=new Registre(m_don);
			Fichier.serialiser(reg,chemin);
		}
		catch (Exception e)
		{FenetreErreur.display(e);}
	}


	/** 
	 * serialisation (sauvegarde) du jeu de données dans le fichier <i>"jeudedonnees.ser"</i> directement depuis r 
	 */
	public void serialiser()
	{serialiser(IHM.m_dir+"/jeudedonnees.ser");}

	/** 
	 * Affiche le jeu de données dans une fenêtre 
	 */
	public void fenetre()
	{
		try
		{FenetreErreur.display("contenu:\n\n"+m_don.toString());}
		catch (Exception e)
		{FenetreErreur.display(e);}
	}


	/**
	 * Indique que l'axe dont le numéro est <b>num</b>
	 * est caractérisé par la modalité dont le nom est <b>nom_modalite_qui_caracterise</b>
	 * selon une valeur test <b>vtest</b>
	 * 
	 * @param num >1 
	 * @param nom_modalite_qui_caracterise
	 * @param vtest
	 */
	public void add_description_dim_par_mod(int num, String nom_modalite_qui_caracterise, double vtest){
		try{
			Dimension dim=m_don.getAxe(num-1);		
			Modalite mod_qui_caracterise = m_don.getModalite(nom_modalite_qui_caracterise);
			if(mod_qui_caracterise==null){
				throw new Exception(nom_modalite_qui_caracterise+" inconnue");
			}
			dim.setDescMod(mod_qui_caracterise, vtest);
		}catch(Exception e){
			FenetreErreur.display(e);
		}
	}

	/**
	 * Indique que l'axe dont le numéro est <b>num</b>
	 * est caractérisée par la continue dont le nom est <b>nom_continue_qui_caracterise</b>
	 * selon une valeur test <b>vtest</b>
	 * 
	 * @param num >1 
	 * @param nom_continue_qui_caracterise
	 * @param vtest
	 */
	public void add_description_dim_par_cont(int num, String nom_continue_qui_caracterise, double vtest){
		try{
			Dimension dim=m_don.getAxe(num-1);		
			ContinueAnalysee cont_qui_caracterise = m_don.getContinue(nom_continue_qui_caracterise);
			if(cont_qui_caracterise==null){
				throw new Exception(nom_continue_qui_caracterise+" inconnue");
			}
			dim.setDescCont(cont_qui_caracterise, vtest);
		}catch(Exception e){
			FenetreErreur.display(e);
		}
	}


	/**
	 * Indique que l'axe dont le numéro est <b>num</b>
	 * est caractérisée par la nominale dont le nom est <b>nom_nominale_qui_caracterise</b>
	 * selon une valeur test <b>vtest</b>
	 * 
	 * @param num >1 
	 * @param nom_nominale_qui_caracterise
	 * @param vtest
	 */
	public void add_description_dim_par_nom(int num, String nom_nominale_qui_caracterise, double vtest){
		try{
			Dimension dim=m_don.getAxe(num-1);		
			Nominale nom_qui_caracterise = m_don.getNominale(nom_nominale_qui_caracterise);
			if(nom_qui_caracterise==null){
				throw new Exception(nom_nominale_qui_caracterise+" inconnue");
			}
			dim.setDescNom(nom_qui_caracterise, vtest);
		}catch(Exception e){
			FenetreErreur.display(e);
		}
	}


	/**
	 * Indique que le modalité dont le nom est <b>nom_modalite_a_caracteriser</b>
	 * est caractérisée par la modalité dont le nom est <b>nom_modalite_qui_caracterise</b>
	 * selon une valeur test <b>vtest</b>
	 * 
	 * @param nom_modalite_a_caracteriser
	 * @param nom_modalite_qui_caracterise
	 * @param vtest
	 */
	public void add_description_mod_par_mod(String nom_modalite_a_caracteriser, String nom_modalite_qui_caracterise, double vtest){
		try{
			Modalite mod_a_caracteriser = m_don.getModalite(nom_modalite_a_caracteriser);
			if(mod_a_caracteriser==null){
				throw new Exception(nom_modalite_a_caracteriser+" inconnue");
			}
			Modalite mod_qui_caracterise = m_don.getModalite(nom_modalite_qui_caracterise);
			if(mod_qui_caracterise==null){
				throw new Exception(nom_modalite_qui_caracterise+" inconnue");
			}
			mod_a_caracteriser.setDescMod(mod_qui_caracterise, vtest);
		}catch(Exception e){
			FenetreErreur.display(e);
		}
	}

	/**
	 * Indique que le modalité dont le nom est <b>nom_modalite_a_caracteriser</b>
	 * est caractérisée par la continue dont le nom est <b>nom_continue_qui_caracterise</b>
	 * selon une valeur test <b>vtest</b>
	 * 
	 * @param nom_modalite_a_caracteriser
	 * @param nom_continue_qui_caracterise
	 * @param vtest
	 */
	public void add_description_mod_par_cont(String nom_modalite_a_caracteriser, String nom_continue_qui_caracterise, double vtest){
		try{
			Modalite mod_a_caracteriser = m_don.getModalite(nom_modalite_a_caracteriser);
			if(mod_a_caracteriser==null){
				throw new Exception(nom_modalite_a_caracteriser+" inconnue");
			}
			ContinueAnalysee cont_qui_caracterise = m_don.getContinue(nom_continue_qui_caracterise);
			if(cont_qui_caracterise==null){
				throw new Exception(nom_continue_qui_caracterise+" inconnue");
			}
			mod_a_caracteriser.setDescCont(cont_qui_caracterise, vtest);
		}catch(Exception e){
			FenetreErreur.display(e);
		}
	}

	/**
	 * Indique que le modalité dont le nom est <b>nom_modalite_a_caracteriser</b>
	 * est caractérisée par la nominale dont le nom est <b>nom_nominale_qui_caracterise</b>
	 * selon une valeur test <b>vtest</b>
	 * 
	 * @param nom_modalite_a_caracteriser
	 * @param nom_nominale_qui_caracterise
	 * @param vtest
	 */
	public void add_description_mod_par_nom(String nom_modalite_a_caracteriser, String nom_nominale_qui_caracterise, double vtest){
		try{
			Modalite mod_a_caracteriser = m_don.getModalite(nom_modalite_a_caracteriser);
			if(mod_a_caracteriser==null){
				throw new Exception(nom_modalite_a_caracteriser+" inconnue");
			}
			Nominale nom_qui_caracterise = m_don.getNominale(nom_nominale_qui_caracterise);
			if(nom_qui_caracterise==null){
				throw new Exception(nom_nominale_qui_caracterise+" inconnue");
			}
			mod_a_caracteriser.setDescNom(nom_qui_caracterise, vtest);
		}catch(Exception e){
			FenetreErreur.display(e);
		}
	}
	
	
	
	
	/**
	 * Indique que le continue dont le nom est <b>nom_continue_a_caracteriser</b>
	 * est caractérisée par la modalité dont le nom est <b>nom_modalite_qui_caracterise</b>
	 * selon une valeur test <b>vtest</b>
	 * 
	 * @param nom_continue_a_caracteriser
	 * @param nom_modalite_qui_caracterise
	 * @param vtest
	 */
	public void add_description_cont_par_mod(String nom_continue_a_caracteriser, String nom_modalite_qui_caracterise, double vtest){
		try{
			ContinueAnalysee cont_a_caracteriser = m_don.getContinue(nom_continue_a_caracteriser);
			Modalite mod_qui_caracterise = m_don.getModalite(nom_modalite_qui_caracterise);
			if(cont_a_caracteriser==null){
				throw new Exception(nom_continue_a_caracteriser+" inconnue");
			}
			if(mod_qui_caracterise==null){
				throw new Exception(nom_modalite_qui_caracterise+" inconnue");
			}
			cont_a_caracteriser.setDescMod(mod_qui_caracterise, vtest);
		}catch(Exception e){
			FenetreErreur.display(e);
		}
	}

	/**
	 * Indique que le continue dont le nom est <b>nom_continue_a_caracteriser</b>
	 * est caractérisée par la continue dont le nom est <b>nom_continue_qui_caracterise</b>
	 * selon une valeur test <b>vtest</b>
	 * 
	 * @param nom_continue_a_caracteriser
	 * @param nom_continue_qui_caracterise
	 * @param vtest
	 */
	public void add_description_cont_par_cont(String nom_continue_a_caracteriser, String nom_continue_qui_caracterise, double vtest){
		try{
			ContinueAnalysee cont_a_caracteriser = m_don.getContinue(nom_continue_a_caracteriser);
			ContinueAnalysee cont_qui_caracterise = m_don.getContinue(nom_continue_qui_caracterise);
			if(cont_a_caracteriser==null){
				throw new Exception(nom_continue_a_caracteriser+" inconnue");
			}
			if(cont_qui_caracterise==null){
				throw new Exception(nom_continue_qui_caracterise+" inconnue");
			}
			cont_a_caracteriser.setDescCont(cont_qui_caracterise, vtest);
		}catch(Exception e){
			FenetreErreur.display(e);
		}
	}

	/**
	 * Indique que le continue dont le nom est <b>nom_continue_a_caracteriser</b>
	 * est caractérisée par la nominale dont le nom est <b>nom_nominale_qui_caracterise</b>
	 * selon une valeur test <b>vtest</b>
	 * 
	 * @param nom_continue_a_caracteriser
	 * @param nom_nominale_qui_caracterise
	 * @param vtest
	 */
	public void add_description_cont_par_nom(String nom_continue_a_caracteriser, String nom_nominale_qui_caracterise, double vtest){
		try{
			ContinueAnalysee cont_a_caracteriser = m_don.getContinue(nom_continue_a_caracteriser);
			Nominale nom_qui_caracterise = m_don.getNominale(nom_nominale_qui_caracterise);
			if(cont_a_caracteriser==null){
				throw new Exception(nom_continue_a_caracteriser+" inconnue");
			}
			cont_a_caracteriser.setDescNom(nom_qui_caracterise, vtest);
			if(nom_qui_caracterise==null){
				throw new Exception(nom_nominale_qui_caracterise+" inconnue");
			}
		}catch(Exception e){
			FenetreErreur.display(e);
		}
	}
	
	
	
	/**
	 * Indique que la nominale dont le nom est <b>nom_nominale_a_caracteriser</b>
	 * est caractérisée par la modalité dont le nom est <b>nom_modalite_qui_caracterise</b>
	 * selon une valeur test <b>vtest</b>
	 * 
	 * @param nom_nominale_a_caracteriser
	 * @param nom_modalite_qui_caracterise
	 * @param vtest
	 */
	public void add_description_nom_par_mod(String nom_nominale_a_caracteriser, String nom_modalite_qui_caracterise, double vtest){
		try{
			Nominale nom_a_caracteriser = m_don.getNominale(nom_nominale_a_caracteriser);
			Modalite mod_qui_caracterise = m_don.getModalite(nom_modalite_qui_caracterise);
			if(nom_a_caracteriser==null){
				throw new Exception(nom_nominale_a_caracteriser+" inconnue");
			}
			if(mod_qui_caracterise==null){
				throw new Exception(nom_modalite_qui_caracterise+" inconnue");
			}
			nom_a_caracteriser.setDescMod(mod_qui_caracterise, vtest);
		}catch(Exception e){
			FenetreErreur.display(e);
		}
	}

	/**
	 * Indique que la nominale dont le nom est <b>nom_nominale_a_caracteriser</b>
	 * est caractérisée par la continue dont le nom est <b>nom_continue_qui_caracterise</b>
	 * selon une valeur test <b>vtest</b>
	 * 
	 * @param nom_nominale_a_caracteriser
	 * @param nom_continue_qui_caracterise
	 * @param vtest
	 */
	public void add_description_nom_par_cont(String nom_nominale_a_caracteriser, String nom_continue_qui_caracterise, double vtest){
		try{
			Nominale nom_a_caracteriser = m_don.getNominale(nom_nominale_a_caracteriser);
			ContinueAnalysee cont_qui_caracterise = m_don.getContinue(nom_continue_qui_caracterise);
			if(nom_a_caracteriser==null){
				throw new Exception(nom_nominale_a_caracteriser+" inconnue");
			}
			if(cont_qui_caracterise==null){
				throw new Exception(nom_continue_qui_caracterise+" inconnue");
			}
			nom_a_caracteriser.setDescCont(cont_qui_caracterise, vtest);
		}catch(Exception e){
			FenetreErreur.display(e);
		}
	}

	/**
	 * Indique que la nominale dont le nom est <b>nom_nominale_a_caracteriser</b>
	 * est caractérisée par la nominale dont le nom est <b>nom_nominale_qui_caracterise</b>
	 * selon une valeur test <b>vtest</b>
	 * 
	 * @param nom_nominale_a_caracteriser
	 * @param nom_nominale_qui_caracterise
	 * @param vtest
	 */
	public void add_description_nom_par_nom(String nom_nominale_a_caracteriser, String nom_nominale_qui_caracterise, double vtest){
		try{
			Nominale nom_a_caracteriser = m_don.getNominale(nom_nominale_a_caracteriser);
			if(nom_a_caracteriser==null){
				throw new Exception(nom_nominale_a_caracteriser+" inconnue");
			}
			Nominale nom_qui_caracterise = m_don.getNominale(nom_nominale_qui_caracterise);
			if(nom_qui_caracterise==null){
				throw new Exception(nom_nominale_qui_caracterise+" inconnue");
			}
			nom_a_caracteriser.setDescNom(nom_qui_caracterise, vtest);
		}catch(Exception e){
			FenetreErreur.display(e);
		}
	}

	
	
	


//	/** 
//	 * main
//	 * @param args
//	 */
//	public static void main(String[] args)
//	{
//		parse p=new parse(null,null);
//		Registre reg=Fichier.chargerDonnees("./test/res.ca.ser");
//		p.m_don= reg.getDonnee();
//		IHM.m_dir="";
//		p.generate();
//	}

	/** Rend les coordonnées depuis une ligne de tableau
	 * @param val
	 * @return un tableau de double qui contient les coordonnées
	 */
	private double[] getCoord(double[] val)
	{
		double[] tab= new double[(int)val[2]];
		int decalage=3;
		for(int i=0 ; i<tab.length;i++)
			tab[i]=val[decalage+i];
		return tab;	
	}

	/** Rend les cosé depuis une ligne de tableau
	 * @param val
	 * @return un tableau de double qui contient les cos2
	 */
	private double[] getCos2(double[] val)
	{
		double[] tab= new double[(int)val[2]];
		int decalage=3+(int)val[2]*2;
		for(int i=0 ; i<tab.length;i++)
			tab[i]=val[decalage+i];
		return tab;	
	}

	/** rend les contributions depuis une ligne de tableau
	 * @param val
	 * @return un tableau de double qui contient les contributions
	 */
	private double[] getContrib(double[] val)
	{
		double[] tab= new double[(int)val[2]];
		int decalage=3+(int)val[2];
		for(int i=0 ; i<tab.length;i++)
			tab[i]=val[decalage+i];
		return tab;	
	}

	/** 
	 * rend les coordonnées dans l'espace de départ depuis une ligne de tableau
	 * @param val
	 * @return un tableau de double qui contient les continue
	 */
	private double[] getZ(double[] val)
	{
		int nbz=(int)val[3+(int)val[2]*3];
		double[] tab= new double[nbz];
		int decalage=3+(int)val[2]*3+1;
		for(int i=0 ; i<tab.length;i++)
			tab[i]=val[decalage+i];
		return tab;
	}

}